#include "AnalysisAlgorithmMap.h"
#include "AnalysisAlgorithm.h"
#include "CalibrationController/IS/TestData.h"
#include <sstream>

using namespace std;

namespace SctAnalysis {

typedef map<string, shared_ptr<AnalysisAlgorithm> > MapType;
    
AnalysisAlgorithmMap& AnalysisAlgorithmMap::instance() {
    static AnalysisAlgorithmMap* instance = new AnalysisAlgorithmMap();
    return *instance;
}
    
shared_ptr <AnalysisAlgorithm> AnalysisAlgorithmMap::getAlgorithm(shared_ptr<const TestData> testData, const string& moduleName) const throw(Sct::InvalidArgument) {
  if (!testData.get()) throw Sct::InvalidArgumentError("Attempt to use null TestData",__FILE__,__LINE__);
    string nameToUse = testData->testName;
    if (nameToUse=="NONE") {
      shared_ptr<AnalysisAlgorithm> alg;
      return alg;
    }

    if (testData->analysisAlgorithm!=""){
      nameToUse=testData->analysisAlgorithm;
    }

    if (algMap.count(nameToUse) == 0) {
	Sct::InvalidArgumentError iae("No map for Test: " + nameToUse, __FILE__, __LINE__);
	iae.setSeverity(MRS_DIAGNOSTIC);
	throw iae;
    }
    
    return (*algMap.find(nameToUse)).second->clone(testData, moduleName);
}
    
bool AnalysisAlgorithmMap::setAlgorithm(const string testName, auto_ptr<AnalysisAlgorithm> alg) throw() {
    algMap[testName] = shared_ptr<AnalysisAlgorithm>(alg);
    return true;    
}

string AnalysisAlgorithmMap::getStatus(string testName) {
    if (algMap.count(testName) == 0) {
	return string();
    }
    return algMap[testName]->getStatus();
}
    
string AnalysisAlgorithmMap::getAllStatus() {
    ostringstream status;
    for (MapType::const_iterator i=algMap.begin(); i!=algMap.end(); ++i) {
	status << i->first << " Algorithm " << i->second->getStatus() << "\n";
    }
    return status.str();
}

std::list<std::string> AnalysisAlgorithmMap::listAlgorithms(){
  std::list<std::string> alg_list;
  for (MapType::const_iterator i=algMap.begin(); i!=algMap.end(); ++i) {
    alg_list.push_back(i->first);
  }
  return alg_list;
}
    
}
