#include "../IOManagerDB.h"
#include "Sct/Archive/IOManagerArchiveFile.h"
#include <cmdl/cmdargs.h>
#include <cmdl/cmdline.h>

int main(int argc, char** argv){
  
  CmdArgBool  to_db ('i', "input", "flag to show we want to input files to db");
  CmdArgStrList names('n', "names", "", "list of names of objects to be transferred");
  CmdArgBool verbose('v', "verbose", "", "print status information at end");


  CmdLine cmd(*argv, &to_db, &names, &verbose, NULL);
  CmdArgvIter arg_iter(--argc, ++argv);
  cmd.description("Test program to read and write data from database");
  cmd.parse(arg_iter);
   
  SctArchiving::IOManagerDB& db_man = SctArchiving::IOManagerDB::instance();
  Sct::Archive::IOManagerArchiveFile& file_man = Sct::Archive::IOManagerArchiveFile::instance();
  
  // Jorgen: needs to set the DataSet name
  db_man.setDataSetName("OxfordSouth");

  for (int i = 0 ; i < names.count() ; i++) {
    try{
      if (to_db) {
	boost::shared_ptr<Sct::Serializable> object = file_man.read((std::string)names[i]);
	db_man.write(*object);
      }else{
	boost::shared_ptr<Sct::Serializable> object = db_man.read((std::string) names[i]);
	file_man.write(*object);
      }
    }catch(Sct::Throwable& e){
      e.sendToMrs();
      std::cerr << e.what() << std::endl;
    }
  }
  
  if (verbose){
    std::cout << "Database Manager -------------------" << std::endl;
    std::cout << db_man.status() << std::endl;

    std::cout << "File Manager -------------------" << std::endl;
    std::cout << file_man.status() << std::endl;
  }
  
}
