#include "ScanLibraryImpl.h"
#include "CalibrationControllerImpl.h"
#include "scripts/ThresholdScan.h"
#include "scripts/StrobeDelayScan.h"
#include "scripts/NMaskScan.h"
#include "scripts/NoiseOccupancyScan.h"
#include "scripts/TokenScan.h"
#include "scripts/DefaultScan.h"
#include "scripts/RawScan.h"
#include "Sct/BugReport.h"

namespace SctCalibrationController {
using namespace Scripts;    
    
ScanLibraryImpl::ScanLibraryImpl() {
    api = Sct_SctApi::SctApiIPC::_nil();
}
    
Sct_CalibrationController::ScanLibrary_ptr ScanLibraryImpl::instance() {
    static ScanLibraryImpl* instance = new ScanLibraryImpl();
    return instance->_this();
}

Sct_CalibrationController::ScanRequest_ptr ScanLibraryImpl::rawScan(unsigned long nTrigs, CORBA::Boolean issueSoftReset) {
    return RawScan::rawScan(nTrigs, issueSoftReset)._this();
}

Sct_CalibrationController::ScanRequest_ptr ScanLibraryImpl::thresholdScan(unsigned long nTrigs) {
    return ThresholdScan::thresholdScan(nTrigs)._this();
}

Sct_CalibrationController::ScanRequest_ptr ScanLibraryImpl::thresholdScanfC(unsigned long nTrigs, double calCharge) {
    return ThresholdScan::thresholdScan(nTrigs, calCharge)._this();    
}

Sct_CalibrationController::ScanRequest_ptr ScanLibraryImpl::strobeDelayScan(unsigned long nTrigs) {
  Sct::BugReport("BUG01", "ScanLibraryImpl.cpp","strobeDelayScan 1/2");
  Sct_CalibrationController::ScanRequest_ptr ans = StrobeDelayScan::strobeDelayScan(nTrigs)._this();
  Sct::BugReport("BUG01", "ScanLibraryImpl.cpp","strobeDelayScan 2/2");
  return ans;
}

Sct_CalibrationController::ScanRequest_ptr ScanLibraryImpl::thresholdScanfCOffset(unsigned long nTrigs, double calCharge, double offset) {
    return ThresholdScan::thresholdScan(nTrigs, calCharge, offset)._this();    
}

Sct_CalibrationController::ScanRequest_ptr ScanLibraryImpl::noiseOccupancyScan() {
    return NoiseOccupancyScan::noiseOccupancyScan()._this();
}


Sct_CalibrationController::ScanRequest_ptr ScanLibraryImpl::nmaskScan(unsigned long nTrigs) {
    return NMaskScan::nmaskScan(nTrigs)._this();
}

Sct_CalibrationController::ScanRequest_ptr ScanLibraryImpl::tokenScan() {
    return TokenScan::tokenScan()._this();
}

Sct_CalibrationController::ScanRequest_ptr ScanLibraryImpl::defaultScan() {
    return DefaultScan::defaultScan()._this();
}

}
