#ifndef SCTCALIBRATIONCONTROLLER_SCANREQUEST_H
#define SCTCALIBRATIONCONTROLLER_SCANREQUEST_H

#include "CalibrationController.hh"
#include <ipc/object.h>
#include <memory>
#include <string>
#include <vector>
#include "boost/shared_ptr.hpp"

using std::vector;
using std::auto_ptr;
using std::string;

namespace SctCalibrationController {
namespace Ipc {
    
class CalibrationController;
    
/**
  See the documentation in CalibrationController.idl for these methods.
  All may throw RequestException.  This is the nice interface
  */

class ScanRequest {
public:	    
    virtual CORBA::Boolean isRawNice()=0;
    virtual CORBA::Boolean configureModulesNice()=0;
    virtual long delayNice()=0;
    virtual void setScanNice(Sct_SctApi::Scan_ptr s)=0;
    virtual CORBA::Boolean clockByTwoNice()=0;
    virtual void setClockByTwoNice(CORBA::Boolean value)=0;
    virtual void setConfigureModulesNice(CORBA::Boolean value)=0;
    virtual long widthNice()=0;
    virtual void setWidthNice(long value)=0;
    virtual Sct_SctApi::Scan_ptr getScanNice()=0;
};
 
 
 class MinimalScanRequest : public virtual ScanRequest {
 protected:
   MinimalScanRequest();
   virtual ~MinimalScanRequest() {};
 public:
   virtual CORBA::Boolean isRawNice();
   virtual CORBA::Boolean configureModulesNice();
   virtual CORBA::Boolean clockByTwoNice();
   virtual void setClockByTwoNice(CORBA::Boolean value);
    virtual void setConfigureModulesNice(CORBA::Boolean value);
   virtual CORBA::Long widthNice();
   virtual void setWidthNice(CORBA::Long width);
   virtual CORBA::Long delayNice();
   virtual void setScanNice(Sct_SctApi::Scan_ptr s);
   virtual Sct_SctApi::Scan_ptr getScanNice();
 protected:
   CORBA::Boolean m_clockByTwo;
   CORBA::Long m_width;
   Sct_SctApi::Scan_var m_scan;  
   CORBA::Boolean m_configureModules;
 };


/**
  This is the internal implementation, to be derived from;
  */
class ScanRequestImpl : 
  public IPCObject<POA_Sct_CalibrationController::ScanRequest>,
  protected MinimalScanRequest {
public:
     ScanRequestImpl();

    virtual CORBA::Boolean isRawCorba();
    virtual CORBA::Boolean configureModulesCorba();
    virtual void setConfigureModulesCorba(CORBA::Boolean value);
    virtual CORBA::Boolean clockByTwoCorba();
    virtual CORBA::Long delayCorba();
    virtual Sct_SctApi::Scan_ptr getScanCorba ();
    virtual void setScanCorba (Sct_SctApi::Scan_ptr scan);
    virtual void setClockByTwoCorba (CORBA::Boolean value);
    virtual CORBA::Long widthCorba();
    virtual void setWidthCorba(CORBA::Long width);
};

}
}
#endif //SCTCALIBRATIONCONTROLLER_SCANREQUEST_H
