#include "TestRequest.h"
#include "CalibrationController.h"
#include "RequestException.h"
#include "Sct/OmniMarshalling.h"
#include "Sct/SequenceUtils.h"
#include "Sct/BugReport.h"

namespace SctCalibrationController {
namespace Ipc {
    
TestRequestImpl::TestRequestImpl() : IPCObject<POA_Sct_CalibrationController::TestRequest>() {
     Sct::BugReport("BUG01", "TestRequest.cpp","ImplConstructor 1");
}

  void TestRequestImpl::startTest (/*Sct_CalibrationControllerStatus *_status, */
				   Sct_SctApi::SctApiIPC_ptr api, //in
				   CORBA::String_out name, //out
				   CORBA::UShort & testVariable, //out
				   CORBA::ULong & nScans, //out
				   Sct_CalibrationController::TestPoints_out testPointsData //out
                                   ){
     Sct::BugReport("BUG01", "TestRequest.cpp","startTest 2");
     
     std::string nameOut;
     unsigned short testVariableOut;
     unsigned long nScansOut;
     vector<double> testPointsOut;

     startTestRealImpl(api, nameOut, testVariableOut, nScansOut, testPointsOut);
     
     name = ::Sct::copyStringToCorba(nameOut);
     testVariable = testVariableOut;
     nScans = nScansOut;
     testPointsData = ::Sct::SequenceUtils::newSafeSequence<Sct_CalibrationController::TestPoints>(testPointsOut.size());
     for (unsigned int i=0; i<testPointsOut.size(); ++i) {
       testPointsData[i]=testPointsOut[i];
     };
     Sct::BugReport("BUG01", "TestRequest.cpp","startTest 2.5");
}

  Sct_CalibrationController::ScanRequest_ptr TestRequestImpl::getNextScan (/*Sct_CalibrationControllerStatus *_status, */CORBA::ULong index){
     Sct::BugReport("BUG01", "TestRequest.cpp","getNextScan 3");
     Sct_CalibrationController::ScanRequest_ptr ans = getNextScanRealImpl(index);
     Sct::BugReport("BUG01", "TestRequest.cpp","getNextScan 4");
     return ans;
}

  CORBA::Boolean TestRequestImpl::canFeedback (/*Sct_CalibrationControllerStatus *_status*/) {
     Sct::BugReport("BUG01", "TestRequest.cpp","canFeedback 4");
    return canFeedbackRealImpl();
}

void TestRequestImpl::setAnalysisAlgorithm(const char* name){
  m_analysisAlgorithm=name;
}

char* TestRequestImpl::getAnalysisAlgorithm(){
  return ::Sct::copyStringToCorba(m_analysisAlgorithm);
}

void TestRequestImpl::setFitAlgorithm(const char* name){
  m_fitAlgorithm=name;
}

char* TestRequestImpl::getFitAlgorithm(){
  return ::Sct::copyStringToCorba(m_fitAlgorithm);
}


  void TestRequestImpl::endTest (/*Sct_CalibrationControllerStatus *_status*/) {
     Sct::BugReport("BUG01", "TestRequest.cpp","endTest 5");
     endTestRealImpl();
     Sct::BugReport("BUG01", "TestRequest.cpp","endTest 5.5");
}


  /*
  TestRequestAdapter::TestRequestAdapter(Sct_CalibrationController::TestRequest_ptr req) : 
    request(Sct_CalibrationController::TestRequest::_duplicate(req)) {
     Sct::BugReport("BUG01", "TestRequest.cpp","AdapterConstructor 6");
}


void TestRequestAdapter::startTest(Sct_SctApi::SctApiIPC_ptr api,
				   std::string& name,
				   unsigned short & testVariable, 
				   unsigned long & nScans,
				   vector<double>& testPoints) {
  //Sct_CalibrationControllerStatus status;
  Sct::BugReport("BUG01", "TestRequest.cpp","startTest 7");
  CORBA::String_var text;
  Sct::BugReport("BUG01", "TestRequest.cpp","startTest 7.4");
  //This line was just a test name = text; // delete this one!
  Sct::BugReport("BUG01", "TestRequest.cpp","startTest 7.6");
    Sct_CalibrationController::TestPoints_var data;
        Sct::BugReport("BUG01", "TestRequest.cpp","startTest 8");
 
    try {
     Sct::BugReport("BUG01", "TestRequest.cpp","startTest 9");
      request->startTest(//&status, 
			 api, text, testVariable, nScans, data);
     Sct::BugReport("BUG01", "TestRequest.cpp","startTest 10");
    } catch (CORBA::Exception & status) {
     Sct::BugReport("BUG01", "TestRequest.cpp","startTest 11");
	throw RequestException(status, "TestRequestAdapter::startTest failed", __FILE__, __LINE__);
    };
     Sct::BugReport("BUG01", "TestRequest.cpp","startTest 12");
    name = text;
     Sct::BugReport("BUG01", "TestRequest.cpp","startTest 13");
    for (unsigned int i = 0; i<data->length(); ++i) {
     Sct::BugReport("BUG01", "TestRequest.cpp","startTest 14");
      testPoints.push_back(data[i]);
    }
     Sct::BugReport("BUG01", "TestRequest.cpp","startTest 15");
}
  
  Sct_CalibrationController::ScanRequest_ptr TestRequestAdapter::getNextScan(unsigned long index) {
     Sct::BugReport("BUG01", "TestRequest.cpp","getNextScan 16");
    Sct_CalibrationController::ScanRequest_ptr scan;
    try {
scan = request->getNextScan(//&status,
index);
    } catch (const CORBA::Exception & status) {
      throw RequestException(status, "TestRequestAdapter::getNextScan failed", __FILE__, __LINE__);    
    };
    if (CORBA::is_nil(scan)) {
     Sct::BugReport("BUG01", "TestRequest.cpp","getNextScan 17");
      return Sct_CalibrationController::ScanRequest::_nil();
    };
    
    //ScanRequest* ptr=dynamic_cast<ScanRequest*>(scan);
    //if (ptr!=0) return ptr;
    //else
    //ScanRequestAdapter * newAdapter = new ScanRequestAdapter(scan);
    //#warning DONT FORGET TO MAKE ScanRequestAdapter INHERIT FROM
     Sct::BugReport("BUG01", "TestRequest.cpp","getNextScan 18");
    return scan;
  }
  
  bool TestRequestAdapter::canFeedback() {
     Sct::BugReport("BUG01", "TestRequest.cpp","canFeedback 19");
    try {
      return request->canFeedback();
    } catch (const CORBA::Exception & status) {
      throw RequestException(status, "TestRequestAdapter::canFeedback failed", __FILE__, __LINE__);    
    };
  }
  
  void TestRequestAdapter::endTest() {
     Sct::BugReport("BUG01", "TestRequest.cpp","endTest 20");
    try {
      request->endTest();    
    } catch (const CORBA::Exception & status) {
      throw RequestException(status, "TestRequestAdapter::endTest failed", __FILE__, __LINE__);    
    };
  }
*/  
  
}
}
