#ifndef SCTCALIBRATIONCONTROLLER_CHIPCOUNTERTEST_H
#define SCTCALIBRATIONCONTROLLER_CHIPCOUNTERTEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/CalibrationController.hh"

#include "../SctApiAccessException.h"
#include "../SctApiCall.h"
#include "../CalibrationControllerImpl.h"
#include "Sct/StdExceptionWrapper.h"
#include "Sct/LogicErrors.h"
#include "SctApi/extraScans.h"

#include "sctConfIPC/configipc.h"

#include <Sct/AbcdScans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {
    
class ChipCounterTest : public TestRequestImpl {
private:
   static const bool registeredInTestLibrary;

public:
   static Sct_CalibrationController::TestRequest_ptr instance() {
	static ChipCounterTest rt;
	rt.setFitAlgorithm("NONE");
	rt.setAnalysisAlgorithm("ChipCounterTest");
	return rt._this();
    }
   
   virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr the_api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
     this->api = Sct_SctApi::SctApiIPC::_duplicate(the_api);
     // No masked channels
     name = "ChipCounterTest";
     testVariable = ST_ELAPSED;
     nScans = 1;
     testPoints.push_back(nTriggers);
     softResetAllModules();
   }
   
   virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
     Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();

     // dont reset between scan points - only at start of test!
     const bool issueSoftReset = false;
     Sct_CalibrationController::ScanRequest_ptr request = sl->rawScan(nTriggers, issueSoftReset);
     Sct_SctApi::Scan_var scan = request->getScanCorba();
       // Number of bytes to read from the conf register: There should be about 210 ish bytes.  We will read a few more just so we have some elbow room.  Rather arbitrarily, then we chose to read 256 bits:
     const int bitsToRead = 64;
     request->setWidthCorba(bitsToRead);
     
     // This next two executable lines (which disables data
     // mode, and thus enables SendConfRegister mode) 
     // is the second line which (beyond the test names!) 
     // should be the source of difference between THIS
     // test and the usual RxThresholdTest:
     APICALL(scan, setOption(Sct_SctApi::Scan::enableDataMode, 1), "Failed to disable dataMode (failed to enable SendConfRegIDMode)")
       
     APICALL(scan, configure(ST_ELAPSED, 0, 127, 1), "Failed to set Scan information")
     return request;
   }
    
    virtual bool canFeedbackRealImpl() {
	return false;
    }
    
    virtual void endTestRealImpl() {
      api = Sct_SctApi::SctApiIPC::_nil();
    }

    virtual void softResetAllModules(){
      using std::list;
      try{
	boost::shared_ptr<SctConfiguration::Configuration> config(new SctConfiguration::ConfigIPC);
	
	Sct_SctApi::Trigger_var trig;    
	APIRETCALL(api, trig, createTrigger(), "ChipCounterTest: Couldn't create trigger")
	APICALL(trig, soft(), "Failed to set trigger type")
        APICALL(trig, setSource(::Sct_SctApi::Trigger::SOURCE_ROD), "Failed to set trigger source")

	//send to all modules
	list<unsigned> partitions=config->listPartitions();
	for (list<unsigned>::const_iterator p=partitions.begin(); p!=partitions.end(); ++p){
	  list<unsigned> crates=config->listCratesInPartition(*p);
	  for (list<unsigned>::const_iterator c=crates.begin(); c!=crates.end(); ++c){
	    std::cout << "ChipCounterTest : sendTriggers ("<<*p<<", "<<*c<<",soft)"<<std::endl;
	    APICALL(api, sendTriggers(*p, *c, trig, 1, 0), "ChipCounterTest Could not send trigger");
	  }
        }
       }catch (std::exception& e){
	 throw Sct::StdExceptionWrapper(e, __FILE__, __LINE__);
       }
    }  
 private:
    ChipCounterTest() {}
    Sct_SctApi::SctApiIPC_var api;    
    static const int nTriggers = 1;
};
}
}
#endif //SCTCALIBRATIONCONTROLLER_CHIPCOUNTERTEST_H
