#ifndef FIVEPOINTGAIN_H
#define FIVEPOINTGAIN_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/CalibrationController.hh"
#include "../CalibrationControllerImpl.h"

#include "../SctApiAccessException.h"
#include "Sct/LogicErrors.h"
#include "../SctApiCall.h"

#include <Sct/AbcdScans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {
    
class FivePointGainTest : public TestRequestImpl {
private:
   static const bool registeredInTestLibrary;

public:	
    static Sct_CalibrationController::TestRequest_ptr instance() {
	static FivePointGainTest rc;
	rc.setFitAlgorithm("Threshold");
	rc.setAnalysisAlgorithm("NPtGainTest");
	return rc._this();
    }
    
    virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr api,
                           string& name,
                           unsigned short& testVariable,
                           unsigned long& nScans,
                           vector<double>& testPoints) {	
	this->api = Sct_SctApi::SctApiIPC::_duplicate(api);

        // Compression mode X1X
        APICALL(api, modifyABCDVar(ST_COMPRESSION, 1), "FivePointGainTest::startTest")

        // Edge detect off
        APICALL(api, modifyABCDVar(ST_EDGE_DETECT, 0), "FivePointGainTest::startTest")

        // No masked channels
        APICALL(api, modifyABCDVar(ST_SEND_MASK, 0), "FivePointGainTest::startTest")

	name = "NPtGainTest";
	testVariable = ST_QCAL;

	charges.push_back(0.75);
	charges.push_back(1.0);
	charges.push_back(1.5);
	charges.push_back(2.0);
	charges.push_back(4.0);
	testPoints = charges;
	nScans = testPoints.size();
    }

    virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
	Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();

	APICALL(api, modifyABCDVar(ST_QCAL, charges[index]), "FivePointGainTest::getNextScan")

	return sl->thresholdScanfC(500, charges[index]);
    }

    virtual bool canFeedbackRealImpl() {
	return true;
    }

    virtual void endTestRealImpl() {
	api = Sct_SctApi::SctApiIPC::_nil();
	charges.clear();
    }


private:
    FivePointGainTest() {}
    Sct_SctApi::SctApiIPC_var api;
    vector<double> charges;
};
}
}
#endif //FIVEPOINTGAIN_H
