#ifndef NOISEOCCUPANCY_H
#define NOISEOCCUPANCY_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/CalibrationController.hh"
#include "../CalibrationControllerImpl.h"

#include "../SctApiAccessException.h"
#include "Sct/LogicErrors.h"
#include "../SctApiCall.h"

#include "NoiseOccupancyScan.h"

#include <Sct/AbcdScans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {

  /** Noise occupancy Test */
    
class NoiseOccupancyTest : public TestRequestImpl {
private:
   static const bool registeredInTestLibrary;

public:	
    static Sct_CalibrationController::TestRequest_ptr instance() {
	static NoiseOccupancyTest no;
	no.setFitAlgorithm("Threshold");
	no.setAnalysisAlgorithm("NoiseOccupancyTest");
	return no._this();
    }
    
    virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr api,
			   string& name, 
			   unsigned short& testVariable, 
			   unsigned long& nScans, 
			   vector<double>& testPoints) {	
	this->api = Sct_SctApi::SctApiIPC::_duplicate(api);

        // Compression mode X1X
        APICALL(api, modifyABCDVar(ST_COMPRESSION, 1), "NoiseOccupancyTest::startTest")

        // Edge detect off
        APICALL(api, modifyABCDVar(ST_EDGE_DETECT, 0), "NoiseOccupancyTest::startTest")

        // No masked channels
        APICALL(api, modifyABCDVar(ST_SEND_MASK, 0), "NoiseOccupancyTest::startTest")

	name = "NoiseOccupancyTest";
	testVariable = ST_ELAPSED;
	nScans = 1;
	testPoints.push_back(0.);
    }
    
    virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
	Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();
       
       // AJB removed the line below 7/4/2005 as it is unnecessary - the scan should have NO CAL PULSE!!!
       // APICALL(api, modifyABCDVar(ST_VCAL, 0.), "NoiseOccupancyTest::getNextScan")

	return sl->noiseOccupancyScan();
    }
    
    virtual bool canFeedbackRealImpl() {
	return false;
    }
    
    virtual void endTestRealImpl() {
	api = Sct_SctApi::SctApiIPC::_nil();
    }
    
    
private:
    NoiseOccupancyTest() {}
    Sct_SctApi::SctApiIPC_var api;    
};
}
}
#endif //RESPONSECURVE_H
