#ifndef QUICKTRIMTEST_H
#define QUICKTRIMTEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/CalibrationController.hh"
#include "../CalibrationControllerImpl.h"

#include "../SctApiAccessException.h"
#include "Sct/LogicErrors.h"
#include "../SctApiCall.h"

#include <Sct/AbcdScans.h>
#include <iostream>
#include <vector>

using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;
using namespace Sct;

namespace SctCalibrationController {
namespace Scripts {

  /** Trim Range Test */
    
class QuickTrimTest : public TestRequestImpl {
private:
   static const bool registeredInTestLibrary;

public:	
    static Sct_CalibrationController::TestRequest_ptr instance() {
	static QuickTrimTest tr;
	tr.setFitAlgorithm("Threshold");
	tr.setAnalysisAlgorithm("TrimRangeTest");
	return tr._this();
    }
    
    virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr inapi, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = Sct_SctApi::SctApiIPC::_duplicate(inapi);

        // Compression mode X1X
        APICALL(api, modifyABCDVar(ST_COMPRESSION, 1), "QuickTrimTest::startTest can't set compression")

        // Edge detect off
        APICALL(api, modifyABCDVar(ST_EDGE_DETECT, 0), "QuickTrimTest::startTest can't set edge detect")

        // No masked channels
        APICALL(api, modifyABCDVar(ST_SEND_MASK, 0), "QuickTrimTest::startTest can't set send mask")

        // QCAL (fC)
        APICALL(api, modifyABCDVar(ST_QCAL, 1), "QuickTrimTest::startTest can't set qcal")

	name = "TrimRangeTest";
	testVariable=ST_TRIM_RANGE;
	for (unsigned int i=0; i<13; ++i) {
	    testPoints.push_back(getValue(i));
	}
	nScans = testPoints.size();
    }

    virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
      if (isFirstPoint(index)) {
	// set trim range
	  APICALL(api, modifyABCDVar(ST_TRIM_RANGE, getRange(index)), "QuickTrimTest::getNextScan")
	  APICALL(api, modifyABCDVarROD(ST_TRIM_RANGE, getRange(index), Sct_SctApi::SCAN_CONFIG), "QuickTrimTest::getNextScan")
	}
        // set trim value
	APICALL(api, modifyABCDVar(ST_TRIM, getValue(index)), "QuickTrimTest::getNextScan")
	APICALL(api, modifyABCDVarROD(ST_TRIM, getValue(index), Sct_SctApi::SCAN_CONFIG), "QuickTrimTest::getNextScan")

	Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();
	double offset = ranges[getRange(index)] * getValue(index);
	return sl->thresholdScanfCOffset(1000, 1.0, offset);  // trim at 1 fC
    }

    unsigned int getRange(unsigned int index) {
	if (index<4) return 0;
	else return (index-1)/3+1;
    }
    
    bool isFirstPoint(unsigned int index) {
      return (index==0 || (index-1)%3) == 0;
    }
    
    unsigned int getValue(unsigned int index) {
	if (index ==0 ) return 0;
	switch ((index-1)%3){
	case 0: return 2;
	case 1: return 5;
	case 2: return 8;
	default:
	  throw IllegalStateError("This index should never happen!", __FILE__, __LINE__);
	}
    }
    
    virtual bool canFeedbackRealImpl() {
	return true;
    }    

    virtual void endTestRealImpl() {
	api = Sct_SctApi::SctApiIPC::_nil();
    }
        
private:
    QuickTrimTest() {}
    Sct_SctApi::SctApiIPC_var api;
    static double ranges[4];
};
double QuickTrimTest::ranges[4] = {3.5, 6.5, 10, 13};
}
}
#endif //QUICKTRIMTEST_H
