#ifndef SYNCH_DOUBLETRIGGERNOISETEST_H
#define SYNCH_DOUBLETRIGGERNOISETEST_H

#include <cmath>  // For fabs

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.hh"
#include "../CalibrationControllerImpl.h"
#include "../ipc/ScanRequest.h"
#include "../SctApiAccessException.h"
#include "../SctApiCall.h"
#include "Sct/LogicErrors.h"

#include <Sct/AbcdScans.h>

using namespace Sct;
using namespace SctCalibrationController::Ipc;
namespace SctCalibrationController {
namespace Scripts {

class SynchDoubleTriggerNoiseTest  : public TestRequestImpl {
private:
   static const bool registeredInTestLibrary;

public:
    static Sct_CalibrationController::TestRequest_ptr instance() {
      static SynchDoubleTriggerNoiseTest dtnt;
      dtnt.setFitAlgorithm("NONE");
      dtnt.setAnalysisAlgorithm("DoubleTriggerNoiseTest");
      return dtnt._this();
    }

    virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = Sct_SctApi::SctApiIPC::_duplicate(api);

        // Compression mode X1X
        APICALL(api, modifyABCDVar(ST_COMPRESSION, 1), "SynchDoubleTriggerNoiseTest::startTest")

        // Edge detect off
        APICALL(api, modifyABCDVar(ST_EDGE_DETECT, 0), "SynchDoubleTriggerNoiseTest::startTest")

        // No masked channels
        APICALL(api, modifyABCDVar(ST_SEND_MASK, 0), "SynchDoubleTriggerNoiseTest::startTest")

        // 1 fC threshold
        APICALL(api, modifyABCDVar(ST_QTHR, 1.0), "SynchDoubleTriggerNoiseTest::startTest")

	name = "SynchDoubleTriggerNoiseTest";
	testVariable = ST_ELAPSED;
	nScans = 1;
	testPoints.push_back(0.);
    }


    virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
        Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();
        Sct_CalibrationController::ScanRequest_ptr request = sl->defaultScan();

	// Create scan
        Sct_SctApi::Scan_var s;
        APIRETCALL(api, s, createScan(), "SynchDoubleTriggerNoiseTest::getNextScan")
        request->setScanCorba(s);

        APICALL(s, setNTrigs(10000), "Failed to set triggers")

        // This doesn't work with group distribution which is the default otherwise
	APICALL(s, setOption(Sct_SctApi::Scan::distSlave, 0), "Failed to set distSlave option")

        // Only capture every 2nd trigger
	APICALL(s, setOption(Sct_SctApi::Scan::nth, 2), "Failed to set nth option")

        // Capture the second of the two triggers (index 0)
	// Note that some data has been taken (e.g. B6 assembly) with nth_rem incorrectly set to 
	// 1, which is the intuitive value but is WRONG. The light-leak peaks seen in B3 are only
	// visible with nth_rem set to 0 (confirmed by Dave R at SR1).
	APICALL(s, setOption(Sct_SctApi::Scan::nth_rem, 0), "Failed to set nth_rem option")

        // Add occupancy-per-event histograms
	APICALL(s, setOption(Sct_SctApi::Scan::ope, 1), "Failed to set ope option")

	// Synchronise!
	APICALL(s, setOption(Sct_SctApi::Scan::tim, 1), "Failed to set tim option")

	// Adjust trigger
        Sct_SctApi::Trigger_var t;
        APIRETCALL(s, t, getTrigger1(), "Failed to get trigger1")

        APICALL(t, doubleL1A(110), "Failed to set trigger type")
        APICALL(t, setFrequency(20.), "Failed to set trigger frequency")
        APICALL(t, setRandom(2), "Failed to set trigger random")
        APICALL(t, setSource(::Sct_SctApi::Trigger::SOURCE_TIM_INTERNAL), "Failed to set trigger random")

	// Increment delay between triggers
	APICALL(t, setCommIncr(1, 1), "SynchDoubleTriggerNoiseTest::getNextScan")

	// Configure scan
	// Note that while setCommIncr is uses to scan the delay with the 
        // pre- 26/7/2005 DSP code, after this it is done through ST_TRIG_DELAY
        APICALL(s, configure(ST_TRIG_DELAY1, 110, 140, 1), "Failed to set Scan information")

	return request;
    }

    virtual bool canFeedbackRealImpl() {
	return false;
    }

    virtual void endTestRealImpl() {
	api = Sct_SctApi::SctApiIPC::_nil();
    }

private:
    SynchDoubleTriggerNoiseTest() {}
    Sct_SctApi::SctApiIPC_var api;
};
    
}
}

#endif //SYNCH_DOUBLETRIGGERNOISETEST_H
