#ifndef SYNCHTRIGGERNOISETEST_H
#define SYNCHTRIGGERNOISETEST_H

#include <cmath>  // For fabs

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.hh"
#include "../CalibrationControllerImpl.h"
#include "../ipc/ScanRequest.h"
#include "../SctApiAccessException.h"
#include "../SctApiCall.h"
#include "Sct/LogicErrors.h"

#include <Sct/AbcdScans.h>

using namespace Sct;
using namespace SctCalibrationController::Ipc;
namespace SctCalibrationController {
namespace Scripts {

class SynchTriggerNoiseTest  : public TestRequestImpl {
private:
   static const bool registeredInTestLibrary;

public:
    static Sct_CalibrationController::TestRequest_ptr instance() {
      static SynchTriggerNoiseTest stnt;
      stnt.setFitAlgorithm("Threshold");
      stnt.setAnalysisAlgorithm("NoiseOccupancyTest");
      return stnt._this();
    }

    virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = Sct_SctApi::SctApiIPC::_duplicate(api);

        // Compression mode X1X
        APICALL(api, modifyABCDVar(ST_COMPRESSION, 1), "SynchTriggerNoiseTest::startTest")

        // Edge detect off
        APICALL(api, modifyABCDVar(ST_EDGE_DETECT, 0), "SynchTriggerNoiseTest::startTest")

        // No masked channels
        APICALL(api, modifyABCDVar(ST_SEND_MASK, 0), "SynchTriggerNoiseTest::startTest")

	name = "NoiseOccupancyTest";
	testVariable = ST_ELAPSED;
	nScans = 1;
	testPoints.push_back(0.);
    }


    virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
        Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();
        Sct_CalibrationController::ScanRequest_ptr request = sl->defaultScan();

	// Create scan
        Sct_SctApi::Scan_var s;
        APIRETCALL(api, s, createScan(), "SynchTriggerNoiseTest::getNextScan")
        request->setScanCorba(s);

        // This doesn't work with group distribution which is the default otherwise
	APICALL(s, setOption(Sct_SctApi::Scan::distSlave, 0), "Failed to set distSlave option")
	  
        const unsigned nth_trigger=15;
	APICALL(s, setOption(Sct_SctApi::Scan::tim, 1), "Failed to set TIM option")
	APICALL(s, setOption(Sct_SctApi::Scan::nth, nth_trigger), "Failed to set nth option")

        // request occupancy per event
	APICALL(s, setOption(Sct_SctApi::Scan::ope, 1), "Failed to set ope option")

	// Adjust trigger
        Sct_SctApi::Trigger_var t;
        APIRETCALL(s, t, getTrigger1(), "Failed to get trigger1")

        APICALL(t, singleL1A(), "Failed to set trigger type")
        APICALL(t, setFrequency(40.), "Failed to set trigger frequency")
        APICALL(t, setRandom(2), "Failed to set random trigger")
        APICALL(t, setSource(::Sct_SctApi::Trigger::SOURCE_TIM_INTERNAL), "Failed to set random trigger")

	// Configure scan - set range in mV relative to trim target (1.0 fC).
        APICALL(s, configure(ST_QTHR, 1.1, 0.8, -0.025), "Failed to set Scan information")
	  
        // set the number of triggers
        APICALL(s, setNTrigs(200000), "Failed to set triggers")

	return request;
    }

    virtual bool canFeedbackRealImpl() {
	return false;
    }

    virtual void endTestRealImpl() {
	api = Sct_SctApi::SctApiIPC::_nil();
    }

private:
    SynchTriggerNoiseTest() {}
    Sct_SctApi::SctApiIPC_var api;
};
    
}
}

#endif //SYNCHTRIGGERNOISETEST_H
