#ifndef TIMCLOCKDELAYTEST_H
#define TIMCLOCKDELAYTEST_H

#include <cmath>  // For fabs

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.hh"
#include "../CalibrationControllerImpl.h"
#include "../ipc/ScanRequest.h"
#include "../SctApiAccessException.h"
#include "../SctApiCall.h"
#include "Sct/LogicErrors.h"

#include <Sct/AbcdScans.h>
#include <SctApi/extraScans.h>

using namespace Sct;
using namespace SctCalibrationController::Ipc;
namespace SctCalibrationController {
namespace Scripts {

class TimClockDelayTest  : public TestRequestImpl {
private:
   static const bool registeredInTestLibrary;

public:
    static Sct_CalibrationController::TestRequest_ptr instance() {
      static TimClockDelayTest stnt;
      stnt.setFitAlgorithm("NONE");
      stnt.setAnalysisAlgorithm("DoubleTriggerNoiseTest");
      return stnt._this();
    }

    virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = Sct_SctApi::SctApiIPC::_duplicate(api);

        // Compression mode X1X
        APICALL(api, modifyABCDVar(ST_COMPRESSION, 1), "TimClockDelayTest::startTest")

        // Edge detect off
        APICALL(api, modifyABCDVar(ST_EDGE_DETECT, 0), "TimClockDelayTest::startTest")

        // No masked channels
        APICALL(api, modifyABCDVar(ST_SEND_MASK, 0), "TimClockDelayTest::startTest")

        // TIM external trigger and clock input
#warning "Not needed if this was in SctApi"
        APICALL(api, timWriteRegister(0, 0, 0x00,0x300), "PreTriggerNoiseTest::startTest")

	name = "TimClockDelayTest";
	testVariable = ST_ELAPSED;
	nScans = 1;
	testPoints.push_back(0.);
    }


    virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
        Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();
        Sct_CalibrationController::ScanRequest_ptr request = sl->defaultScan();

	// Create scan
        Sct_SctApi::Scan_var s;
        APIRETCALL(api, s, createScan(), "TimClockDelayTest::getNextScan")
        request->setScanCorba(s);

        // This doesn't work with group distribution which is the default otherwise
	APICALL(s, setOption(Sct_SctApi::Scan::distSlave, 0), "Failed to set distSlave option")
	  
        const unsigned nth_trigger=1;
	APICALL(s, setOption(Sct_SctApi::Scan::tim, 8), "Failed to set TIM option")
	APICALL(s, setOption(Sct_SctApi::Scan::nth, nth_trigger), "Failed to set nth option")

        // request occupancy per event
	APICALL(s, setOption(Sct_SctApi::Scan::ope, 1), "Failed to set ope option")

	// Adjust trigger
        Sct_SctApi::Trigger_var t;
        APIRETCALL(s, t, getTrigger1(), "Failed to get trigger1")
        APICALL(t, singleL1A(), "Failed to set trigger type")

	// Configure scan - 0->25 ns in 0.5 ns steps (1 unit=0.5 ns).
        APICALL(s, configure(SCT_TIM_CLOCK_DELAY, 0, 50, 1), "Failed to set Scan information")
	  
        // set the number of triggers
        APICALL(s, setNTrigs(10), "Failed to set triggers")

	return request;
    }

    virtual bool canFeedbackRealImpl() {
	return false;
    }

    virtual void endTestRealImpl() {

        // TIM internal triggers
        APICALL(api, timWriteRegister(0, 0,0x0,0x0), "PreTriggerNoiseTest::startTest")

	api = Sct_SctApi::SctApiIPC::_nil();
    }

private:
    TimClockDelayTest() {}
    Sct_SctApi::SctApiIPC_var api;
};
    
}
}

#endif //TIMCLOCKDELAYTEST_H
