#include "FitAlgorithmMap.h"
#include "FitAlgorithm.h"
#include "SctData/ConfigurationVariable.h"

#include <iostream>
using namespace std;

namespace SctFitter {

FitAlgorithmMap& FitAlgorithmMap::instance() {
    static FitAlgorithmMap* map = new FitAlgorithmMap();
    return *map;
}
    
const FitAlgorithm& FitAlgorithmMap::getAlgorithm(const std::string name) const throw(Sct::InvalidArgument) {
  if (algMap.count(name) == 0) {
    std::cout << "Known names for algorithms" << endl;
    for (map<std::string, shared_ptr<FitAlgorithm> >::const_iterator i=algMap.begin();
	 i!=algMap.end(); ++i){
      std::cout << "[" << (*i).first << "] " << endl;
    }
    Sct::InvalidArgumentError iae("No map for ConfigurationVariable: " + name, __FILE__, __LINE__ );
    iae.setSeverity(MRS_DIAGNOSTIC);	
    throw iae;	
  }
  return *(*algMap.find(name)).second;
}
    
bool FitAlgorithmMap::setAlgorithm(string variableName, auto_ptr<FitAlgorithm> alg) throw() {
    algMap[variableName] = shared_ptr<FitAlgorithm>(alg);
    return true;
}

std::list<std::string> FitAlgorithmMap::listAlgorithms() const{
   std::list<std::string> alg_list;
   for (map<std::string, shared_ptr<FitAlgorithm> >::const_iterator i=algMap.begin();
        i!=algMap.end(); ++i){
      alg_list.push_back((*i).first); 
   }
   return alg_list;
}

}
