#include "ISNPtGainSummaryData.h"
#include "ISSummaryManager.h"
#include "ISNPtGainSummaryWriter.h"

#include "Sct/ISUtilities.h"
#include "Sct/SctParameters.h"

#include "SctData/NPtGainTestResult.h"
#include "SctData/ResponseCurve.h"
#include "SctData/Stat.h"

#include <iomanip>
#include <TF1.h>
#include <string>

namespace SctData {
    namespace ISTestSummary{
	using namespace std;

	void ISNPtGainSummaryWriter::write(const TestResult& t) const 
	{
	    const NPtGainTestResult& result  = dynamic_cast<const NPtGainTestResult&>(t);

	    ISNPtGainSummaryData summary;

	    writeHeader(result, summary);

	    summary.gainByChip_size = Sct::nChipModule;
	    summary.gainByChip = new double[Sct::nChipModule];
	    summary.gainRMSByChip_size = Sct::nChipModule;
	    summary.gainRMSByChip = new double[Sct::nChipModule];
	    summary.offsetByChip_size = Sct::nChipModule;
	    summary.offsetByChip = new double[Sct::nChipModule];
	    summary.offsetRMSByChip_size = Sct::nChipModule;
	    summary.offsetRMSByChip = new double[Sct::nChipModule];
	    summary.noiseByChip_size = Sct::nChipModule;
	    summary.noiseByChip = new double[Sct::nChipModule];
	    summary.noiseRMSByChip_size = Sct::nChipModule;
	    summary.noiseRMSByChip = new double[Sct::nChipModule];

	    for (short unsigned ichip=0; ichip<nChipModule; ++ichip){
		Stats<float> gain(nChannelChip);
		Stats<float> offset(nChannelChip);
		Stats<float> innse(nChannelChip);
		
		for (unsigned ichan=0; ichan<nChannelChip; ++ichan){
		    unsigned ichannel = ichan + ichip*nChannelChip;
		    bool valid=result.getDefects().defectSeverityEncompassingElement(ModuleElement::Channel(ichannel)) < SERIOUS;		    

		    gain.modifyAt(ichan).valid = valid;
		    offset.modifyAt(ichan).valid = valid;
		    innse.modifyAt(ichan).valid = valid;
		    
		    gain.modifyAt(ichan).value = result.getChannelData(ichannel).gain;
		    innse.modifyAt(ichan).value = result.getChannelData(ichannel).noise;
		    offset.modifyAt(ichan).value = result.getChannelData(ichannel).offset;
		}
		
		summary.gainByChip[ichip] = gain.mean();
		summary.gainRMSByChip[ichip] = sqrt(gain.var());
		summary.offsetByChip[ichip]  = offset.mean();
		summary.offsetRMSByChip[ichip] = sqrt(offset.var());
		summary.noiseByChip[ichip] = innse.mean();
		summary.noiseRMSByChip[ichip] = sqrt(innse.var());
		
	    }

	    string name =  "ISSummaryData.ISNPtGainSummaryData.";
	    name += result.getHeader().getModuleName();

	    // cout << " I am about to write the ISSummary " << endl;
	    // cout << " name " << name <<endl;

	    Sct::ISUtilities::addOrUpdateOrThrow(name,summary,__FILE__,__LINE__, MRS_WARNING );
	    
	}
	
	
	bool ISNPtGainSummaryWriter::inMap  = ISSummaryManager::instance().addWriter("SctData::NPtGainTestResult", shared_ptr<ISSummaryWriter>(new ISNPtGainSummaryWriter()));
    }
}
