#include "ISNoiseOccupancySummaryData.h"
#include "ISSummaryManager.h"
#include "ISNoiseOccupancySummaryWriter.h"
#include "Sct/ISUtilities.h"
#include "Sct/SctParameters.h"

#include "SctData/NoiseOccupancyTestResult.h"

#include <TF1.h>

#include <iomanip>

namespace SctData {
    namespace ISTestSummary{
	using namespace std;

	void ISNoiseOccupancySummaryWriter::write(const TestResult& t) const 
	{
	    const NoiseOccupancyTestResult& result  = dynamic_cast<const NoiseOccupancyTestResult&>(t);

	    ISNoiseOccupancySummaryData summary;

	    writeHeader(result, summary);

	    summary.offsetByChip_size = Sct::nChipModule;
	    summary.offsetByChip = new double[Sct::nChipModule];

	    summary.occupancyByChip_size = Sct::nChipModule;
	    summary.occupancyByChip = new double[Sct::nChipModule];
	    summary.occupancyRMSByChip_size = Sct::nChipModule;
	    summary.occupancyRMSByChip = new double[Sct::nChipModule];

	    summary.noiseByChip_size = Sct::nChipModule;
	    summary.noiseByChip = new double[Sct::nChipModule];

	    for (unsigned ichip=0; ichip<nChipModule; ++ichip){
	      const SctData::ChipNOResult& c = result.getChipResult(ichip);
	      
	      summary.offsetByChip[ichip] = c.offset;
	      summary.occupancyByChip[ichip] = c.mean;
	      summary.occupancyRMSByChip[ichip] = c.rms;
	      summary.noiseByChip[ichip] = c.getNoise();
	    }
	    
	    string name =  "ISSummaryData.ISNoiseOccupancySummaryData.";
	    name += result.getHeader().getModuleName();
	    
	    // cout << " I am about to write the ISSummary " << endl;
	    // cout << " name " << name <<endl;
	    
	    Sct::ISUtilities::addOrUpdateOrThrow(name,summary,__FILE__,__LINE__, MRS_WARNING );
	    
	}


	bool ISNoiseOccupancySummaryWriter::inMap = ISSummaryManager::instance().addWriter("SctData::NoiseOccupancyTestResult", shared_ptr<ISSummaryWriter>(new ISNoiseOccupancySummaryWriter()));
    }
}
