#include "ISSummaryManager.h"
#include "ISSummaryWriter.h"
#include "SctData/TestResult.h"
#include "Sct/SctNames.h"
#include <sstream>
#include <iomanip>
#include <cstdlib>
#include <boost/date_time/posix_time/posix_time.hpp>

using namespace Sct;
using namespace boost;
using namespace std;

namespace SctData {
    namespace ISTestSummary{
	using boost::shared_ptr;

	ISSummaryManager::ISSummaryManager() {}

	ISSummaryManager::~ISSummaryManager() {}

	ISSummaryManager& ISSummaryManager::instance()  {
	    static ISSummaryManager bertha;
	    return bertha;
	}

	void ISSummaryManager::write(const TestResult& t) {
	    shared_ptr<const ISSummaryWriter> writer = getWriter(t.getClassName());
	    writer->write(t);
	}
	
	shared_ptr<const ISSummaryWriter> ISSummaryManager::getWriter(const string& classname) const  {
	    WriterMap::const_iterator it = theMap.find(classname);
	    if ( it == theMap.end() ){
		cerr << "ISSummaryManager::getWriter no writer for " << classname << endl;
		cerr << "known writers:" << endl ;
		for (WriterMap::const_iterator it = theMap.begin(); it != theMap.end(); ++it){
		    cerr << (*it).first << endl;
		}
		throw InvalidArgumentError( string("ISSummaryManager::getWriter no writer for ") + classname, __FILE__, __LINE__ );
	    } else {
		return (*it).second ;
	    }
	}

	bool ISSummaryManager::addWriter(const string& testname, shared_ptr<ISSummaryWriter> writer) {
	    if (theMap.find(testname) == theMap.end() ){
		theMap[testname] = writer;
		return true;
	    }
	    return false;
	}

    } // end of namespace TestISSummary
} // end of namespace SctData
