#ifndef SCTDATA_SUMMARYWRITER_H
#define SCTDATA_SUMMARYWRITER_H

#include <boost/utility.hpp>
#include "ISSummaryData.h"

/**
   Classes which take SctData::TestResult data and publish some of it 
   into the Information Service, IS, so it can be put into COOL by CDI. 
   @author Bilge Demirkoz
   @date 14 November 2005
*/
namespace SctData {
    class TestResult;

    namespace ISTestSummary{
	/** 
	    An ISSummaryWriter is a type of output streamer for a TestResult 
	    Concrete SummaryWriters should add themselves to the SummaryManager's
	    map, and impliment the write() method.
	*/

	class ISSummaryWriter : boost::noncopyable {
	public:
	    /** 
		append a test to the appropriate SctDaq-style file 
		@param t the TestResult to write the summary for.
		@param out the output stream to write to.
	    */
	  virtual void writeHeader(const TestResult& r, ISSummaryData& summary) const ;

	  virtual void write(const TestResult& r) const =0;

	  /** do NOT delete the singleton ! */
	  virtual ~ISSummaryWriter();
	  
	protected:
	  ISSummaryWriter(); /// used only by sub-classes to add themselves to the map.
	  
	};
    }
} // end of namespace SctDaqSummary

#endif // #ifdef SCTDATA_SUMMARYWRITER_H
