#include <iostream>
using namespace std;

#include <ctype.h>

#include <unistd.h>

#include "RodModule.h"
#include "BocCard.h"

#include "RCCVmeInterface.h"
#include "registerIndices.h"

using namespace SctPixelRod;

void usage();

int main(int argc, char **argv) {
  const unsigned long mapSize=0xc00040;         // Map size
  const long numSlaves=4;                       // Number of slaves

  int slot = -1;
  int link = -1;

  unsigned long baseAddress;

  if (argc > 1) {
    for (int i=1; i<argc; i++) {
      string option;
      option = argv[i];
      if (option[0] != '-') break;
      switch (option[1]) {
      case 's': slot = atoi(option.substr(2).c_str()); break;
      case 'l': link = atoi(option.substr(2).c_str()); break;
      default: {
        break;
      }
      }
    }
  }

  // Prompt for slot number
  if (slot < 0 || link < 0) {
    cout << "Must specify all parameters\n";
    usage();
    exit(1);
  }
  baseAddress = slot << 24;

  // Create VME interface
  RCCVmeInterface vme;
  
  // Create RodModule
  RodModule rod(baseAddress, mapSize, vme, numSlaves);
  rod.initialize(false);

  int formatter = link/12;
  int intLink = link%12;

  int primLength = sizeof(RW_REG_FIELD_IN)/4;
  long regData[primLength];
  struct RW_REG_FIELD_IN &regStruct = *(RW_REG_FIELD_IN *)regData;

  regStruct.registerID = FMT_LINK_DATA_TEST_MUX(formatter);
  regStruct.offset = 0;
  regStruct.width = 4;
  regStruct.readNotWrite = 0;
  regStruct.dataIn = intLink;

  RodPrimitive registerPrim(primLength+4, 1, RW_REG_FIELD, R_RW_REG_FIELD, regData);
  rod.synchSendPrim(registerPrim);

  regStruct.registerID = EFB_CMND_0;
  regStruct.offset = 4;
  regStruct.width = 3;
  regStruct.readNotWrite = 0;
  regStruct.dataIn = formatter & 0x7;

  RodPrimitive registerPrim2(primLength+4, 1, RW_REG_FIELD, R_RW_REG_FIELD, regData);
  rod.synchSendPrim(registerPrim2);
  
  // No clean up to do before exiting
  return 0;  
}

void usage() {
  cout << "Usage:\n";
  cout << " SetTestLinkOut -s[slot] -l[link]\n";
}
