#include <config/Configuration.h>
// #include <config/Change.h>
// #include <config/ConfigObject.h>
// #include <oksconfig/OksConfiguration.h>

#include <boost/bind.hpp>
#include <boost/thread.hpp>

#include "DAL/Partition.h"

static void cb1(const std::vector<ConfigurationChange *> & changes, void * dummyParameter);
static std::string buildString(std::string base, int number);

void changeThread(Configuration &conf) {
  ConfigurationSubscriptionCriteria crit;
  //  crit.add(SCTDAL::Partition::s_class_name);
  Configuration::CallbackId callback = conf.subscribe(crit, cb1, NULL, true);
}

int main() {
  Configuration conf("oksconfig");

  std::list<std::string> includes;
  includes.push_back("sectorSchema.xml");
  //  includes.push_back("online/segments/setup.data.xml"); // online infrastructure

  const char * db_name = "test-data3.xml"; // new database file name
  if(conf.create("", db_name, includes) == false) {
    std::cerr << "ERROR: failed to create file \'" << db_name << "\'\n";
    conf.abort();
  }

  //  boost::thread *changeNotification = new boost::thread(boost::bind(changeThread, boost::ref(conf)));

  //  conf.check_notification();

  //  const char * db = "/tmp/my-db.data.xml";
  const SCTDAL::Partition * host = conf.create<SCTDAL::Partition>(db_name, "partition-1");
  if(host == 0) {
    std::cerr << "ERROR: failed to create object \'host-1\' at \'" << db_name << "\'\n";
  }
  else {
    if(conf.create<SCTDAL::Partition>(*host, "partition-2") == 0) {
      std::cerr << "ERROR: failed to create object \'host-2\' at file of " << host;
    }
  }

  conf.check_notification();

  bool success = true;

  if(success) {
    std::cout << "commit changes\n";
    conf.commit();
  }
  else {
    std::cout << "ERROR: something was wrong, abort changes\n";
    conf.abort();
  }


  const_cast<SCTDAL::Partition*>(host)->set_name("Hello world");
  conf.check_notification();

  conf.commit();
  conf.check_notification();

  //  changeNotification->join();
  std::cout << "Leaving...\n";
}

/**
* The callback function is called when changes occurred.
* It contains all changes.
*/
static void cb1(const std::vector<ConfigurationChange *> & changes, void * dummyParameter) {
  std::cout << "CALLBACK 1 (report all changes):\n";
  // iterate changes sorted by classes
  for(std::vector<ConfigurationChange *>::const_iterator j = changes.begin();
      j != changes.end(); ++j) {
    // get class name
    const std::string& class_name = (*j)->get_class_name();
    std::cout << "- there are changes in class \"" << class_name << "\"\n";
    std::vector<std::string>::const_iterator i;
    // print modified objects
    for(i = (*j)->get_modified_objs().begin();
        i != (*j)->get_modified_objs().end(); ++i) {
      std::cout << " * object \"" << *i << "\" was modified\n";
    }
    // print removed objects
    for(i = (*j)->get_removed_objs().begin();
        i != (*j)->get_removed_objs().end(); ++i) {
      std::cout << " * object \"" << *i << "\" was removed\n";
    }
    // print created objects
    for(std::vector<std::string>::const_iterator i = (*j)->get_created_objs().begin(); i != (*j)->get_created_objs().end(); ++i) {
      std::cout << " * object \"" << *i << "\" was created\n";
    }
  }
}

