#include <config/Configuration.h>

#include <boost/bind.hpp>
#include <boost/thread.hpp>

#include "DAL/GeographyMUR.h"
#include "DAL/Row.h"

/*
  Problem seen by dalImpl:
    Sometimes Configuration::referenced_by returns a list of 1 null pointer
    So far only seen in: translateToBarrel getting reference from MUR to Row
                         translateFromSN getting reference from Module to Map

    This was due to a bug in my code... so this test is abondonned

 */

SCTDAL::Row *createRow(int barrel, int row);
SCTDAL::GeographyMUR *createMUR(int mur, std::string position);
void linkRowToMur(SCTDAL::Row *rowObj, SCTDAL::GeographyMUR *murObj);

const SCTDAL::Row *getBackLink(SCTDAL::GeographyMUR *murObj);
const SCTDAL::GeographyMUR *getForwardLink(SCTDAL::Row *rowObj);

std::string testFilename("reverseTest.data.xml");
Configuration conf("oksconfig");

int main() {
  // In case the previous run didn't tidy up
  system(("rm .oks-lock-" + testFilename).c_str());
  system(("rm " + testFilename).c_str());

  std::list<std::string> includes;
  includes.push_back("SCT.schema.xml");

  if(conf.create("", testFilename, includes) == false) {
    std::cerr << "ERROR: failed to create file \'" << testFilename << "\'\n";
    conf.abort();
    return 1;
  }

  SCTDAL::Row *rowObj = createRow(0, 10);
  std::cout << "Created row\n";
  if(rowObj) 
    rowObj->print(2, true, std::cout);
  SCTDAL::GeographyMUR *murObj = createMUR(1000, "-1");
  std::cout << "Created MUR\n";
  if(murObj)
    murObj->print(2, true, std::cout);
  linkRowToMur(rowObj, murObj);
  std::cout << "Created link\n";
  if(rowObj) 
    rowObj->print(2, true, std::cout);

  const SCTDAL::GeographyMUR *forwardMur = getForwardLink(rowObj);
  if(forwardMur) 
    forwardMur->print(2, true, std::cout);

  const SCTDAL::Row *backRowObj = getBackLink(murObj);
  if(backRowObj)
    backRowObj->print(2, true, std::cout);
}

SCTDAL::Row *createRow(int barrel, int row) {
  SCTDAL::Row *rowOb = const_cast<SCTDAL::Row*>(conf.create<SCTDAL::Row>(testFilename, ""));

  if(!rowOb) {
    std::cerr << "Can't create row\n";
    conf.abort();
    return 0;
  }

  rowOb->set_id(row);

  conf.commit();

  return rowOb;
}

SCTDAL::GeographyMUR *createMUR(int mur, std::string position) {
  SCTDAL::GeographyMUR *murOb = const_cast<SCTDAL::GeographyMUR*>(conf.create<SCTDAL::GeographyMUR>(testFilename, ""));

  if(!murOb) {
    std::cerr << "Can't create Geog MUR\n";
    conf.abort();
    return 0;
  }

  murOb->set_id(mur);
  murOb->set_position(position);

  conf.commit();

  return murOb;
}

void linkRowToMur(SCTDAL::Row *rowObj, SCTDAL::GeographyMUR *murObj) {
  std::vector<const SCTDAL::GeographyMUR *> murs(rowObj->get_murs());
  murs.push_back(murObj);
  rowObj->set_murs(murs);
  conf.commit();
}

const SCTDAL::Row *getBackLink(SCTDAL::GeographyMUR *murObj) {
  std::vector<const SCTDAL::Row *> possibles;
  conf.referenced_by(*murObj, "murs", possibles);

  if(possibles.size() == 1) {
    const SCTDAL::Row *theRow = possibles.front();

    std::cout << "Found back link " << theRow << std::endl;
    return theRow;
  } else {
    std::cout << "Something wrong, size " << possibles.size() << std::endl;
  }
  return 0;
}

const SCTDAL::GeographyMUR *getForwardLink(SCTDAL::Row *rowObj) {
  std::vector<const SCTDAL::GeographyMUR *> possibles(rowObj->get_murs());

  if(possibles.size() == 1) {
    const SCTDAL::GeographyMUR *murObj = possibles.front();

    std::cout << "Found Forward link " << murObj << std::endl;
    return murObj;
  } else {
    std::cout << "Something wrong, size " << possibles.size() << std::endl;
  }
  return 0;
}
