
#include "MultiMessageDebugStream.h"

#include "mrs/message.h"
#include "SctNames.h"

namespace Sct {

  void MultiMessageDebugStream::severity(unsigned int s) {
    m_severity = s;
  };

  bool MultiMessageDebugStream::renew() {
    return (m_oss = new std::ostringstream);
  };

  MultiMessageDebugStream::MultiMessageDebugStream(const bool out,
                                                   const bool err,
                                                   const bool mrs) : 
    m_out(out),
    m_err(err),
    m_mrs(mrs),
    m_severity(MRS_INFORMATION),
    m_oss(0) {
  };
  
  MultiMessageDebugStream::~MultiMessageDebugStream() {
    this->flush();
    delete m_oss;
    m_oss=0;
  };
  
  void MultiMessageDebugStream::flush() {
    if (m_oss) {
      const std::string s = m_oss->str();
      delete m_oss;
      m_oss=0;
      this->announce(s);
    };
  };

  void MultiMessageDebugStream::announce(const std::string & s) {
    if (m_out) {
      std::cout << s << std::endl;
    };
    if (m_err) {
      std::cerr << s << std::endl;
    };
    if (m_mrs) {
      IPCPartition p(SctNames::getPartitionName());
      if(p.isValid()) {
        MRSStream mrs(p);
        mrs << static_cast<SEVERITY>(m_severity) << "MultiMessageDebugStream" << MRS_TEXT(s) << ENDM;
      };
    };
  };
  

};
