#include "UniqueID.h"
#include "LogicErrors.h"

#include <sstream>

using namespace std;
using namespace Sct;

namespace SctData {

UniqueID::UniqueID(){
}

UniqueID& UniqueID::operator=(const UniqueID& other){
  runNumber=other.runNumber;
  scanNumber=other.scanNumber;
  module=other.module;
  nameAsString=other.nameAsString;
  return *this;
}

bool UniqueID::operator==(const UniqueID& other) const{
  return (runNumber==other.runNumber &&
	  scanNumber==other.scanNumber &&
	  module==other.module &&
	  nameAsString==other.nameAsString);
}

bool UniqueID::operator!=(const UniqueID& other) const{
  return (!(*this==other));
}


UniqueID::UniqueID(string idAsString) : nameAsString(idAsString) {
    unsigned int dot = nameAsString.find('.');
    if (dot == nameAsString.npos) throw InvalidArgumentError("UniqueID can parse " + nameAsString + " - no run number", __FILE__, __LINE__);
    std::string temp = nameAsString.substr(0, dot);
    std::istringstream iss(temp);
    iss >> runNumber;

    unsigned int dot2 = nameAsString.find('.', ++dot);
    if (dot2 == nameAsString.npos) throw InvalidArgumentError("UniqueID can parse " + nameAsString + " - no scan number", __FILE__, __LINE__);
    temp = nameAsString.substr(dot, dot2-dot);
    std::istringstream iss2(temp);
    iss2 >> scanNumber;

    module = nameAsString.substr(++dot2);
}

UniqueID::UniqueID(unsigned int runNumber, unsigned int scanNumber, string module)
    : runNumber(runNumber), scanNumber(scanNumber), module(module) {
    ostringstream oss;
    oss << runNumber << "." << scanNumber << "." << module;
    nameAsString = oss.str();
}

UniqueID UniqueID::getUniqueID() const {
    return nameAsString;
}


UniqueID::operator string () const {
    return nameAsString;
}

string UniqueID::getModule() const {
    return module;
}

unsigned int UniqueID::getRunNumber() const {
    return runNumber;
}

unsigned int UniqueID::getScanNumber() const {
    return scanNumber;
}

}

std::ostream& operator<<(std::ostream& os, const Sct::UniqueID& id){
  os << (string)id;
  return os;
}

std::istream& operator>>(std::istream& is, Sct::UniqueID& id){
  std::string parseMe;
  is >> parseMe;
  id=UniqueID(parseMe);
  return is;
}

