#ifndef SCTDATA_DCS_DATA_H
#define SCTDATA_DCS_DATA_H

#include "Sct/Streamable.h"
#include <boost/shared_ptr.hpp>
#include <string>
#include <map>
#include <list>
#include <owl/time.h>

namespace SctData{
  /** 
      Class intended to hold the DCS data coming from a module,
      so that it can be archived along with test results etc. 
      for easy cross-referencing.
  */
class DcsData : public Sct::Streamable{
  /** internal class associates a time with each value */
  class DataPoint{
  public:
    DataPoint();
    DataPoint(float value, OWLTime time);
    float value;
    OWLTime time;
    ~DataPoint();
  };
 public:
  /* default ctr */
  DcsData();
  /* constructor from map of parameters */
  DcsData(const std::map<std::string, float> inputs);
  /** Streamable over-ride */
  virtual std::string getClassName() const;
  /** detr*/
  virtual ~DcsData();
  /// set parameter by name (Names defined by PVSS).
  void setParameter(const std::string& name, float val, OWLTime time);
  /// get time of value by name. @throw LogicError if not available.
  OWLTime getTime(const std::string& name) const;
  /// get parameter by name. @throw LogicError if not available.
  float getParameter(const std::string& name) const;
  /// check if parameter has a value
  bool hasParameter(const std::string& name) const;
  /// get list of parameters available
  std::list <std::string> getAllParameterNames() const;
 private:
  std::map<std::string, DataPoint> m_data;
  DataPoint getDataPoint(const std::string& name) const;
};
 
}

#endif //#ifndef SCTDATA_DCS_DATA_H
