#include "../NPtGainTestResult.h"
#include "../RootStreamableAdapter.h"
#include "../ResponseCurve.h"
#include "NPtGainTestResultStreamer_v2.h"

#include <TF1.h>
#include <vector>

using namespace Sct;

namespace SctData {
namespace IO {

/* READ NOTES ON STREAMERS AND VERSIONS BEFORE EDITING THIS FILE! */
unsigned NPtGainTestResultStreamer_v2::s_version=2;


NPtGainTestResultStreamer_v2::NPtGainTestResultStreamer_v2() throw() {}

bool NPtGainTestResultStreamer_v2::inMap = IOManager::addToMap("SctData::NPtGainTestResult",  auto_ptr<Streamer>(new NPtGainTestResultStreamer_v2()));


shared_ptr<Streamable> NPtGainTestResultStreamer_v2::read(IStream& in, const IOManager& manager) const throw(LogicError, IoError) {
    shared_ptr<Streamable> ad (new NPtGainTestResult());
    read(in, *ad, manager);
    return ad;
}

void NPtGainTestResultStreamer_v2::write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    //cout << "NPtGainTestResultIS::publishGuts" << endl;
    //Let superclass do its work
    manager.writeImpl(out, ob, "SctData::TestResult");

    const NPtGainTestResult& mytest = dynamic_cast <const NPtGainTestResult&>(ob);

    out << mytest.getNChipData();
    for (unsigned int i = 0; i < mytest.getNChipData(); ++i) {
        writeData(out, mytest.getChipData(i), manager);
    }

    out << mytest.getNChannelData();
    for (unsigned int i = 0; i < mytest.getNChannelData(); ++i) {
        writeData(out, mytest.getChannelData(i), manager);
    }

    putSlope(mytest.noiseSlope, out);
    putSlope(mytest.gainSlope, out);
    putSlope(mytest.offsetSlope, out);
}

void NPtGainTestResultStreamer_v2::writeData(OStream& out, const NPtGainTestResultData& ob, const IOManager& manager) const throw(LogicError, IoError) {
    manager.writeImpl(out, *ob.rc, false);
    manager.writeImpl(out, RootStreamableAdapter(*ob.graph), false);
    out << ob.gain << ob.noise << ob.offset;
}

void NPtGainTestResultStreamer_v2::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    manager.readImpl(in, ob, "SctData::TestResult");

    NPtGainTestResult& mytest = dynamic_cast < NPtGainTestResult & >(ob);

    unsigned int n = 0;
    in >> n;
    mytest.setChipDataSize(n);

    for (unsigned int i = 0; i<n; ++i) {
        readData(in, mytest, mytest.getChipData(i), manager);
    }

    in >> n;
    mytest.setChannelDataSize(n);
    for (unsigned int i = 0; i<n; ++i) {
        readData(in, mytest, mytest.getChannelData(i), manager);
    }
    
    getSlope(mytest.noiseSlope, in);
    getSlope(mytest.gainSlope, in);
    getSlope(mytest.offsetSlope, in);
}

void NPtGainTestResultStreamer_v2::getSlope(Sct::RangedVector<float>& d, Sct::IStream& is) const{
  size_t size=0;
  float* data=0;
  is.get(&data, size);
  d.resize(size);
  for (unsigned i=0; i<size; ++i){
    d[i]=data[i];
  }
  delete[] data;
}

void NPtGainTestResultStreamer_v2::putSlope(const Sct::RangedVector<float>& d, Sct::OStream& os) const{
    float data[d.size()];
    for (unsigned i=0; i<d.size(); ++i){
      data[i]=d[i];
    }
    os.put(data, d.size());
}

void NPtGainTestResultStreamer_v2::readData(IStream& in, NPtGainTestResult& mytest, NPtGainTestResultData& ob, const IOManager& manager) const throw(LogicError, IoError) {
    //Response Curve
    shared_ptr<Streamable> str = manager.readImpl(in, "SctData::ResponseCurve");
    shared_ptr<ResponseCurve> rc = boost::dynamic_pointer_cast<ResponseCurve>(str) ;
    ob.rc = rc;

    //TGraph
    ob.graph = shared_ptr<TGraph>(new TGraph());
    RootStreamableAdapter ad2(*ob.graph);
    manager.readImpl(in, ad2, false);

    //Rest
    in >> ob.gain >> ob.noise >> ob.offset;
}

}
} 

