#include "TH1DStreamer_v1.h"
#include "TH1.h"
#include "../RootStreamableAdapter.h"

using std::auto_ptr;

using namespace Sct;

namespace SctData {
namespace IO {

/* READ NOTES ON STREAMERS AND VERSIONS BEFORE EDITING THIS FILE! */
unsigned TH1DStreamer_v1::s_version=1;
  
bool TH1DStreamer_v1::inMap = IOManager::addToMap(TH1D::Class()->GetName(),  auto_ptr<Streamer>(new TH1DStreamer_v1()));
  
TH1DStreamer_v1::TH1DStreamer_v1() throw() {}
 
void TH1DStreamer_v1::write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
  const RootStreamableAdapter& ad = dynamic_cast<const RootStreamableAdapter&>(ob);
  TH1D* hist = (TH1D*)ad.getObject();
  out << hist->GetName();
  out << hist->GetTitle();
  out << hist->GetNbinsX();
  out.put(hist->GetArray(), hist->GetNbinsX()+2 );
  out.put(hist->GetXaxis()->GetXbins()->GetArray(), hist->GetNbinsX()+1 );
}
 
shared_ptr<Streamable> TH1DStreamer_v1::read(IStream& in, const IOManager& manager) const throw(LogicError, IoError) {
  shared_ptr<Streamable> ad (new RootStreamableAdapter(*new TH1D()));
  read(in, *ad, manager);
  return ad;
}
  
void TH1DStreamer_v1::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
  RootStreamableAdapter& ad = dynamic_cast<RootStreamableAdapter&>(ob);
  TH1D* hist = (TH1D*)ad.getObject();
  std::string name, title;
  unsigned int n =0;
  in >> name >> title;
  hist->SetName(name.c_str());
  hist->SetTitle(title.c_str());
  in >> n;
  double data[n+2], bins[n+1];
  in.get(data, n+2);
  in.get(bins, n+1);
  
  hist->SetBinsLength(n+2);
  hist->GetXaxis()->Set(n,bins);
 
  for (unsigned int i=0; i<n+2; ++i) {
    hist->SetBinContent(i,data[i]);
  } 
  }
}
}
