#ifndef OPERESULT_H
#define OPERESULT_H

#include "Sct/Serializable.h"
#include "Sct/RangedVector.h"
#include <boost/shared_ptr.hpp>

class TH1;

namespace SctData {
class ChipOpeResult;

    /**
     * small class holds all data relevant to one chip.
     */
 class ChipOpeResult : Sct::Streamable {
    public:
        ChipOpeResult() : badmax(0.), threshold(0.){;}
	/// Make a result with hist, max. 
	/// @param graph of rms versus threshold
	/// @param max rms on that chip
	ChipOpeResult(boost::shared_ptr<TH1>h, double r) : 
	  hist(h), badmax(r), threshold(r) {;}
	~ChipOpeResult(){;}
	shared_ptr<TH1> hist;
	double badmax, threshold;
	ChipOpeResult& operator=(const ChipOpeResult&);
	string ChipOpeResult::getClassName() const {
	  return "SctData::ChipOpeResult";
	}
 };



/**
The class represents the OPE test
*/
class OpeResult : public Sct::Serializable {
public:
    /**     Constructor.     */
    OpeResult(Sct::UniqueID id);
    
    /**     Destructor     */
    virtual ~OpeResult();

    //Override
    virtual string getClassName() const;
    
    virtual Sct::UniqueID getUniqueID() const;
    
    /// get all the results for a particular chip @{
    ChipOpeResult& getChipResult(unsigned ichip){ return chipResult[ichip];}
    const ChipOpeResult& getChipResult(unsigned ichip) const { return chipResult[ichip];}
    ///@}
private:
    OpeResult();
    Sct::RangedVector<ChipOpeResult> chipResult;
    OpeResult& operator=(const OpeResult&);
    friend class OpeResultIOHelper;
    Sct::UniqueID m_uniqueID;
};

}

#endif //#ifndef OPERESULT_H
