#include "SctApiConfigCache.h"
#include "ConfigurationUtility.h"
#include "SctApiException.h"

using boost::shared_ptr;
using SctApi::SctApiConfigCache;

SctApiConfigCache::SctApiConfigCache(){
}

SctApiConfigCache::~SctApiConfigCache(){
}

void SctApiConfigCache::set(UINT32 mid, shared_ptr<ModuleBanks> banks){
  boost::recursive_mutex::scoped_lock lock(cache_mutex);
  m_map[mid] = banks;
}

void SctApiConfigCache::clearMid(const UINT32 mid){
  boost::recursive_mutex::scoped_lock lock(cache_mutex);
  std::map<UINT32, boost::shared_ptr<ModuleBanks> >::iterator mit=m_map.find(mid);
  if (mit != m_map.end() ) m_map.erase(mit);
}

void SctApiConfigCache::clearAll(){
  boost::recursive_mutex::scoped_lock lock(cache_mutex);
  m_map.clear();
}

shared_ptr<SctApi::SctApiConfigCache::ModuleBanks> SctApiConfigCache::getFromMid(const UINT32 mid){
  boost::recursive_mutex::scoped_lock lock(cache_mutex);
  shared_ptr<ModuleBanks> banks;
  if (m_map.find(mid)!=m_map.end()) banks = m_map[mid];
  return banks;
}

shared_ptr<const SctApi::SctApiConfigCache::ModuleBanks> SctApiConfigCache::getFromMid(const UINT32 mid) const {
  boost::recursive_mutex::scoped_lock lock(cache_mutex);
  boost::shared_ptr<const ModuleBanks> banks;
  std::map<UINT32, boost::shared_ptr<ModuleBanks> >::const_iterator it=m_map.find(mid);
  if (it!=m_map.end()) banks = (*it).second;
  return banks;
}

void SctApiConfigCache::copyAllModules(BankType from, BankType to){
  for (std::map<UINT32, boost::shared_ptr<ModuleBanks> >::const_iterator it=m_map.begin(); it!= m_map.end(); ++it){
    shared_ptr<ModuleBanks> banks = it->second;
    if (!banks.get()) throw SctApiException("Internal error - null pointer in SctApiConfigCache copyAllModules");
    banks->copy(from, to);
  }
}

void SctApiConfigCache::copyAllModules(BankType from, std::list<BankType> to){
  for (std::list<BankType>::const_iterator i = to.begin(); i!= to.end(); ++i){
    copyAllModules(from, *i);
  }
}

boost::shared_ptr<ABCDModule> SctApiConfigCache::ModuleBanks::get(BankType b){
  boost::recursive_mutex::scoped_lock lock(bank_mutex);
  boost::shared_ptr<ABCDModule> config;
  std::map<BankType, boost::shared_ptr<ABCDModule> >::iterator it=m_map.find(b);
  if (it!=m_map.end()) config=(*it).second;
  return config;
}

boost::shared_ptr<const ABCDModule> SctApiConfigCache::ModuleBanks::get(BankType b) const{
  boost::recursive_mutex::scoped_lock lock(bank_mutex);
  boost::shared_ptr<const ABCDModule> config;
  std::map<BankType, boost::shared_ptr<ABCDModule> >::const_iterator it=m_map.find(b);
  if (it!=m_map.end()) config=(*it).second;
  return config;
}

void SctApiConfigCache::ModuleBanks::set(BankType bank, boost::shared_ptr<ABCDModule> config){
  boost::recursive_mutex::scoped_lock lock(bank_mutex);
  m_map[bank]=config;
}

void SctApiConfigCache::ModuleBanks::copy(BankType from, std::list<BankType> to){
  for (std::list<BankType>::const_iterator i = to.begin(); i!= to.end(); ++i){
    copy(from, *i);
  }
}

void SctApiConfigCache::ModuleBanks::copy(BankType from, BankType to){
  boost::recursive_mutex::scoped_lock lock(bank_mutex);
  boost::shared_ptr<ABCDModule> copyme=get(from);
  if (!copyme.get()) {
    std::string message("Attempt to copy null bank");
    message += __FILE__;
    message += __LINE__;
    throw SctApiException(message);
  }
  std::auto_ptr<ABCDModule> copy=ConfigUtility::clone(*copyme);
  m_map[to]=copy;
}
