// enum RunType {PHYSICS, CALIBRATION};
#include "SctApi.h"

#include "Sct/MultiMessageDebugStream.h"
#include "CommonWithDsp/registerIndices.h"

#include "sctConf/configuration.h"

#include "primListWrapper.h"
#include "PrimBuilder.h"

#include "CommonWithDsp/primParams.h"

namespace SctApi {
  bool SctApi::changeRunMode(enum RunType mode) {
    switch(mode) {
    case SCTAPI_RUN_PHYSICS:
      {
	{	
	  Sct::MultiMessageDebugStream m(true,false,true);
	  m << ucid() << " Is changing to Physics mode\n";
	}

	// Modules already in Physics mode?
	if(!checkAllModulesProbe("E")) {
#warning "Possibly disable the ones that aren't returning events? (with warnings!)"
	  if(mrs) {
	    *mrs << "SCAN_ABORTED" << MRS_ERROR << MRS_QUALIF("SCTAPI") 
		 << MRS_TEXT("Switch to physics mode aborting (All modules not returning events)") 
		 << MRS_PARAM<const char*>("ucid",static_cast<std::string>(ucid()).c_str()) << ENDM;
	  }
	  return false;
	}
	// What resets to do?

	// Prepare a longish primitive list 
	boost::shared_ptr<PrimListWrapper> preTimList(new PrimListWrapper(4));

	for(int f=0; f<8; f++) {
	  //   PrimBuilder::instance().writeRegister(preTimList, DM_DFLT_LUT(f), 0, 16, 0xaaaa);   // Decrement L1 for check
	  //   PrimBuilder::instance().writeRegister(preTimList, DM_DFLT_LUT(f), 0, 16, 0x5555);   // Increment L1 for check
	  // 	PrimBuilder::instance().writeRegister(preTimList, DM_DFLT_LUT(f), 0, 16, 0x0);   // Clear L1 inc
	}

	//for(int ch=0; ch<48; ch++) {
	//  PrimBuilder::instance().writeRegister(preTimList, Utility::EfbErrorMask(0, ch), 5, 1, 1);  // Mask L1ID check
	//  PrimBuilder::instance().writeRegister(preTimList, Utility::EfbErrorMask(1, ch), 5, 1, 1);  // Mask L1ID check
	//}

	//       std::cout << "Sending preparatory TIM setup list\n";
	//       synchSendPrimListAll(preTimList);
	//       std::cout << " done sending preparatory TIM setup list\n";
	
	boost::shared_ptr<PrimListWrapper> physicsSetupList(new PrimListWrapper(4));
	
	PrimBuilder &builder = PrimBuilder::instance();

	bool doingHistograms = false;
	if(doingHistograms) {
	  // SET_TRIGGER (ie tell histograms about next bin)
	  boost::shared_ptr<PrimListWrapper> setTriggerList(new PrimListWrapper(4));
	  // 	PrimBuilder::instance().setTrigger(setTriggerList, binCount, Utility::translateBank(SCTAPI_BANK_SCAN));
	  
	  // Send to all slaves
	  for(int s=0; s<4; s++) {
	    PrimBuilder::instance().slavePrimList(physicsSetupList, setTriggerList, s, 1, 0);
	  }
	}

	// Set offset for BCID check
	//  (this value tested at CERN SR1 with local LTP resets and triggers)
	builder.writeRegister(physicsSetupList, EFB_CMND_0, 24, 8, 0xfb);
	builder.rodMode(physicsSetupList, DATA_TAKING_MODE, 0, 1, 0, 0, 1);

	// Don't mask LDown# or S-link!!!
	builder.writeRegister(physicsSetupList, RTR_CMND_STAT, RTR_INHIBIT_SLNK_WE_O, 1, 0);
	builder.writeRegister(physicsSetupList, RTR_CMND_STAT, RTR_SLINK_DOWN_OVERRIDE_O, 1, 0);

	std::cout << "Sending TIM setup list\n";
	synchSendPrimListAll(physicsSetupList);
	std::cout << " done sending TIM setup list\n";

 	// Set BCID offset to 0  (reset value is 6!)
 	timWriteRegister(0x14, 0x0000);

	// Specify some sane defaults
	unsigned short triggerEnablesInternal = 0x0;
	unsigned short triggerEnablesTTC = 0x62f; // We know this works, 0xc7af has more things

	try {
	  SctConfiguration::TimConfig timConf;
	  timConf = config->getTimConfig(ucid().partition(), ucid().crate());

	  if(timConf.validMask & 0x18) { // One of triggerEnablesInternal or triggerEnablesTTC
	    timConf.triggerEnablesInternal = triggerEnablesInternal;
	    timConf.triggerEnablesTTC = triggerEnablesTTC;
	  }
	} catch(SctConfiguration::ConfigurationException &c) {
	  //
	}

	timWriteRegister(0x18, triggerEnablesTTC);
	timWriteRegister(0x0, triggerEnablesInternal);

	std::cout << ucid() << " Is in Physics mode\n";

	currentRunMode = SCTAPI_RUN_PHYSICS;

	return true;
      }
    case SCTAPI_RUN_CALIBRATION:
      {
	// Not really much to do... ???

	timWriteRegister(0x18, 0);
	timWriteRegister( 0x0, 0);

	calib_init();

	boost::shared_ptr<PrimListWrapper> calSetupList(new PrimListWrapper(4));
	PrimBuilder::instance().writeRegister(calSetupList, EFB_CMND_0, 24, 8, 0x0);
	std::cout << "Resetting BC offset for cal\n";
	synchSendPrimListAll(calSetupList);
	std::cout << " done resetting BC offset\n";

	std::cout << ucid() << " Is in calibration mode\n";

	currentRunMode = SCTAPI_RUN_CALIBRATION;

	return true;
      }
    }
    return false;
  }

  bool SctApi::changeEventFormat(enum EventFormatType format) {
    boost::shared_ptr<PrimListWrapper> formatPrimList(new PrimListWrapper(4));
    switch(format) {
    case SCTAPI_EVENT_FORMAT_NORMAL:
      {      
	for(int f=0; f<8; f++) {
	  PrimBuilder::instance().writeRegister(formatPrimList, FMT_EXP_MODE_EN(f), 0, 12, 0x0);
	  PrimBuilder::instance().writeRegister(formatPrimList, FMT_CONFIG_MODE_EN(f), 0, 12, 0x0);
	}
	break;
      }

    case SCTAPI_EVENT_FORMAT_EXPANDED:
      {      
	for(int f=0; f<8; f++) {
	  PrimBuilder::instance().writeRegister(formatPrimList, FMT_EXP_MODE_EN(f), 0, 12, 0xfff);
	  PrimBuilder::instance().writeRegister(formatPrimList, FMT_CONFIG_MODE_EN(f), 0, 12, 0x0);
	}
	break;
      }

    case SCTAPI_EVENT_FORMAT_RAW:
      {      
	for(int f=0; f<8; f++) {
	  PrimBuilder::instance().writeRegister(formatPrimList, FMT_EXP_MODE_EN(f), 0, 12, 0x0);
	  PrimBuilder::instance().writeRegister(formatPrimList, FMT_CONFIG_MODE_EN(f), 0, 12, 0xfff);
	}
	break;
      }
    }
    synchSendPrimListAll(formatPrimList);
  }
}
