#include "Trigger.h"
#include "RodCrate/TimDefine.h"
#include <iostream>
#include <sstream>
#include "SctApiException.h"
#include "CommonWithDsp/Sct/ABCDchip.h"

namespace SctApi{
  Trigger::~Trigger(){}


  int Trigger::getUniqueSlowCommandType(const ::SctApi::Trigger::RODTriggers& rTrigs){
    unsigned slow_command_type=0;
    for (::SctApi::Trigger::RODTriggers::const_iterator i=rTrigs.begin(); 
	 i!=rTrigs.end(); ++i){
      switch ((*i).first) {
      case CALIBRATION_PULSE :  
      case PULSE_INPUT_REG :
      case ENABLE_DATA_TAKING : {
	if (slow_command_type && slow_command_type!=(*i).first){
	  std::ostringstream message;
	  message << "Invalid TIM trigger. Contains more than one type of slow command, first " 
		  << slow_command_type << " and then later " << (*i).first;
	  throw SctApiException(message.str());
	}
	slow_command_type=(*i).first;
      }
      default: continue;
      }
    }
    return slow_command_type;
  }

  // AJB 7/8/05 this is based on the information in 
  // http://www.hep.ucl.ac.uk/~jbl/SCT/TIM_registers.html#RAM
  ::SctApi::Trigger::TIMTriggers Trigger::convertToTIMTriggers(const ::SctApi::Trigger::RODTriggers& rTrigs){
    //TIM can only send a single "CAL" signal, so only a single type of slow command can be sent
    unsigned slow_command_type=0;

    using namespace SctPixelRod;
    ::SctApi::Trigger::TIMTriggers tTrigs;
    for (::SctApi::Trigger::RODTriggers::const_iterator i=rTrigs.begin(); 
	 i!=rTrigs.end(); ++i){
      switch ((*i).first) {
      case L1_TRIGGER : tTrigs.push_back(TIM_L1A); continue;
      case BC_RESET   : tTrigs.push_back(TIM_BCR); continue;
      case SOFT_RESET : tTrigs.push_back(TIM_ECR); continue;
	// Can deal with any one of the following slow commands multiple times:
      case CALIBRATION_PULSE :  
      case PULSE_INPUT_REG :
      case ENABLE_DATA_TAKING : {
	if (slow_command_type && slow_command_type!=(*i).first){
	  std::ostringstream message;
	  message << "Invalid TIM trigger. Contains more than one type of slow command, first " 
		  << slow_command_type << " and then later " << (*i).first;
	  throw SctApiException(message.str());
	}
	slow_command_type=(*i).first;
	tTrigs.push_back(TIM_CAL); 
	continue;
      }
	// for delay, and a bunch of zeros at the end:
      case DELAY      : tTrigs.resize(tTrigs.size()+(*i).second, 0); continue;
      default: {
	std::ostringstream message;
	message << "Not a valid TIM trigger type: " << (*i).first;
	throw SctApiException(message.str());
      }
      }
    }
    return tTrigs;
  }

  std::string Trigger::getNameOfRODTrigger(RODTrigger t){
    switch (t.first){
    case L1_TRIGGER: return "L1A";
    case BC_RESET: return "BCR";
    case SOFT_RESET: return "SOFT" ;
    case CALIBRATION_PULSE: return "CAL";
    case DELAY: {
      std::ostringstream name;
      name << "DELAY (" << t.second<< ")";
      return name.str();
    }
    case ENABLE_DATA_TAKING: return "ENABLE_DATA_TAKING" ;
    case PULSE_INPUT_REG: return "PULSE_INPUT_REG" ;
    default: {
      std::ostringstream name;
      name << t.first << "=UNKNOWN"; 
      return name.str();
    }
  }
}

}
