#include "../SctApiConfigCache.h"
#include "../ConfigurationUtility.h"
#include "../SctApi.h"
#include "../SctApiException.h"
#include "ConfigCache_Test.h"
#include <iostream>

namespace SctApi {
namespace Tests {

using boost::unit_test_framework::test_suite;
  using boost::shared_ptr;
using std::auto_ptr;

static SctApiConfigCache cache;
const UINT32 mid=101;
static ABCDModule* module;

void check_put(){
  module=new ABCDModule;
  shared_ptr<SctApiConfigCache::ModuleBanks> in_banks (new SctApiConfigCache::ModuleBanks);
  cache.set(mid, in_banks);
  
  auto_ptr<ABCDModule> copy =ConfigUtility::clone(*module);
  shared_ptr<ABCDModule> in_module (copy.release());
  in_banks->set(SCTAPI_BANK_PHYSICS, in_module);
}

void check_get(){
  shared_ptr<SctApiConfigCache::ModuleBanks> out_banks = cache.getFromMid(mid);
  BOOST_CHECK(out_banks.get()!=0);
  shared_ptr<ABCDModule> out_module = out_banks->get(SCTAPI_BANK_PHYSICS);
  BOOST_CHECK(out_module.get()!=0);
  BOOST_CHECK_EQUAL(ConfigUtility::diff(*out_module,*module), (unsigned)0);
}

void check_copy_throw(){
  shared_ptr<SctApiConfigCache::ModuleBanks> out_banks = cache.getFromMid(mid);
  BOOST_CHECK_THROW(out_banks->copy(SCTAPI_BANK_SCAN, SCTAPI_BANK_PHYSICS), SctApiException);
}

void check_copy1(){
  shared_ptr<SctApiConfigCache::ModuleBanks> out_banks = cache.getFromMid(mid);
  out_banks->copy(SCTAPI_BANK_PHYSICS, SCTAPI_BANK_CALIBRATION);
  out_banks->copy(SCTAPI_BANK_CALIBRATION, SCTAPI_BANK_SCAN);
  shared_ptr<ABCDModule> out_module = out_banks->get(SCTAPI_BANK_SCAN);
  BOOST_CHECK(out_module.get()!=0);
  BOOST_CHECK_EQUAL(ConfigUtility::diff(*out_module,*module), (unsigned)0);
}

void check_copy2(){
  shared_ptr<SctApiConfigCache::ModuleBanks> out_banks = cache.getFromMid(mid);
  BOOST_CHECK(out_banks.get());
  out_banks->get(SCTAPI_BANK_SCAN)->chip[4].trim[2]=20;

  cache.copyAllModules(SCTAPI_BANK_SCAN, SCTAPI_BANK_PHYSICS);
  BOOST_CHECK_EQUAL(out_banks->get(SCTAPI_BANK_SCAN), 
		    out_banks->get(SCTAPI_BANK_PHYSICS));
  
  out_banks->get(SCTAPI_BANK_SCAN)->chip[3].trim[0]=43;
  std::list<BankType> bank_list;
  bank_list.push_back(SCTAPI_BANK_PHYSICS);
  bank_list.push_back(SCTAPI_BANK_CALIBRATION);
  cache.copyAllModules(SCTAPI_BANK_SCAN, bank_list);
  
  BOOST_CHECK_EQUAL(out_banks->get(SCTAPI_BANK_SCAN), 
		    out_banks->get(SCTAPI_BANK_PHYSICS));

  BOOST_CHECK_EQUAL(out_banks->get(SCTAPI_BANK_SCAN), 
		    out_banks->get(SCTAPI_BANK_CALIBRATION));
}

ConfigCache_Test::ConfigCache_Test() {
  // order is important!
  add(BOOST_TEST_CASE(&check_put));
  add(BOOST_TEST_CASE(&check_get));
  add(BOOST_TEST_CASE(&check_copy_throw)); 
  add(BOOST_TEST_CASE(&check_copy1));
  add(BOOST_TEST_CASE(&check_copy2));
}

}
}
