#include "../ConfigurationUtility.h"
#include "ConfigurationUtility_Test.h"
#include <iostream>
namespace SctApi {
namespace Tests {

using boost::unit_test_framework::test_suite;
using std::auto_ptr;

ABCDBasic create_basic(){
  ABCDBasic t;
  const unsigned size=sizeof(ABCDBasic)/sizeof(char);
  char *ptr = reinterpret_cast<char*>(&t);
  for (unsigned i=0; i<size; ++i){
    ptr[i]=0;
  }
  return t;
}

void check_equality_config(){
  using namespace std;
  BOOST_CHECK_EQUAL(sizeof(ABCDConfig),sizeof(UINT32));
  ABCDConfig a;
  ABCDConfig b;

  // zero everyting:
  UINT32* ptra=reinterpret_cast<UINT32*>(&a);
  UINT32* ptrb=reinterpret_cast<UINT32*>(&b);
  *ptra=0;
  *ptrb=0;

  BOOST_CHECK_EQUAL( *ptra, *ptrb);
  BOOST_CHECK_EQUAL( ConfigUtility::diff(a,b), (unsigned)0 );
  a.padding=1;
  b.padding=0;

  BOOST_CHECK(*ptra != *ptrb);
  BOOST_CHECK( ConfigUtility::diff(a,b)==0 );
  a.edgeDetect=0;
  b.edgeDetect=1;
  BOOST_CHECK(*ptra != *ptrb);
  BOOST_CHECK_EQUAL( ConfigUtility::diff(a,b), 1<<SCTAPI_CONFIG_CFG);

  b.edgeDetect=0;
   BOOST_CHECK_EQUAL( ConfigUtility::diff(a,b), (unsigned)0 );
  b.trimRange=3;
  BOOST_CHECK_EQUAL( ConfigUtility::diff(a,b), 1<<SCTAPI_CONFIG_CFG);
}

void check_basic1(){
  ABCDBasic a= create_basic();
  ABCDBasic b= create_basic();
  BOOST_CHECK_EQUAL(ConfigUtility::diff(a,b), (unsigned)0);
}
void check_basic2(){
  ABCDBasic a= create_basic();
  ABCDBasic b= create_basic();
  BOOST_CHECK_EQUAL(ConfigUtility::diff(a,b), (unsigned)0);
  a.vthr=3;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(a,b), 1<<SCTAPI_CONFIG_BASIC);
}
void check_basic3(){
  ABCDBasic a= create_basic();
  ABCDBasic b= create_basic();
  BOOST_CHECK_EQUAL(ConfigUtility::diff(a,b), (unsigned)0);
  a.config.end=1;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(a,b), 1<<SCTAPI_CONFIG_CFG);
}
void check_basic4(){
  ABCDBasic a= create_basic();
  ABCDBasic b= create_basic();
  BOOST_CHECK_EQUAL(ConfigUtility::diff(a,b), (unsigned)0);
  a.mask[0]=5;
  a.mask[2]=1;
  a.mask[3]=3;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(a,b), 1<<SCTAPI_CONFIG_MASK);
}

void check_clone_basic(){
  ABCDBasic a= create_basic();
  a.mask[0]=5;
  a.mask[2]=1;
  a.mask[3]=3;
  a.config.end=1;
  auto_ptr<ABCDBasic> b= ConfigUtility::clone(a);
  BOOST_CHECK_EQUAL(ConfigUtility::diff(a,*b), (unsigned)0);
}

ABCDCaldata create_caldata(){
  ABCDCaldata a;
  const unsigned size=sizeof(ABCDCaldata)/sizeof(char);
  char *ptra = reinterpret_cast<char*>(&a);
  for (unsigned i=0; i<size; ++i){
    ptra[i]=0;
  }
  return a;
}

void check_cal1(){
  ABCDCaldata a=create_caldata();
  ABCDCaldata b=create_caldata();
  BOOST_CHECK(ConfigUtility::diff(a,b)==0);
  a.unused[0]=5;
  a.unused[1]=6;
  a.unused[2]=7;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(a,b), (unsigned)0);
}

void check_cal2(){
  ABCDCaldata a=create_caldata();
  ABCDCaldata b=create_caldata();
  a.rc_function=1;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(a,b), 1<<SCTAPI_CONFIG_OTHER);
}

void check_cal3(){
  ABCDCaldata a=create_caldata();
  ABCDCaldata b=create_caldata();
  a.rc_params[2]=1.3;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(a,b), 1<<SCTAPI_CONFIG_OTHER);
}

void check_cal4(){
  ABCDCaldata a=create_caldata();
  ABCDCaldata b=create_caldata();
  a.c_factor=30.;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(a,b), 1<<SCTAPI_CONFIG_OTHER);
}

ABCDModule create_module(){
  ABCDModule a;
  const unsigned size=sizeof(ABCDModule)/sizeof(char);
  char *ptra = reinterpret_cast<char*>(&a);
  for (unsigned i=0; i<size; ++i){
    ptra[i]=0;
  }
  return a;
}

void check_module1(){
  ABCDModule ma = create_module();
  ABCDModule mb = create_module();
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma,mb), (unsigned)0);
  ma.unused1[1]=4;
  ma.unused2[3]=5;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma,mb), (unsigned)0);
}


void check_module2(){
  ABCDModule ma = create_module();
  ABCDModule mb = create_module();
  ma.active=1;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma,mb), 1<<SCTAPI_CONFIG_OTHER);
}
  
void check_module3(){
  ABCDModule ma = create_module();
  ABCDModule mb = create_module();
  mb.groupId=2;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma,mb), 1<<SCTAPI_CONFIG_OTHER);
}

void check_module4(){
  ABCDModule ma = create_module();
  ABCDModule mb = create_module();
  mb.pTTC=20;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma,mb), 1<<SCTAPI_CONFIG_OTHER);
}

void check_module5(){
  ABCDModule ma = create_module();
  ABCDModule mb = create_module();
  mb.rx[1]=3;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma,mb), 1<<SCTAPI_CONFIG_OTHER);
}

void check_clone_module1(){
  ABCDModule ma = create_module();
  auto_ptr<ABCDModule> mb = ConfigUtility::clone(ma);
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma,*mb), (unsigned)0);
}

void check_clone_module2(){
  ABCDModule ma = create_module();
  ma.rx[1]=3;
  ma.pTTC=20;
  auto_ptr<ABCDModule> mb = ConfigUtility::clone(ma);
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma,*mb), (unsigned)0);
}

void check_differ1(){
  ABCDModule ma = create_module();
  auto_ptr<ABCDModule> mb = ConfigUtility::clone(ma);
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma,*mb), (unsigned)0);
  ma.pTTC=21;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma, *mb), 1<<SCTAPI_CONFIG_OTHER);
}

void check_differ2(){
  ABCDModule ma = create_module();
  auto_ptr<ABCDModule> mb = ConfigUtility::clone(ma);
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma,*mb), (unsigned)0);
  ma.chip[3].trim[1]=3;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma, *mb), 1<<SCTAPI_CONFIG_TRIM);
}

void check_differ3(){
  ABCDModule ma = create_module();
  auto_ptr<ABCDModule> mb = ConfigUtility::clone(ma);
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma, *mb), (unsigned)0);
  ma.chip[3].basic.vcal=3;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma, *mb), 1<< SCTAPI_CONFIG_BASIC);
}

void check_differ4(){
  ABCDModule ma = create_module();
  auto_ptr<ABCDModule> mb = ConfigUtility::clone(ma);
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma,*mb), (unsigned)0);

  ma.chip[3].basic.vcal=3;
  ma.chip[2].trim[1]=3;
  unsigned mask = 1<<SCTAPI_CONFIG_BASIC & SCTAPI_CONFIG_TRIM;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma, *mb), mask);

  ma.chip[1].basic.config.edgeDetect=1;
  mask |= 1<< SCTAPI_CONFIG_CFG;
  BOOST_CHECK_EQUAL(ConfigUtility::diff(ma, *mb), mask);
}

void check_trig_types(){
  BOOST_CHECK_EQUAL(ConfigUtility::variableType(ST_TRIG_DELAY1),ConfigUtility::TRIGGER_VAR);
}

ConfigurationUtility_Test::ConfigurationUtility_Test() {
  add(BOOST_TEST_CASE(&check_equality_config));
  add(BOOST_TEST_CASE(&check_basic1));
  add(BOOST_TEST_CASE(&check_basic2));
  add(BOOST_TEST_CASE(&check_basic3));
  add(BOOST_TEST_CASE(&check_basic4));
  add(BOOST_TEST_CASE(&check_clone_basic));
  add(BOOST_TEST_CASE(&check_cal1));
  add(BOOST_TEST_CASE(&check_cal2));
  add(BOOST_TEST_CASE(&check_cal3));
  add(BOOST_TEST_CASE(&check_cal4));
  add(BOOST_TEST_CASE(&check_module1));
  add(BOOST_TEST_CASE(&check_module2));
  add(BOOST_TEST_CASE(&check_module3));
  add(BOOST_TEST_CASE(&check_clone_module1));
  add(BOOST_TEST_CASE(&check_clone_module2));
  add(BOOST_TEST_CASE(&check_differ1));
  add(BOOST_TEST_CASE(&check_differ2));
  add(BOOST_TEST_CASE(&check_differ3));
  add(BOOST_TEST_CASE(&check_trig_types));
}

}
}
