// problems: if runnin in burst, random mode with FFT veto
// - how to poll for end?
// - how to check for vetos?

void doubleTriggerBurstSequence(bool burst=true, bool random=false){
 if (random && !burst) {
    printf("random only allowed in burst mode\n");
    return;
 } 
printf("starting double trigger sequence:, burst %b, random %b\n",burst,random);

const int length=300;
const unsigned long ntrigs=100000;

int* ttrigs = new int[length];
printf ("allocated ttrigs[%d]\n",length);

for (int i=0; i<length; ++i){
  ttrigs[i]=0;
}

// add a couple of L1As:
ttrigs[0]=0x01;
ttrigs[130]=0x01;
// sequence end


// reset sequence control - need to do this before loading the sequence
tapi.timRegLoad(0x1a,0x0200);
tapi.timRegLoad(0x1a,0x0);

// load the sequence
tapi.timRegLoad(0x1C,length-1);
for (int i=0; i<length; ++i){
  tapi.timRegLoad(0x8000 + i*2, ttrigs[i]);
  int in = tapi.timReadRegister(0x8000 + i*2);
  //printf("in[%d]=0x%x (%x)\n",i,in,trigs[i]);
}

// enables
if (!random){
  tapi.timRegLoad(0x0,0xFDFD);
  tapi.timRegLoad(0x18,0xFFFD);
}

// L1 counters
tapi.timRegLoad(0x12,0x0); 
tapi.timRegLoad(0x10,0x0);

printf("Using burst register? %b\n",burst);

if (burst){
  // set the burst counter
  tapi.timRegLoad(0x04, (ntrigs & 0xffff ) );
  tapi.timRegLoad(0x8e, (ntrigs >> 16) );

  // set burst mode
  tapi.timRegLoad(0x02,0x200);

  if (random) {
    // trigger, go
    tapi.timRegLoad(0x1a,0x0401);
    tapi.timRegLoad(0x42,0x100);
    tapi.timRegLoad(0x00,0x12);
  }else{
    // trigger, cyclic, go
    tapi.timRegLoad(0x1a,0x0C01);
  }

  // set burst go + burst mode
  tapi.timRegLoad(0x02,0x600);

}else{
  // set up sequencer in trigger, cyclic go mode
  tapi.timRegLoad(0x1a,0x0C01);
}

unsigned long l1id=0;
unsigned long poll=0;
unsigned long unchanged_polls=0;
bool stalled=false;
unsigned long lastl1id=-1;
do{
  poll++;
  l1id = ((tapi.timReadRegister(0x12)<<16) + (tapi.timReadRegister(0x10)));

  //intermittent diagnostics
  if (poll%(ntrigs/10)==0){
    printf("%d of %d sent \n",l1id,ntrigs,lastl1id);

    // check if l1id has stopped increasing
    if (l1id==lastl1id) {
      unchanged_polls++;
      if (unchanged_polls>10) stalled=true;
    }else{
      unchanged_polls=0;
    }
  }
  lastl1id=l1id;
} while ((l1id<ntrigs) && !stalled);

if (stalled) printf("stalled\n");
const bool incomplete=(burst && l1id!=ntrigs);
if (incomplete){
  printf("Incorrect number of triggers sent - %d rather than %d\n",l1id,ntrigs);
}

if (incomplete || stalled) {
  for (unsigned reg=0; reg<0x1F; ++reg){
     printf ("reg 0x%x=0x%x\t",reg,tapi.timReadRegister(reg));
     if (reg%4==0) printf("\n");
  }
  printf("\n\n");
}

// stop the sequence, reset other registers
tapi.timRegLoad(0x1a,0x0);
tapi.timRegLoad(0x02,0x0);
tapi.timRegLoad(0x04,0x0);
tapi.timRegLoad(0x8e,0x0);
if (random) tapi.timRegLoad(0x00,0x0);
if (burst)  tapi.timRegLoad(0x02,0x0);

unsigned outl1id = tapi.timReadRegister(0x10) + (tapi.timReadRegister(0x12)<<16);
printf("number of l1ids sent was %d \n\n",outl1id);


for (int i=0; i<length; ++i){
  int out = tapi.timReadRegister(0x8000 + i*2);
  if (out==0){
    printf(".");
  }else if (out==0xffff){
    printf("x");
  }else{
    printf("\nOutput[%d]=0x%x\n",i,out);
  }
}
printf("\n");

delete[] ttrigs;

} // end of macro


/**
Hi Alan,

Send a burst of N-trigs with the sequencer as the source:
 >  - load the burst count registers
Set BurstMode timRegLoad(0x02,0x200)
Set Sequencer going:
 > timRegLoad(0x1a,0x0401)
Set BurstGo + BurstMode timRegLoad(0x02,0x600)
Wait until burst done
Clear BurstGo, (and set it again if you want another burst)

I'll write a receipe for sending a burst of random sequencer double trigs 
tomorrow!

cheers
Matt
*/
