#ifndef SCTAPI_UTILITY_H
#define SCTAPI_UTILITY_H

#include <iostream>
#include <boost/shared_array.hpp>

#include "SctApiFwd.h"

#include "processor.h"
#include "registerIndices.h"
#include <list>

namespace SctApi {
  namespace Utility {
    inline UINT32 EfbErrorMask(UINT32 efb, UINT32 link){
#ifdef EFB_ERROR_MASK
      return EFB_ERROR_MASK(efb, link);
#elif (defined(ERROR_MASK))
      return ERROR_MASK(efb, link)
#else
#error "Can't work out how to find an error mask - cant compile"
#endif
    }

    inline void getpcrc(UINT32 mid, unsigned int &partition, unsigned int &crate, unsigned int &rod, unsigned &channel) 
    {
      partition = (mid & 0xff000000) >> 24;
      crate = (mid & 0xff0000) >> 16;
      rod = (mid & 0xff00) >> 8;
      channel = (mid & 0xff) >> 0;
    }

    inline UINT32 packpcrc(unsigned int &partition, unsigned int &crate, unsigned int &rod, unsigned &channel)
    {
      UINT32 mid = 0;
      mid |= (partition & 0xff) << 24;
      mid |= (crate & 0xff) << 16;
      mid |= (rod & 0xff) << 8;
      mid |= (channel & 0xff) << 0;
      return mid;
    }

    inline void printHex(UINT32 num, int width) {
      std::cout << "0x";
      char saveFill = std::cout.fill('0');
      std::cout.width(width);
      std::cout.setf(std::ios::right);
      std::cout << std::hex << num << std::dec;
      std::cout.fill(saveFill);
    }

    void printMemoryBlock(std::ostream &os, unsigned long *mem, unsigned long words, int wordsPerLine, int level = 0);

    int translateBank(BankType bank);

    UINT32 translateApiBanksToRodCfgBitSet(std::list<BankType> banks);

    /** Translate single SctApi::ConfigType into config type for get, set and copy primitives */
    unsigned translateConfigType(ConfigType cfg);

    /** Translate single SctApi::ConfigType into config type for send primitives */
    unsigned translateConfigTypeForSendPrimitive(ConfigType cfg);

    /** Translate chip from 0->11 format into M0 -> E13 format */
    unsigned translateChip(unsigned chip);

    /** Translate bitset of multiple SctApi::ConfigType's 
	into bitset of config type for get, set and copy primitives */
    UINT32 translateConfigTypeBitset(UINT32 api_bits, bool verbose=false);

    /** 
	Only some sub-structures of ABCDModule can actually be set as primitives.
	This function returns the smallest setable sub-structure bigger than the 
	the parts you want to send.
     */
    UINT32 getSetableConfigType(UINT32 cfgType, bool verbose=false);

    class MemoryBlock {
      unsigned int words;
      boost::shared_array<unsigned long> buffer;

    public:
      MemoryBlock() : words(0), buffer(0) {}
      MemoryBlock(unsigned int size, unsigned long *pointer) : words(size), buffer(pointer) {}

      operator bool() { return buffer; }

      unsigned int size() { return words; } 

      unsigned long operator[] (unsigned int i) {return buffer[i]; }

      unsigned long *address() {return buffer.get(); }
    };
  }
}

#endif
