
#include "CrateStore.h"
#include "Sct/OmniMarshalling.h"

namespace Sct {

  void CrateStore::registerNewCrateController(Sct_SctApi::CrateIPC_ptr ccTmp)
  {
    boost::recursive_mutex::scoped_lock lk(m_apisMutex);

    Sct_SctApi::CrateIPC_var cc = Sct_SctApi::CrateIPC::_duplicate(ccTmp);

    const Sct::UCID ucid = copyCorbaToUCID(cc->getIdiosyncrasy()->ucid()); 

    this->m_apis.push_front(cc);
    this->m_mapUCIDToWorkersIterator[ucid] = m_apis.begin();
  }

  CrateStore::Worker CrateStore::getCrate(const Sct::UCID & ucid) {
    Map::iterator it = m_mapUCIDToWorkersIterator.find(ucid);
    if (it==m_mapUCIDToWorkersIterator.end()) {
      return WorkerBase::_nil();
    };
    return *(it->second);
  }

  CrateStore::Worker CrateStore::getCrate(const Sct::Corba::UCID & ucid) {
    return this->getCrate(copyCorbaToUCID(ucid));
  }
  
  CrateStore::Worker CrateStore::getCrate(const Sct::URID & urid) {
    return this->getCrate(urid.ucid());
  }

  CrateStore::Worker CrateStore::getCrate(const Sct::Corba::URID & urid) {
    return this->getCrate(copyCorbaToURID(urid));
  }

  CrateStore::UCIDList CrateStore::ucidList() {
    boost::recursive_mutex::scoped_lock lk(m_apisMutex);
    UCIDList ret;
    for (Workers::const_iterator wit = m_apis.begin();
	 wit != m_apis.end();
	 ++wit) {
      Sct_SctApi::Idiosyncrasy_var id = (*wit)->getIdiosyncrasy();
      ret.push_back(copyCorbaToUCID( id->ucid() ));
    };
    return ret;
  };

  CrateStore::Workers & CrateStore::workers() {
    return m_apis;
  }

}
