
#include "ScanWrapper.h"
#include "Sct_SctApi/SctApi.hh"
//#include "RODTriggerTranslation.h"
#include "TriggerWrapper.h"
#include "TriggerServer.h"
#include "Sct/SequenceUtils.h"
#include "Converter.h"
#include <boost/date_time/posix_time/posix_time.hpp>

const boost::shared_ptr< ::SctApi::Trigger> ScanWrapper::getTrigger1() const { 
  Sct_SctApi::Trigger_ptr t_ptr = corbaScanServer()->getTrigger1(); 

  boost::shared_ptr< ::SctApi::Trigger> triggerWrapper 
    = boost::shared_ptr< ::SctApi::Trigger>( new TriggerWrapper(t_ptr, TriggerWrapper::doNotTakeOwnershipOfObjectReference));

  return triggerWrapper;
};
const boost::shared_ptr< ::SctApi::Trigger> ScanWrapper::getTrigger2() const { 
  Sct_SctApi::Trigger_ptr t_ptr = corbaScanServer()->getTrigger2(); 

  boost::shared_ptr< ::SctApi::Trigger> triggerWrapper 
    = boost::shared_ptr< ::SctApi::Trigger>( new TriggerWrapper(t_ptr, TriggerWrapper::doNotTakeOwnershipOfObjectReference));

  return triggerWrapper;
};

boost::shared_ptr< ::SctApi::Trigger> ScanWrapper::getTrigger1() { 
  Sct_SctApi::Trigger_ptr t_ptr = corbaScanServer()->getTrigger1(); 

  boost::shared_ptr< ::SctApi::Trigger> triggerWrapper 
    = boost::shared_ptr< ::SctApi::Trigger>( new TriggerWrapper(t_ptr, TriggerWrapper::doNotTakeOwnershipOfObjectReference));

  return triggerWrapper;
};
boost::shared_ptr< ::SctApi::Trigger> ScanWrapper::getTrigger2() { 
  Sct_SctApi::Trigger_ptr t_ptr = corbaScanServer()->getTrigger2(); 

  boost::shared_ptr< ::SctApi::Trigger> triggerWrapper 
    = boost::shared_ptr< ::SctApi::Trigger>( new TriggerWrapper(t_ptr, TriggerWrapper::doNotTakeOwnershipOfObjectReference));

  return triggerWrapper;
};

void ScanWrapper::setTrigger1(boost::shared_ptr< ::SctApi::Trigger> trigger) { 
  TriggerServer triggerServer(trigger);
  Sct_SctApi::Trigger_var triggerVar = triggerServer._this();
  corbaScanServer()->setTrigger1(triggerVar);
};
void ScanWrapper::setTrigger2(boost::shared_ptr< ::SctApi::Trigger> trigger) { 
  TriggerServer triggerServer(trigger);
  Sct_SctApi::Trigger_var triggerVar = triggerServer._this();
  corbaScanServer()->setTrigger2(triggerVar);
};

const ::SctApi::Scan::TrigPoints ScanWrapper::getVariableTrigs() const {
  const Sct_SctApi::Scan::TrigPoints * const trigPoints = corbaScanServer()->getVariableTrigs();
  const unsigned int siz = trigPoints->length();

  ::SctApi::Scan::TrigPoints ans(siz);
  for (unsigned int i=0; i<siz; ++i) {
    ans[i] = (*trigPoints)[i];
  };

  delete trigPoints;

  return ans;
};
void ScanWrapper::setVariableTrigs(const ::SctApi::Scan::TrigPoints &scans) {
  const unsigned int siz = scans.size();

  Sct_SctApi::Scan::TrigPoints tmp = ::Sct::SequenceUtils::safeSequence<Sct_SctApi::Scan::TrigPoints>(siz);
  for (unsigned int i=0; i<siz; ++i) {
    tmp[i] = scans[i];
  };
  corbaScanServer()->setVariableTrigs(tmp);
};

void ScanWrapper::setVariableTrigRange(unsigned short start, unsigned short end, unsigned long value) {
  corbaScanServer()->setVariableTrigRange(start, end, value);
};


const ::SctApi::Scan::ScanPoints ScanWrapper::getScanPoints1() const {
  const Sct_SctApi::Scan::ScanPoints * const tmp = corbaScanServer()->getScanPoints1();
  const unsigned int siz = tmp->length();
  ::SctApi::Scan::ScanPoints ans(siz);
  for (unsigned int i=0; i<siz; ++i) {
    ans[i] = (*tmp)[i];
  };
  delete tmp;
  return ans;
};
const ::SctApi::Scan::ScanPoints ScanWrapper::getScanPoints2() const {
  const Sct_SctApi::Scan::ScanPoints * const tmp = corbaScanServer()->getScanPoints2();
  const unsigned int siz = tmp->length();
  ::SctApi::Scan::ScanPoints ans(siz);
  for (unsigned int i=0; i<siz; ++i) {
    ans[i] = (*tmp)[i];
  };
  delete tmp;
  return ans;
};

void ScanWrapper::setScanPoints1(const ScanPoints &scans) {
  const unsigned int siz = scans.size();

  Sct_SctApi::Scan::ScanPoints tmp = ::Sct::SequenceUtils::safeSequence<Sct_SctApi::Scan::ScanPoints>(siz);
  for (unsigned int i=0; i<siz; ++i) {
    tmp[i] = scans[i];
  };
  corbaScanServer()->setScanPoints1(tmp);
};
void ScanWrapper::setScanPoints2(const ScanPoints &scans) {
  const unsigned int siz = scans.size();

  Sct_SctApi::Scan::ScanPoints tmp = ::Sct::SequenceUtils::safeSequence<Sct_SctApi::Scan::ScanPoints>(siz);
  for (unsigned int i=0; i<siz; ++i) {
    tmp[i] = scans[i];
  };
  corbaScanServer()->setScanPoints2(tmp);
};

int ScanWrapper::getOption(ScanOptionsEnum scanOption) const {
  return corbaScanServer()->getOption(SctApiIPC::Converter::scanOptionsEnumToCorba(scanOption));
};
void ScanWrapper::setOption(ScanOptionsEnum scanOption, int option) {
  corbaScanServer()->setOption(SctApiIPC::Converter::scanOptionsEnumToCorba(scanOption), option);
};
void ScanWrapper::setScanNumber(unsigned int scan) {
  corbaScanServer()->setScanNumber(scan);
};
unsigned int ScanWrapper::getScanNumber() const {
  return corbaScanServer()->getScanNumber();
};
void ScanWrapper::setRunNumber(unsigned int scan) {
  corbaScanServer()->setRunNumber(scan);
};
unsigned int ScanWrapper::getRunNumber() const {
  return corbaScanServer()->getRunNumber();
};

#if !USE_CONST_SCAN
std::list<std::string> ScanWrapper::getModuleList(unsigned int group) const {
  std::list<std::string> ans;

  const sctConf::ModuleList * const tmp = corbaScanServer()->getModuleList(group);
  const unsigned int siz = tmp->length(); 

  for(unsigned int i=0; i<siz; ++i) {
    ans.push_back(   ((*tmp)[i])._NP_ref() );
  };

  delete tmp;
  return ans;
};
void ScanWrapper::setModuleList(unsigned int group, const ::SctApi::ModuleList & newList) {
  const unsigned int siz = newList.size();
  sctConf::ModuleList tmp = ::Sct::SequenceUtils::safeSequence<sctConf::ModuleList>(siz);
  ::SctApi::ModuleList::const_iterator it = newList.begin();
  for (unsigned int i=0; i<siz; (++i,++it)) {
    const std::string part = (*it);
    tmp[i] = CORBA::string_dup(part.c_str());
  };

  corbaScanServer()->setModuleList(group, tmp);
};

unsigned int ScanWrapper::getNGroups() const {
  return corbaScanServer()->getNGroups();
};
void ScanWrapper::setNGroups(unsigned int val) {
  corbaScanServer()->setNGroups(val);
};
  
boost::posix_time::ptime ScanWrapper::getStartTime() const {
  const char * const corbaTim = corbaScanServer()->getStartTime();
  boost::posix_time::ptime pt = boost::posix_time::from_iso_string(corbaTim);
  delete corbaTim;
  return pt;  
};
void ScanWrapper::setStartTime(boost::posix_time::ptime t) {
  corbaScanServer()->setStartTime(boost::posix_time::to_iso_string(t).c_str());
};
  
boost::posix_time::ptime ScanWrapper::getEndTime() const {
  const char * const corbaTim = corbaScanServer()->getEndTime();
  boost::posix_time::ptime pt = boost::posix_time::from_iso_string(corbaTim);
  delete corbaTim;
  return pt;  
};
void ScanWrapper::setEndTime(boost::posix_time::ptime t) {
  corbaScanServer()->setEndTime(boost::posix_time::to_iso_string(t).c_str());
};
#endif

std::string ScanWrapper::print() const {
    CORBA::String_var output=m_worker->print();
    std::string output_as_string (output);
    return output_as_string; 

};
