
#ifndef SCT_SCTAPIIPC_SCTAPIDDCSERVER_H
#define SCT_SCTAPIIPC_SCTAPIDDCSERVER_H

//fwddec
namespace SctApiDDC {
  class SctApiDDCServer;
};

// includes
#include <boost/shared_ptr.hpp>
#include <ipc/object.h>
#include <ipc/server.h>
#include "Sct_SctApi/SctApi.hh"

//other fwddec
namespace SctApi {
  class SctApiDDC;
};

//defn
namespace SctApiDDC {

  class SctApiDDCServer : public IPCNamedObject<POA_Sct_SctApi::SctApiDDCServer>,
                          public IPCServer {
  private:
    CORBA::Short m_ddcSingleTimeout;
    CORBA::Short m_ddcMultipleTimeout;
    boost::shared_ptr<SctApi::SctApiDDC> m_worker;
    SctApi::SctApiDDC * worker();
  public:
    /// Constructor
    SctApiDDCServer();
  public:
    /// CORBA Methods:
    virtual void requestHardReset (CORBA::ULong mid);
    virtual void requestIVCurve (CORBA::ULong mid, CORBA::Float start, CORBA::Float stop, CORBA::Float step, CORBA::UShort delay, CORBA::Float currentLim);
    virtual void setSelect (CORBA::ULong mid, CORBA::Boolean value);
    virtual void setSelectAllInCrate (CORBA::Long crate, CORBA::Boolean value);
    virtual void changeRunState (CORBA::Long state);
    virtual void ddcSetSingleTimeout (CORBA::Short timeout);
    virtual CORBA::Short ddcGetSingleTimeout ();
    virtual void ddcSetMultipleTimeout (CORBA::Short timeout);
    virtual CORBA::Short ddcGetMultipleTimeout ();
    virtual void ddcChangeState (CORBA::Long crate, CORBA::Long channel, CORBA::Short state);
    virtual void ddcChangeStates (CORBA::Long crate, const char* name, CORBA::Short state);
    virtual void ddcChangeStateAll (CORBA::Long crate, CORBA::Short state);
    virtual void ddcLoadConfiguration (CORBA::Long crate, CORBA::Short state);
    virtual void ddcHardReset (CORBA::Long crate, CORBA::Long channel);
    virtual void ddcHardResetAll (CORBA::Long crate);
    virtual CORBA::ULong ddcGetChannelParameterInt (CORBA::Long crate, CORBA::Long channel, const char * name);
    virtual CORBA::Float ddcGetChannelParameterFloat (CORBA::Long crate, CORBA::Long channel, const char * name);
    virtual char * ddcGetChannelParameterString (CORBA::Long crate, CORBA::Long channel, const char * name);
    virtual CORBA::ULong ddcGetCardParameterInt (CORBA::Long crate, CORBA::Long card, const char * name);
    virtual CORBA::Float ddcGetCardParameterFloat (CORBA::Long crate, CORBA::Long card, const char * name);
    virtual char * ddcGetCardParameterString (CORBA::Long crate, CORBA::Long card, const char * name);
    virtual CORBA::ULong ddcGetCrateParameterInt (CORBA::Long crate, const char * name);
    virtual CORBA::Float ddcGetCrateParameterFloat (CORBA::Long crate, const char * name);
    virtual char * ddcGetCrateParameterString (CORBA::Long crate, const char * name);
    virtual void ddcGetCrateParameters (CORBA::Long crate, const Sct_SctApi::NameList & names, Sct_SctApi::NameValueList_out nameValues);
    virtual void ddcSetCrateParameterFloat(CORBA::Long crate, const char* name, CORBA::Float value);
    virtual void ddcSetCardParameterFloat(CORBA::Long crate, CORBA::Long card, const char* name, CORBA::Float value);
    virtual void ddcSetChannelParameterFloat(CORBA::Long crate, CORBA::Long channel, const char* name, CORBA::Float value);
  public:
    /// Methods for IPCServer:
    virtual void shutdown() { this->stop(); };
  };

};


#endif
