#include <ipc/core.h>
#include <ipc/object.h>
#include <ipc/server.h>
#include <pmg/pmg_initSync.h>

#include <boost/bind.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/thread.hpp>

#include "SctApi/SctApi.h"
#include "sctConfIPC/configipc.h"

#include "SctApiServer.h"
#include "Sct/ApplicationStartupDebugTools.h"
#include "Sct/MultiMessageDebugStream.h"
#include "Sct/LogicErrors.h"
#include "Sct/SctNames.h"

// Not needed AT THE MOMENT but keep for future use: void dcsPollThread(SctApi::SctApi *sctApi);

int main(int argc, char** argv) 
{
  IPCCore::init(argc, argv); // Try reentrant for DDC IS callbacks
  Sct::ApplicationStartupDebugTools::announceStartOfMain(argc, argv, __FILE__, __LINE__);

  std::cout << std::endl;
  std::cout << "Example usage:" << std::endl;
  std::cout << "     " << (argc >= 1 ? argv[0] : "SctApiServer") << " -crate UCID:2.5 -crate UCID:2.6 -crate UCID:1.4" << std::endl;
  std::cout << std::endl;

  IPCPartition p(::Sct::SctNames::getPartitionName());
  try {    

    SctApiServer::CrateDescriptors crateDescriptors;
    // Find the Crate Controllers! Eventually this set
    // of crateDescriptors should be obtained from command line 
    // args or from a database.  For the moment, hard code in a
    // default valie:

    for (int i=1; i<argc; ++i) {
      const std::string theArg = argv[i];
      
      if (theArg == "-crate") {

	++i;
	if (i<argc) {

	  const std::string theCrate = argv[i];
	  
	  try {

	    const Sct::UCID ucidForSomeCrate(theCrate);	    
	    crateDescriptors.push_back( ucidForSomeCrate );

	  } catch (const Sct::InvalidArgumentError e) {
	    Sct::MultiMessageDebugStream m(true,false,true);
	    m << "Someone tried to start SctApiServer with a bad crate id on the command line: The form used was '-crate " << theCrate << "' but it should have looked something like '-crate UCID:2.5' (your numbers may vary!). It looks like you may be starting the SctApiServer wrongly!  Check config/databases/*.data.xml"; 
	  };
	
	} else {
	  Sct::MultiMessageDebugStream m(true,false,true);
	  m << "A '-crate' command line argument has been passed to SctApiServer which was not followed by a crate identifier (for example 'UCID:2.5').  It looks like you may be starting the SctApiServer wrongly!  Check config/databases/*.data.xml";
	};

      };
    };


    if (crateDescriptors.empty()) {
      Sct::MultiMessageDebugStream m(true,false,true);
      m << "SctApiServer was started without ANY valid '-crate' command line arguments.  It uses these arguents to define which crates it should to contact.  It is hard to imagine a situation in which you might SctApiServer to contact NO crates at all, so I think you have probably messed up. Check config/databases/*.data.xml for example.  See SctApiServer's standard output for a usage message!";
    };
    
    std::auto_ptr<SctApiServer> h(new SctApiServer
                                  (crateDescriptors, 
                                   Sct_SctApi::SctApiIPC::instanceName
                                   ));
    
    // Try publishing it
    if (!h->publish()) {
      std::cout << "Failed to publish SctApi" << std::endl;
      return 0;
    } 

    std::cout << "Serving...\n";
    // The following line commented by Alan 8/11/2004 folloing
    // discussion with bruce about problems during scanning
    // if this thread runs
    // boost::thread dcsThread(boost::bind(&dcsPollThread, api.get()));
    std::cout << "sent bind to dctPollThread" << std::endl;
    pmg_initSync();
    h->run();
    std::cout << "Run over" << std::endl;
    h->withdraw();
  } catch(SctConfiguration::ConfigurationException &c) {
    std::cout << "ConfigurationException: " << c.what() << std::endl;

    MRSStream mrsStream(p);
    mrsStream << "SCTAPI_FAIL" << MRS_FATAL << MRS_QUALIF("SctApiServer") << MRS_TEXT("Configuration server not running!") << ENDM;
    
    exit(1);
  }
}

/* Whole method needs a rethink for multicrate, so commented out for the moment since it is not used anayway

void dcsPollThread(SctApi::SctApi *sctApi) {
  // Let everything settle down first
  sleep(3);

  IPCPartition partition(::Sct::SctNames::getPartitionName());

  ISInfoDictionary isDict(partition);

  while(1) {
    std::cout << "Polling BOC DCS\n";

    std::list<SctApi::RodLabel> rodList = sctApi->listRods();

    std::string isBaseName("SCTAPIServer.BOC_DCS");

    for(std::list<SctApi::RodLabel> ::const_iterator rodI = rodList.begin();
	rodI != rodList.end();
	rodI ++) {
      std::vector<double> thisMonitor 
	= sctApi->getBOCMonitorArray(rodI->partition, rodI->crate, rodI->rod);

      std::string isRodName(isBaseName
			    + "." + boost::lexical_cast<std::string>(rodI->partition) 
			    + "." + boost::lexical_cast<std::string>(rodI->crate) 
			    + "." + boost::lexical_cast<std::string>(rodI->rod));

      for(int i=0; i<12; i++) {
	std::string isName(isRodName + ".ADC" + boost::lexical_cast<std::string>(i));

	ISInfoDouble isValue(thisMonitor[i]);

	if(isDict.contains(isName.c_str())) {
	  isDict.update(isName.c_str(), isValue);
	} else {
	  isDict.insert(isName.c_str(), isValue);
	}
      }
    }

    sleep(60);
  }
}

*/
