#ifdef linux
#undef linux
#endif

#include "RunController.h"
#include <rc/RCStateInfo.h>
#include <rc/RunParams.h>
#include <rc/RunInfo.h>
#include "Sct/IoExceptions.h"
#include "Sct/SctNames.h"
#include "is/info.h"
#include <is/infodictionary.h>
#include <sstream>
#include <sys/types.h>
#include <sys/stat.h>
#include "Sct_SctApi/SctApi.hh"
#include "Sct/MultiMessageDebugStream.h"

namespace SctConfigurationController {
    
RunControl::RunControl(SctConfiguration::Configuration& c) : config(&c) {
}

RunControl::~RunControl(){
}
   
RunControl& RunControl::initialize(SctConfiguration::Configuration& s) {
    static RunControl rc(s);
    return rc;
}

bool RunControl::loadAction() {
  //config->loadConfiguration(""); // moved to calibration controller
  return true;
}


// AJB was startAction
bool RunControl::prepareAction() {
  try{
    doSave("start");  
  } catch(std::exception & e) {
    Sct::MultiMessageDebugStream m(true,true,true);
    m.severity(MRS_ERROR);
    m << "Could not save end of run configuration due to std::exception ["<<e.what()<<"] at " << __FILE__ << " " << __LINE__;
    return false;
  } catch(CORBA::Exception & e) {
    Sct::MultiMessageDebugStream m(true,true,true);
    m.severity(MRS_ERROR);
    m << "Could not save end of run configuration due to CORBA::Exception ["<<e._name()<<"] at " << __FILE__ << " " << __LINE__;
    return false;
  } catch(...) {
    Sct::MultiMessageDebugStream m(true,true,true);
    m.severity(MRS_ERROR);
    m << "Could not save end of run configuration due to unknown exception at " << __FILE__ << " " << __LINE__;
    return false;
  };
  return true;
}

  /// was exitRunning
bool RunControl::stopEFAction() {
  try{
    Sct_SctApi::SctApiIPC_var api = getApi();
    api->storeModuleConfigurations(Sct_SctApi::PHYSICS_CONFIG);
    api->storeBOCConfigurations();
    doSave("end");
  } catch(std::exception & e) {
    Sct::MultiMessageDebugStream m(true,true,true);
    m.severity(MRS_ERROR);
    m << "Could not save end of run configuration due to std::exception ["<<e.what()<<"] at " << __FILE__ << " " << __LINE__;
    return false;
  } catch(CORBA::Exception & e) {
    Sct::MultiMessageDebugStream m(true,true,true);
    m.severity(MRS_ERROR);
    m << "Could not save end of run configuration due to CORBA::Exception ["<<e._name()<<"] at " << __FILE__ << " " << __LINE__;
    return false;
  } catch(...) {
    Sct::MultiMessageDebugStream m(true,true,true);
    m.severity(MRS_ERROR);
    m << "Could not save end of run configuration due to unknown exception at " << __FILE__ << " " << __LINE__;
    return false;
  };
  return true;
}

Sct_SctApi::SctApiIPC_var RunControl::getApi(){
  unsigned count=0;
  bool good=false;
  Sct_SctApi::SctApiIPC_var api;
  while (!good && count<5){
    if ( api==0 || CORBA::is_nil(api) || api->_non_existent()){
      api=Sct::SctNames::getPartition().lookup<Sct_SctApi::SctApiIPC>(Sct_SctApi::SctApiIPC::instanceName);
    }else{
      good=true;
    }
    ++count;
  }
  if (good) return api;
  throw Sct::ConfigurationException("Could not get SctApi", __FILE__, __LINE__);
}

void RunControl::doSave(const std::string& suffix){
  RunParams runInfo;
  ISInfo::Status status = Sct::SctNames::getISDictionary().findValue("RunParams.RunParams", runInfo);
  if (status != ISInfo::Success) {
    throw Sct::IsException(status, "Could not get RunParams, no save of configuration!", __FILE__, __LINE__);
  }

  std::ostringstream oss;
  oss << Sct::SctNames::getPersistentDir() << "/configArchive_" 
      << runInfo.run_number << "_" << suffix << ".xml.gz";
  config->saveConfiguration(oss.str());
  chmod(oss.str().c_str(), S_IRUSR | S_IRGRP);

  std::ostringstream message;
  message << "Save [" <<suffix << "]";
  Sct::SctNames::Mrs() << "SCT_CONFIG_SAVE" << MRS_INFORMATION << MRS_QUALIF("sctConf") << MRS_TEXT(message.str().c_str()) << MRS_PARAM<const char*>("Filename", oss.str().c_str()) << ENDM;
  
}

}
