#include <stdlib.h>
#include <ipc/object.h>
#include <ipc/server.h>
#include <ipc/core.h>
#include <boost/lexical_cast.hpp>

#include "Sct/SctNames.h"

#include "sctConfIPC/configuration.hh"
#include "configArchiveServer.h"
#include "configserver.h"

using namespace SctConfiguration;
using namespace Sct;
using namespace boost;

using namespace std;

IPCServer* ipcServer;

ConfigArchiveServer::ConfigArchiveServer(IPCPartition& p, const char* name) :
  IPCNamedObject<POA_sctConf::ConfigurationArchive, ipc::single_thread>(p, name)
{
}

ConfigArchiveServer::~ConfigArchiveServer() 
{
  //deactivate();
}

sctConf::Configuration_ptr ConfigArchiveServer::getOldRunConfiguration (CORBA::ULong runNumber, CORBA::Boolean start)
{

  try {
    string instanceName = "configArchive_" + lexical_cast<string>(runNumber) + (start ? "_start" : "_end");
    string fileName = SctNames::getPersistentDir() + instanceName + ".xml.gz";    
    
    // first try to find if its already there...
    sctConf::Configuration_ptr handle = partition().lookup<sctConf::Configuration>(instanceName);
    if (!CORBA::is_nil(handle)){ return handle; }
    
#warning "Default to 'xml'"
    ConfigServer *oldConfig = new ConfigServer(partition(), instanceName, "xml", fileName);

    oldConfig->publish();
    return oldConfig->_this();
  } catch (ConfigurationException& e) {
    throw sctConf::ConfigurationException(e.what());
  }
}

void ConfigArchiveServer::shutdown(){
}

sctConf::RunNumberList* ConfigArchiveServer::listConfigurationRuns () 
{
  //@todo Needs c++ izing...

  string command = "ls -1 " + SctNames::getPersistentDir() + "/configArchive_*_start.*";
  FILE *lsInput = popen(command.c_str(), "r");

  std::list<unsigned int> mylist;

  while(!feof(lsInput)) {
    char *file = new char[512];
    
    file = fgets(file, 512, lsInput);

    if(file) {
      int runNumber = 0;

      string format = SctNames::getPersistentDir() + "/configArchive_%d_start";
      sscanf(file, format.c_str(), &runNumber);

      mylist.push_back(runNumber);
    }
  }
  
  sctConf::RunNumberList* result= new sctConf::RunNumberList(mylist.size());
  
  unsigned i=0;
  for (std::list<unsigned int>::const_iterator it=mylist.begin();
       it!=mylist.end();
       ++it){
    (*result)[i]=(*it);
  }
  return result;
}

int main(int argc, char** argv) 
{
  IPCCore::init(argc, argv);   // No multithreads
  IPCPartition p(::Sct::SctNames::getPartitionName());
  ipcServer = new IPCServer();

  ConfigArchiveServer *h = new ConfigArchiveServer(p,sctConf::ConfigurationArchive::instanceName);
  
  if (!h->publish()) {
    cout << "Failed to publish Configuration Archive" << endl;
    return 0;
  } 
  cout << "Configuration Archiving server published:\n";
  
  cout << "\n ... awaiting commands\n";

  try {
    ipcServer->run();
  } catch(ConfigurationException &c) {
    cout << "Configuration run threw exception " << c.what() << endl;
  }

  cout << "Configuration Archive Server run over" << endl;

  h->withdraw();
  delete h;
  delete ipcServer;
}
