#include "NoiseOccupancyDisplayer.h"
#include "DisplayManager.h"
#include "DisplayInfo.h"
#include "OpeDisplayer.h"
#include "SctData/OpeResult.h" 

#include "SctData/NoiseOccupancyTestResult.h"
#include "SctData/DefectList.h"

#include "TF1.h"
#include "TGraph.h"
#include "TCanvas.h"
#include "TLatex.h"

#include <iomanip>
#include <string>
#include <boost/shared_ptr.hpp>

using namespace std;
using namespace SctData;
using namespace Sct;
using namespace boost;

namespace SctDataDisplay {


class NoiseOccupancyDisplayData : public DisplayData {
public:
    vector<shared_ptr<TCanvas> > canvas;
    shared_ptr<const NoiseOccupancyTestResult> test;
    vector<shared_ptr<const DisplayData> > otherData;
};        
    
bool NoiseOccupancyDisplayer::inMap = DisplayManager::addToMap("SctData::NoiseOccupancyTestResult", shared_ptr<Displayer>(new NoiseOccupancyDisplayer()));
    
    
shared_ptr<DisplayData> NoiseOccupancyDisplayer::display(shared_ptr<const Sct::Serializable> serial, const DisplayInfo& info, std::ostream& os) { 
    shared_ptr<NoiseOccupancyDisplayData> data (new NoiseOccupancyDisplayData());    
    data->test = dynamic_pointer_cast<const NoiseOccupancyTestResult>(serial);
    
    //Now print out defects:
    //os << endl << "Defects: " << endl;
    //printDefectList(data->test->getDefects(), os);
    //os << endl;

    if (info.displayChips.size() > 0) displayChips(*data, info, os);
    
    shared_ptr<const SctData::OpeResult> ope = data->test->getOpeResult();

    if (ope.get() && info.occupancyHistograms) {
      
      shared_ptr<DisplayData> dd = DisplayManager::instance().display(ope, info, os); 
      data->otherData.push_back(dd);
    }
    return data;
}

void NoiseOccupancyDisplayer::displayChips(NoiseOccupancyDisplayData& data, const DisplayInfo& info, std::ostream& os) {
    shared_ptr<TCanvas> c = createCanvas((string)data.test->getUniqueID() + "_chipNO", "Noise Occupancy plots for " + data.test->getModuleName());
    data.canvas.push_back(c);
    divideCanvas(info.displayChips.size(), *c);
    
    for (unsigned int i=0; i<info.displayChips.size(); ++i) {
        const ChipNOResult& r = data.test->getChipResult(info.displayChips[i]);
	os << "Chip " << i << " noise: " << r.getNoise() << "  Fit params: ";
	for (int j=0; j<r.fit->GetNpar(); ++j) os << r.fit->GetParameter(j) << "   ";
	os << endl;
        c->cd(i+1);
	gPad->SetLeftMargin(0.15);
	gPad->SetBottomMargin(0.15);
	gPad->SetTopMargin(0.03);
	gPad->SetRightMargin(0.03);

	if (r.graph.get()){
	  if (r.graph->GetN()!=0){
	    if (DisplayManager::rootMode()){
	      shared_ptr<TDirectory> file=DisplayManager::getTDirectory();
	      if (file) {
		file->cd();
		std::ostringstream obname;
		obname << data.test->getUniqueID() << "_chipNO_" << i;
		r.graph->Write(obname.str().c_str());
	      }
	    }else{
	      r.graph->Draw("ap");
	    }
	    if (r.graph->GetHistogram()){
	      r.graph->GetHistogram()->SetXTitle("Threshold^2 (fC^2)");
	      r.graph->GetHistogram()->SetYTitle("Ln Occupancy");
	      r.graph->GetHistogram()->GetXaxis()->SetLabelSize(0.06);
	      r.graph->GetHistogram()->GetYaxis()->SetLabelSize(0.06);
	      r.graph->GetHistogram()->GetXaxis()->SetTitleSize(0.06);
	      r.graph->GetHistogram()->GetYaxis()->SetTitleSize(0.06);
	    }
	    try{
	      std::ostringstream text;
	      text << std::scientific << std::setprecision(1) << r.mean;
	      TLatex* l=new TLatex();
	      l->SetTextAlign(11);
	      l->SetTextSize(0.07);
	      l->DrawLatex(1.1,log(r.mean*1.5),text.str().c_str());
	    }catch(Sct::Throwable& e){
	      std::cerr << e.what();
	    }
	  }
	}
	if (r.fit.get()) {
	  r.fit->SetLineWidth(1);
	  r.fit->SetLineColor(2);
	  r.fit->SetFillStyle(0);
	  r.fit->Draw("lsame");
	}	
    }

}
}
