#include "OpeDisplayer.h"
#include "DisplayManager.h"
#include "DisplayInfo.h"

#include "SctData/OpeResult.h"

#include "TH1.h"
#include "TCanvas.h"
#include "TLatex.h"

#include <iomanip>
#include <string>
#include <boost/shared_ptr.hpp>

using namespace std;
using namespace SctData;
using namespace Sct;
using namespace boost;

namespace SctDataDisplay {

bool OpeDisplayer::inMap = DisplayManager::addToMap("SctData::OpeResult", shared_ptr<Displayer>(new OpeDisplayer()));
    
class OpeDisplayData : public DisplayData {
public:
    vector<shared_ptr<TCanvas> > canvas;
    shared_ptr<const OpeResult> ope;
};        
   
   
shared_ptr<DisplayData> OpeDisplayer::display(shared_ptr<const Sct::Serializable> serial, const DisplayInfo& info, std::ostream& os) { 
    shared_ptr<OpeDisplayData> data (new OpeDisplayData());
    data->ope = dynamic_pointer_cast<const OpeResult>(serial);

    if (info.displayChips.size() > 0) displayChips(*data, info, os);

    return data;
}

void OpeDisplayer::displayChips(OpeDisplayData& data, const DisplayInfo& info, std::ostream& os) {
  shared_ptr<TCanvas> c = createCanvas((string)data.ope->getUniqueID() + "_chipNO", "Noise Occupancy OPE plots for " + (string)data.ope->getUniqueID());
    data.canvas.push_back(c);
    divideCanvas(info.displayChips.size(), *c);

    for (unsigned int i=0; i<info.displayChips.size(); ++i) {
      const ChipOpeResult& r = data.ope->getChipResult(info.displayChips[i]);
	os << "Chip " << i << " badmax: " << r.badmax << " threshold: " << r.threshold << endl;
        c->cd(i+1);   
	gPad->SetLeftMargin(0.15);
	gPad->SetBottomMargin(0.15);
	gPad->SetTopMargin(0.03);
	gPad->SetRightMargin(0.03);

	if (r.hist.get()){
	  // series of obscure root incantations to make anything actually draw!
	  r.hist->SetXTitle("Threshold-TrimT(fC)");
	  r.hist->SetYTitle("Badness");
	  r.hist->SetLineWidth(2);
	  r.hist->SetMaximum(r.hist->GetMaximum()*1.2);
	  r.hist->GetYaxis()->Set(0, 0, r.hist->GetMaximum()*1.2);
	  r.hist->GetXaxis()->SetLabelSize(0.06);
	  r.hist->GetYaxis()->SetLabelSize(0.06);
	  r.hist->GetXaxis()->SetTitleSize(0.06);
	  r.hist->GetYaxis()->SetTitleSize(0.06);
	  displayHistogram(r.hist,"");
	}
    }
    
}
}

