#include "StrobeDelayDisplayer.h"
#include "DisplayManager.h"
#include "DisplayInfo.h"

#include "Sct/IS/IOManagerIS.h"
#include "Sct/Archive/IONameArchiveFile.h"
#include "Sct/Archive/IOManagerArchiveFile.h"
#include "SctData/StrobeDelayTestResult.h"
#include "SctData/FitScanResult.h"
#include "SctData/ConfigurationVariable.h"
#include "FitDisplayer.h"
#include <TCanvas.h>
#include <string>
#include <iomanip>
#include <boost/scoped_ptr.hpp>

using namespace Sct;
using namespace SctData;
using namespace std;
using namespace boost;

namespace SctDataDisplay {

bool StrobeDelayDisplayer::inMap = DisplayManager::addToMap("SctData::StrobeDelayTestResult", shared_ptr<Displayer>(new StrobeDelayDisplayer()));    
    
shared_ptr<DisplayData> StrobeDelayDisplayer::display(shared_ptr<const Sct::Serializable> data, const DisplayInfo& info, std::ostream& os) {
    shared_ptr<const StrobeDelayTestResult> result = dynamic_pointer_cast<const StrobeDelayTestResult>(data);
    
    os << "Module: " << result->getModuleName() << endl;

    //Now print out defects:
    os << "Defects: " << endl;
    printDefectList(result->getDefects(), os);
    
    for (unsigned int ichip=0; ichip<result->getNOptima(); ++ichip) {
	os << "StrobeDelay for chip: " << ichip << "  " << setprecision(2) << (int) (result->getOptimum(ichip)+0.5) << endl;
    }
    shared_ptr<const SctData::FitScanResult> fit;
    try {
      string fitName = result->getHeader().getUniqueID();
      shared_ptr<Serializable> ob = Sct::IS::IOManagerIS::instance().read("FittedData.SctData::FitScanResult."+ fitName);
	fit = dynamic_pointer_cast<const FitScanResult>(ob);
    } catch (Sct::IoException& e){
    }
    if (fit.get()==0) try {
       Sct::Archive::IONameArchiveFile name(result->getHeader().getUniqueID(),"SctData::FitScanResult");
       shared_ptr<Serializable> ob = Sct::Archive::IOManagerArchiveFile::instance().read(name.getIOName());
       fit = dynamic_pointer_cast<const FitScanResult>(ob);
    } catch (Sct::IoException& e) {
    }
    if (fit.get()==0) {
      return shared_ptr<DisplayData>();
    }else{
      return DisplayManager::instance().display(fit, info, os);
    }
}
}
