#ifndef CDIDATAHANDLE_H
#define CDIDATAHANDLE_H
///////////////////////////////////////////////////////////////////////
///   cdiDataHandle.h                                               ///
///                                                                 ///
///               Luis Pedro ATLAS/DAQ 2003                         ///
///     Interface class to access the ConditionsDB methods          ///
///                                                                 ///
///////////////////////////////////////////////////////////////////////

#include "ConditionsDB/ICondDBMgr.h"
#include "ConditionsDB/CondDBMySQLMgrFactory.h"
#include "ConditionsDB/ICondDBTable.h"
#include "ConditionsDB/CondDBMySQLTableFactory.h"

#include "cdi/cdiAux.h"
//#include <is/isinfo.h>
#include <is/info.h>
#include <vector>
#include <map>
//#include <is/isinfoany.h>
#include <is/infodocument.h>
#include <is/infodictionary.h>
#include <is/inforeceiver.h>

#include "cdi/cdiHandle.h"

class cdiDataHandle: public cdiHandle
{  
 public:
  static const std::string configuration_name;
  
  cdiDataHandle ( int verbosity, const IPCPartition & partition );
  
  
  void getConfig ( std::vector<std::string> & names );  
  void process_data ( const std::string & name, ISCallbackInfo * isc );
  
  void storeObject ( const std::string& name, ISInfoAny &isa);

  std::string getConfName() {
    if (configuration_name.size() != 0)
      return configuration_name;
    else
      return "";
  }

 private:
  IPCPartition	partition;
  cdiAux		mout;
  ICondDBMgr *	condDBmgr;
  ICondDBDataAccess *	condDataAccess;
  ICondDBFolderMgr *	condFolderMgr;
  
  std::map <ISType::Basic, ICondDBTable::cdb_types>	types;
  std::map <ISType::Basic, ICondDBTable::cdb_types>	a_types;
  std::map <std::string, ICondDBTable*>		tables;
  
  std::string getProfile ( );
  void init ( const std::string & profile ); 
  void create_folder ( const std::string & name, ISInfoAny & isa );
  void add_table ( ICondDBTable * table, const std::string & name );
  bool is_table_exist ( const std::string & name );
};
#endif //CDIDATAHANDLE_H

