
////////////////////////////////////////////////////////////////////////
//    create_info.cc
//
//    Test application for the IS library
//
//    Sergei Kolos,  January 2002
//
//    description:
//          Test the functionality of the ISNamedInfo class
//      
////////////////////////////////////////////////////////////////////////

#include <iostream>
#include "ipc/core.h"
#include <ipc/partition.h>
#include "cdi/Conditions.h"
#include <unistd.h>

int main(int argc, char ** argv)
{
  IPCCore::init( argc, argv );
  
  std::cout << OWLTime() << " :: cdiISTest is initializing!" << std::endl;
  std::cout << OWLTime() << " :: cdiISTest -> Initializing partition [be_test]!" << std::endl;
  IPCPartition      p("be_test");

  std::cout << OWLTime() << " :: cdiISTest -> Initializing the dictionary!" << std::endl;
  ISInfoDictionary dict(p); 
  
  std::cout << OWLTime() << " :: cdiISTest -> Initializing Conditions object!" << std::endl;
  Conditions c;
  
  std::vector<std::string> a;

  std::cout << OWLTime() << " :: cdiISTest -> Adding Information objects to the list!" << std::endl;
  a.push_back("RunCtrl.Application_dsa_supervisor");
  a.push_back("RunEvents.*");
  a.push_back("RunParams.RunParams");
  a.push_back("RunParams.Conditions");



  /*
    std::string *a = new std::string[3];
    
    a[0] = "RunCtrl.Application_dsa_supervisor";  
    a[1]  = "RunEvents.*";
    
    a[2] ="RunParams.RunParams";
    
  */
  
  // define the information about the Conditions
  c.ISServers_size = 4;
  c.ISServers = a;
  
  //c.ISServers = new std::string("RunParams.RunParams");
  
  std::cout << OWLTime() << " :: cdiISTest -> subscribing the servers to the dictionary!" << std::endl;
  dict.insert("MyServer.Conditions", c);
  
  
  //c.ISServers = new std::string("RunCtr.RunCtrl");
  
  // let's create multiple threads to submit data
  int pid; /* alias: pid_t pid; */
  int pid2;
  pid = fork(); /* the familiar kitchenware! */
  
  if (pid == -1) {
    std::cerr << OWLTime() << " :: cdiISTest failed to create multiple threads!" << std::endl;
    exit(1);
  } /* fork() returns -1 on error, and much knownigly sets the global errno variable */
  
  if (pid != 0) {
    std::cout << OWLTime() << " :: cdiISTest -> Main thread executing: PID: [" << getpid() << "]!" << std::endl;
    std::cout << OWLTime() << " :: cdiISTest -> Forking again the execution" << std::endl;
    
    pid2 = fork();

    if (pid2 == -1) {
      std::cerr << OWLTime() << " :: cdiISTest failed to create multiple threads!" << std::endl;
      exit(1);
    }
    
    // this is the main thread...it'll update information to the IS
    if (pid != 0) {
      
      while(1){
	//a[0]="RunParams.RunParams";
	//a[1] = "Ola";
	//c.ISServers_size = 2;
	//c.ISServers = a;
	dict.update ("MyServer.Conditions", c);
	sleep(5);
      }
    }

  } /* if the value returned is non zero and not -1, then this is the parent process */
  printf("\n\n\tWe love Unix!\n\n"); /* this is a common task for both processes */
  if (pid == 0) {
    printf("Hello World, my name is GoogooGaga!\n");
    printf("My process id is %d\n", getpid());
    printf("I have ONE papa (ofcourse), his name is Pappy, and has a process id %d\n", getppid());
  } /* if the returned value is 0 then you're the child process */
  exit(0);


  while(1){
    //a[0]="RunParams.RunParams";
    //a[1] = "Ola";
    //c.ISServers_size = 2;
    //c.ISServers = a;
    dict.update ("MyServer.Conditions", c);
    sleep(5);
  }
  
  return 0;
}
