////////////////////////////////////////////////////////////////////////
//    cdi_set_condition  
//
//    Application to set up CDI with the proper ISServers.Information
//    that will be used to store data in the ConditionsDatabaes
//
//    Lus Pedro,  March 2004
//    Nuno Barros, January 2005
//
//    description:
//          Set up the IS information RunParams.Conditions with the 
//          Informations to be stored in the Conditions Database
//          By default RunParams.RunParams and RunParams.RunInfo
//      
////////////////////////////////////////////////////////////////////////

#include <unistd.h>
#include <iostream>
#include <ipc/partition.h>
#include <ipc/core.h>
#include <cmdl/cmdargs.h>
#include <list>
#include "rc/Conditions.h"
//changes after new IS api
#include <is/infodictionary.h>

int main(int argc, char ** argv)
{
  IPCCore::init(argc, argv);
    
  // Declare arguments
   CmdArgStr            partition_name('p', "partition", "partition-name", "partition to work in.", CmdArg::isREQ);
   CmdArgInt	        verb_level('v', "verbose", "verbosity-level", "0:Silent, 1:Normal, 2:Trace (defaults to 0).");
   //   CmdArgStr            backend('B',"backend","Choose the ConditionsDB API (COOL,CONDDB)");
   CmdArgStrList	isexpr('I', "isnames", "is-names", "list of ISServer.ISInformation names to be taken by CDI.", 
			       CmdArg::isREQ | CmdArg::isLIST);
         
   CmdLine  cmd(*argv, &partition_name, &verb_level/*, &backend */, &isexpr,NULL);
   CmdArgvIter  arg_iter(--argc, ++argv);
                
   cmd.description("This program implements the functionality to store IS the name of IServers and its\n"
		   "Informations (ISServers.ISInformation) in the RunParams.Conditions Information.\n"
		   "The RunParams.Conditions information will then be used by cdi_daq to collect data\n"
		   "to the Conditions Database.\n");
   
   verb_level=0;
   cmd.parse(arg_iter); 
        
   IPCPartition		p(partition_name);
   ISInfoDictionary	dict(p); 
  
   Conditions conditions;
   
   for( size_t i = 0; i < isexpr.count(); i++ )
     conditions.isservers.push_back( std::string(isexpr[i]) );
   
   ISInfo::Status s = dict.insert( "RunParams.Conditions", conditions );
   
   if ( s == ISInfo::AlreadyExist )
     {
       s = dict.update( "RunParams.Conditions", conditions );
     }
   
   if (s!=ISInfo::Success) {
     // let's try to get more information
     switch (s) {
     case ISInfo::CommFailure:
       std::cout << OWLTime() << "Inserting/Update \"RunParams.Conditions\" FAILED: CommFailure !" << std::endl;
       break;
        // nbarros new additions
     case ISInfo::NotFound:
       std::cout << OWLTime() << "Inserting/Update \"RunParams.Conditions\" FAILED: NotFound !" << std::endl;
       break;
     case ISInfo::InvalidInfo:
       std::cout << OWLTime() << "Inserting/Update \"RunParams.Conditions\" FAILED: InvalidInfo !" << std::endl;
       break;
     case ISInfo::IncompatibleType:
       std::cout << OWLTime() << "Inserting/Update \"RunParams.Conditions\" FAILED: IncompatibleType !" << std::endl;
       break;
     case ISInfo::InvalidCriteria:
       std::cout << OWLTime() << "Inserting/Update \"RunParams.Conditions\" FAILED: InvalidExpression !" << std::endl;
       break;
     case ISInfo::RepositoryNotFound:
       std::cout << OWLTime() << "Inserting/Update \"RunParams.Conditions\" FAILED: RepositoryNotFound !" << std::endl;
       break;
     case ISInfo::ProviderNotFound:
       std::cout << OWLTime() << "Inserting/Update \"RunParams.Conditions\" FAILED: ProviderNotFound !" << std::endl;
       break;
     case ISInfo::InvalidName:
       std::cout << OWLTime() << "Inserting/Update \"RunParams.Conditions\" FAILED: InvalidName !" << std::endl;
       break;
     default:
       std::cout << OWLTime() << "Inserting/Update \"RunParams.Conditions\" FAILED: Unknown error !" << std::endl;
       break;
     }
   }
   return !(s==ISInfo::Success);
   //return s;
}
