
/**
 * This test checks for the minimum time for consecutive updates
 * Uses 1 ISServer and 1 Object
 * Only one parameter is updated
 * The times are stored as miliseconds
 */

/// Standard C++ includes
#include <iostream>
#include <vector>
#include <string>
#include <fstream>

// Other standard C includes
extern "C" {
#include <unistd.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <signal.h>
#include <sys/time.h>
};

// some cclasses needed to store data
#include "ipc/core.h"
#include <ipc/partition.h>
#include "cdi/cdiHandle.h"

// The objects to be stored
#include "rc/Conditions.h"
#include "rc/RunParams.h"




// the number of data updates that must be made
#define MAX_UPD 1000
#define log std::cout << OWLTime()





int main(int argc, char**argv){
  IPCCore::init(argc, argv);

  setenv("TEST_NAME", __FILE__ ,1);  
  if (argc != 2 ){
    log << "::cdi_test -> Time interval not defined" << std::endl;
    return 1;
  }

  timeval t_start, t_end;

  std::ofstream fout("logs/cdi_test1.log");
  double interval;
  unsigned int tIntv = atoi(argv[1]);
  log << "::cdi_test -> Storing data with interval of " << tIntv << " microseconds" << std::endl;

  std::cout << OWLTime() << " :: cdi_test -> Initializing partition [be_test]!" << std::endl;
  
  fout << "------------------ Test1 START------------------------------" << std::endl;
  fout << "Initialize Online Environment : [" ;
  
  gettimeofday(&t_start,NULL);
  
//IPCPartition      p("part_localhost");
IPCPartition      p("be_test");
  
  if (p.isValid()) {
    std::cout << OWLTime() << " :: partition is valid!" << std::endl;
  }
  else
    std::cout << OWLTime() << " :: partition is NOT valid!" << std::endl;
  
  
  
  std::cout << OWLTime() << " :: cdi_test -> Initializing the dictionary!" << std::endl;
  
  ISInfoDictionary dict(p); 
  
  std::cout << OWLTime() << " :: cdi_test -> Initializing Conditions object!" << std::endl;
  //  Conditions c;
 

  gettimeofday(&t_end,NULL);
  interval = (double)(((double)t_end.tv_sec*1000+(double)t_end.tv_usec/1000)-((double)t_start.tv_sec*1000+(double)t_start.tv_usec/1000));
  fout << interval << "]" << std::endl;
  
  
  RunParams rp;
  
  std::vector<std::string> a;
  
  std::cout << OWLTime() << " :: cdi_test -> Adding Information objects to the list!" << std::endl;
  // a.push_back("RunCtrl.Application_dsa_supervisor");
  //  a.push_back("RunEvents.*");
  a.push_back("RunParams.RunParams");
  //  a.push_back("RunParams.Conditions");
  
  rp.run_number = 0;
  rp.max_events = 1000;
  rp.rec_enable = 1;
  rp.trigger_type = 1;
  rp.detector_mask = 1;
  rp.beam_type = 1;
  rp.beam_energy = 1000;
  rp.filename_tag = "Test string";
  rp.timeSOR = OWLTime();
  rp.timeEOR = OWLTime();
  rp.totalTime = 12312;
  
                //  c.ISServers_size = 3;
  // c.isservers = a;
  
  
  //  std::cout << OWLTime() << " :: cdi_test -> Publishing the servers to the dictionary!" << std::endl;
  //  dict.insert("RunParams.Conditions", c);
  std::cout << OWLTime() << " :: cdi_test -> Insert RunParams to the dictionary!" << std::endl;

  fout << "Subscribing the object :["; 
  gettimeofday(&t_start,NULL);
  dict.insert("RunParams.RunParams",rp);
  gettimeofday(&t_end,NULL);
  interval = (double)(((double)t_end.tv_sec*1000+(double)t_end.tv_usec/1000)-((double)t_start.tv_sec*1000+(double)t_start.tv_usec/1000));
  fout << interval << "]" << std::endl;  
  sleep(30);

  //c.ISServers = new std::string("RunCtr.RunCtrl");
  
  /** We don't need any more this sleep period...let's just wait for the child to return
      std::cout << OWLTime() << " ::cdi_test (parent) -> Sleeping 5sec to give time to child to prepare" << std::endl;
      sleep(5);
      
  
  waitpid(pid,NOHANG,NULL);
  */  
  std::cout << OWLTime() << " ::cdi_test (parent) -> Starting to update the objects" << std::endl;
  //after having a nice beauty sleep we need to get to work again
  // let's start to update data
  
  int status;
  timeval tt_s, tt_e;
  gettimeofday(&t_start,NULL);

  for (unsigned int i = 1; i <= MAX_UPD; i++) {
    std::ostringstream t;
    rp.run_number += 1;
    rp.max_events += 10;
    rp.rec_enable = ((rp.rec_enable) % i);
    rp.trigger_type = 1;
    rp.detector_mask = 1;
    rp.beam_type = 1;
    rp.beam_energy += 11;
    t << "_" << i;
    rp.filename_tag = std::string("Teste_str" + t.str());
    /*
      rp.timeSOR = OWLTime();
      rp.timeEOR = OWLTime();
      rp.totalTime = 12312;
    */
    fout << "Updating object [" << i << ": -> [";
    gettimeofday(&tt_s,NULL);
    dict.update("RunParams.RunParams",rp);
    gettimeofday(&tt_e,NULL);
    interval = (double)(((double)tt_e.tv_sec*1000+(double)tt_e.tv_usec/1000)-((double)tt_s.tv_sec*1000+(double)tt_s.tv_usec/1000));
    fout << interval << "]" << std::endl;
    // number of microseconds to sleep
    status = usleep(tIntv);
    if (status != 0) {
    fout << "========> FAILED TO SLEEP!!"<< std::endl;
    fout.close();
    exit (1);
    }
    
  }
  gettimeofday(&t_end,NULL);
  interval = (double)(((double)t_end.tv_sec*1000+(double)t_end.tv_usec/1000)-((double)t_start.tv_sec*1000+(double)t_start.tv_usec/1000));
  fout << "Total update time: [" << interval << "]" << std::endl;
  fout << "-------------------------------------END OF TEST ----------------------------------";

  fout.close();
  log << "::cdi_test -> Test finished....exiting" << std::endl;
  return 0;
}
