#include "boost/format.hpp"
#include "boost/lexical_cast.hpp"
#include "dalImplUtils.h"
#include "dalImpl.h"

#include "Channel.h"
#include "RodRModule.h"
#include "Barrel.h"
#include "Crate.h"
/* #include "Geography.h" */
#include "GeographyMUR.h"
#include "Partition.h"
#include "PowerCard.h"
#include "PowerChannel.h"
#include "PowerCrate.h"
#include "PowerParam.h"
#include "Rod.h"
#include "RodMUR.h"
/* #include "RodModule.h" */
#include "Row.h"
/* #include "SCT_Chip.h" */
#include "SCT_Module.h"
/* #include "Slave.h" */
/* #include "Tim.h" */

namespace ConfDALUtils {
  SCTDAL::Partition *Builder::getOrCreatePartition(unsigned int partition) {
    std::cerr << "getOrCreatePartition " << partition << std::endl;
    std::string query = "(this (\"id\" " + boost::lexical_cast<std::string>(partition) + " =))";
    std::vector<const SCTDAL::Partition *> partitions;
    m_conf.get<SCTDAL::Partition>(partitions, false, true, query);

    log << "Found " << partitions.size() << " partitions\n";

    SCTDAL::Partition * object = 0;
    if(partitions.size() == 1) {
      object = const_cast<SCTDAL::Partition*>(partitions.front());
      assert(object->get_id() == partition);
    } else if(partitions.size() == 0) {
      object = const_cast<SCTDAL::Partition*>(m_conf.create<SCTDAL::Partition>(m_impl.lastFilename, ""));
      if(object == 0) {
	m_conf.abort();
	throw SctConfiguration::ConfigurationException("Failed to create partition");
      }

      object->set_id(partition);
      m_conf.commit();
    }

    return object;
  }

  const SCTDAL::Partition *LookUp::getPartition(unsigned int partition) {
    std::string query = "(this (\"id\" " + boost::lexical_cast<std::string>(partition) + " =))";
    std::vector<const SCTDAL::Partition *> partitions;
    m_conf.get<SCTDAL::Partition>(partitions, false, true, query);

    log << "Found " << partitions.size() << " partitions\n";

    const SCTDAL::Partition * object = 0;
    if(partitions.size() == 1) {
      object = partitions.front();
      assert(object->get_id() == partition);
    } else if(partitions.size() == 0) {
      throw SctConfiguration::ConfigurationException("Can't find partition");
    } else {
      throw SctConfiguration::ConfigurationException("Too many partitions");
    }

    return object;
  }

  SCTDAL::Crate *Builder::getOrCreateCrate(unsigned int partition, unsigned int crate) {
    std::cerr << "GetOrCreateCrate " << partition << " " << crate << "\n";
    SCTDAL::Partition *partOb = getOrCreatePartition(partition);

    if(!partOb) {
      throw SctConfiguration::ConfigurationException("Failed to create partition");
    }

    std::vector<const SCTDAL::Crate *> crates = partOb->get_crates();

    log << "Found " << crates.size() << " crates\n";

    SCTDAL::Crate * object = 0;

    if(crates.size() > 0) {
      for(SCTDAL::CrateIterator i = crates.begin(); i!= crates.end(); i++) {
	if((*i)->get_id() == crate) {
	  if(!object) {
	    object = const_cast<SCTDAL::Crate*>(*i);
	  } else {
	    std::cerr << "Found too many crates\n";
	    //	    printVector(crates);
	    throw SctConfiguration::ConfigurationException("Too many crates");
	  }
	}
      }
    }

    if(!object) {
      object = const_cast<SCTDAL::Crate*>(m_conf.create<SCTDAL::Crate>(m_impl.lastFilename, ""));
      if(object == 0) {
	m_conf.abort();
	throw SctConfiguration::ConfigurationException("Failed to create crate");
      }

      object->set_id(crate);
      crates.push_back(object);
      partOb->set_crates(crates);

      m_conf.commit();
    }

    return object;
  }

  const SCTDAL::Crate *LookUp::getCrate(unsigned int partition, unsigned int crate) {
    log << "GetCrate\n";
    const SCTDAL::Partition *partOb = getPartition(partition);

    if(!partOb) {
      throw SctConfiguration::ConfigurationException("Failed to get partition");
    }

    std::vector<const SCTDAL::Crate *> crates = partOb->get_crates();

    log << "Found " << crates.size() << " crates\n";

    const SCTDAL::Crate * object = 0;
    if(crates.size() == 0) {
      throw SctConfiguration::ConfigurationException("Not enough crates");
    } else {
      for(SCTDAL::CrateIterator i = crates.begin(); i!= crates.end(); i++) {
	if((*i)->get_id() == crate) {
	  if(!object) {
	    object = *i;
	  } else {
	    std::cerr << "Found too many crates for id\n";
	    printVector(crates);
	    throw SctConfiguration::ConfigurationException("Too many crates");
	  }
	}
      }
    }

    if(!object) {
      throw SctConfiguration::ConfigurationException("Can't find crate " 
						     + boost::lexical_cast<std::string>(partition) 
						     + " " + boost::lexical_cast<std::string>(crate));
    }

    return object;
  }

  SCTDAL::Rod *Builder::getOrCreateRod(unsigned int partition, unsigned int crate, unsigned int rod) {
    std::cerr << "GetOrCreateRod " << partition << " " << crate << " " << rod << "\n";
    SCTDAL::Crate *crateOb = getOrCreateCrate(partition, crate);

    if(!crateOb) {
      throw SctConfiguration::ConfigurationException("Failed to create crate");
    }

    typedef std::vector<const SCTDAL::Rod *> RodVector;
    RodVector rods = crateOb->get_rods();

    log << "Found " << rods.size() << " rods\n";

    SCTDAL::Rod * object = 0;
    if(rods.size() > 0) {
      for(RodVector::iterator i = rods.begin(); i!= rods.end(); i++) {
	if((*i)->get_id() == rod) {
	  if(!object) {
	    object = const_cast<SCTDAL::Rod*>(*i);
	  } else {
	    throw SctConfiguration::ConfigurationException("Too many RODs");
	  }
	}
      } 
    }

    if(!object) {
      object = const_cast<SCTDAL::Rod*>(m_conf.create<SCTDAL::Rod>(m_impl.lastFilename, ""));
      if(object == 0) {	
	m_conf.abort();
	throw SctConfiguration::ConfigurationException("Failed to create ROD");
      }

      object->set_id(rod);
      rods.push_back(object);
      crateOb->set_rods(rods);

      m_conf.commit();
    }

    return object;
  }

  const SCTDAL::Rod *LookUp::getRod(unsigned int partition, unsigned int crate, unsigned int rod) {
    log << "GetRod\n";
    const SCTDAL::Crate *crateOb = getCrate(partition, crate);

    std::vector<const SCTDAL::Rod*> rods = crateOb->get_rods();

    log << "Found " << rods.size() << " rods\n";

    const SCTDAL::Rod * object = 0;
    if(rods.size() > 0) {
      for(SCTDAL::RodIterator i = rods.begin(); i!= rods.end(); i++) {
	if((*i)->get_id() == rod) {
	  if(!object) {
	    object = *i;
	  } else {
	    std::cerr << "Too many RODs in:\n";
	    printVector(rods);
	    throw SctConfiguration::ConfigurationException("Too many RODs");
	  }
	}
      } 
    }

    if(!object) {
      throw SctConfiguration::ConfigurationException("No RODs");
    }

    return object;
  }

  const SCTDAL::Barrel *LookUp::getBarrel(unsigned int barrel) {
    std::string query = "(this (\"id\" " + boost::lexical_cast<std::string>(barrel) + " =))";

    std::vector<const SCTDAL::Barrel *> barrels;
    m_conf.get<SCTDAL::Barrel>(barrels, false, true, query);

    log << "Found " << barrels.size() << " barrels\n";

    const SCTDAL::Barrel *barrelOb = 0;

    if(barrels.size() > 0) {
      // typedef std::vector<const SCTDAL::Barrel *> BarrelVector;
      for(SCTDAL::BarrelIterator i = barrels.begin(); i != barrels.end(); i++) {
	if( (*i)->get_id() == barrel) {
	  barrelOb = *i;
	  break;
	}
      }
    }

    return barrelOb;
  }

  SCTDAL::Barrel *Builder::getOrCreateBarrel(unsigned int barrel) {
    std::cerr << "GetOrCreateBarrel\n";

    std::string query = "(this (\"id\" " + boost::lexical_cast<std::string>(barrel) + " =))";

    std::vector<const SCTDAL::Barrel *> barrels;
    m_conf.get<SCTDAL::Barrel>(barrels, false, true, query);

    log << "Found " << barrels.size() << " barrels\n";

    SCTDAL::Barrel *barrelOb = 0;

    if(barrels.size() > 0) {
      // typedef std::vector<const SCTDAL::Barrel *> BarrelVector;
      for(SCTDAL::BarrelIterator i = barrels.begin(); i != barrels.end(); i++) {
	if( (*i)->get_id() == barrel) {
	  barrelOb = const_cast<SCTDAL::Barrel*>(*i);
	  break;
	}
      }
    }

    if(!barrelOb) {
      barrelOb = const_cast<SCTDAL::Barrel*>(m_conf.create<SCTDAL::Barrel>(m_impl.lastFilename, ""));
      if(barrelOb == 0) {
	m_conf.abort();
	throw SctConfiguration::ConfigurationException("Failed to create Barrel");
      }

      barrelOb->set_id(barrel);

#warning "Could create geography node"
      m_conf.commit();
    }

    return barrelOb;
  }

  const SCTDAL::Row *LookUp::getBarrelRow(unsigned int barrel, unsigned int row) {
    const SCTDAL::Barrel *barrelOb = getBarrel(barrel);

    std::vector<const SCTDAL::Row *> rows = barrelOb->get_rows();

    log << "Found " << rows.size() << " rows\n";

    const SCTDAL::Row *rowOb = 0;

    if(rows.size() > 0) {
      typedef std::vector<const SCTDAL::Row *> RowVector;
      for(RowVector::iterator i = rows.begin(); i != rows.end(); i++) {
	if( (*i)->get_id() == row) {
	  rowOb = *i;
	  break;
	}
      }
    }

    return rowOb;
  }

  SCTDAL::Row *Builder::getOrCreateBarrelRow(unsigned int barrel, unsigned int row) {
    std::cerr << "GetOrCreateBarrelRow\n";

    SCTDAL::Barrel *barrelOb = getOrCreateBarrel(barrel);

    std::vector<const SCTDAL::Row *> rows = barrelOb->get_rows();

    log << "Found " << rows.size() << " rows\n";

    SCTDAL::Row *rowOb = 0;

    if(rows.size() > 0) {
      typedef std::vector<const SCTDAL::Row *> RowVector;
      for(RowVector::iterator i = rows.begin(); i != rows.end(); i++) {
	if( (*i)->get_id() == row) {
	  rowOb = const_cast<SCTDAL::Row*>(*i);
	  break;
	}
      }
    }

    if(!rowOb) {
      rowOb = const_cast<SCTDAL::Row*>(m_conf.create<SCTDAL::Row>(m_impl.lastFilename, ""));
      if(rowOb == 0) {
	m_conf.abort();
	throw SctConfiguration::ConfigurationException("Failed to create Row");
      }

      rowOb->set_id(row);

      rows.push_back(rowOb);
      barrelOb->set_rows(rows);

      m_conf.commit();
    }    

    return rowOb;
  }

  SCTDAL::PowerCrate *Builder::getOrCreatePowerCrate(unsigned int crate) { 
    std::cerr << "getOrCreatePowerCrate " << crate << std::endl;
    std::string query = "(this (\"id\" " + boost::lexical_cast<std::string>(crate) + " =))";
    std::vector<const SCTDAL::PowerCrate *> crates;
    m_conf.get<SCTDAL::PowerCrate>(crates, false, true, query);

    log << "Found " << crates.size() << " crates\n";

    SCTDAL::PowerCrate * object = 0;
    if(crates.size() == 1) {
      object = const_cast<SCTDAL::PowerCrate*>(crates.front());
      assert(object->get_id() == crate);
    } else if(crates.size() == 0) {
      object = const_cast<SCTDAL::PowerCrate*>(m_conf.create<SCTDAL::PowerCrate>(m_impl.lastFilename, ""));
      if(object == 0) {
	m_conf.abort();
	throw SctConfiguration::ConfigurationException("Failed to create power crate");
      }

      object->set_id(crate);
      m_conf.commit();
    }

    return object;
  }

  SCTDAL::PowerChannel *Builder::getOrCreatePowerChannel(unsigned int crate, unsigned int channel) {
    SCTDAL::PowerCrate *crateObj = getOrCreatePowerCrate(crate);

    std::vector<const SCTDAL::PowerChannel *> channels(crateObj->get_channels());

    SCTDAL::PowerChannel *chanObj = 0;
    for(SCTDAL::PowerChannelIterator i = channels.begin(); i!=channels.end(); i++) {
      if((*i)->get_id() == channel) {
	chanObj = const_cast<SCTDAL::PowerChannel *>(*i);
      }
    }

    if(!chanObj) {
      chanObj = const_cast<SCTDAL::PowerChannel*>(m_conf.create<SCTDAL::PowerChannel>(m_impl.lastFilename, ""));
      if(!chanObj) {
	m_conf.abort();
	throw SctConfiguration::ConfigurationException("Couldn't create PowerChannel");
      }

      chanObj->set_id(channel);
      channels.push_back(chanObj);
      crateObj->set_channels(channels);

      m_conf.commit();
    }

    return chanObj;
  }
 

  SCTDAL::Channel *Builder::getOrCreateChannelInRodMur(SCTDAL::RodMUR *murObj, int position) {
    std::vector<const SCTDAL::Channel *> channels(murObj->get_channels());

    SCTDAL::Channel *chanObj = 0;
    for(SCTDAL::ChannelIterator i = channels.begin(); i!=channels.end(); i++) {
      if((*i)->get_id() == position) {
	chanObj = const_cast<SCTDAL::Channel *>(*i);
      }
    }

    if(!chanObj) {
      chanObj = const_cast<SCTDAL::Channel*>(m_conf.create<SCTDAL::Channel>(m_impl.lastFilename, ""));
      if(!chanObj) {
	m_conf.abort();
	throw SctConfiguration::ConfigurationException("Couldn't create Channel");
      }

      chanObj->set_id(position);
      channels.push_back(chanObj);
      murObj->set_channels(channels);

      m_conf.commit();
    }

    return chanObj;
  }

  SCTDAL::PowerParam *Builder::getOrCreatePowerParamInChannel(SCTDAL::PowerChannel *chanObj, std::string state, std::string name) {
#warning "State one of 'OFF', 'STB', 'ON', 'ALL'"
    std::vector<const SCTDAL::PowerParam *> params = chanObj->get_params();

    SCTDAL::PowerParam *paramObj = 0;
    for(SCTDAL::PowerParamIterator i = params.begin(); i!=params.end(); i++) {
      paramObj = const_cast<SCTDAL::PowerParam *>(*i);
      if(paramObj->get_state() == state && paramObj->get_name() == name) {
	break;
      } else {
	paramObj = 0;
      }
    }

    if(!paramObj) {
      paramObj = const_cast<SCTDAL::PowerParam*>(m_conf.create<SCTDAL::PowerParam>(m_impl.lastFilename, ""));
      if(!paramObj) {
	m_conf.abort();
	throw SctConfiguration::ConfigurationException("Couldn't create PowerParam");
      }

      paramObj->set_state(state);
      paramObj->set_name(name);
      params.push_back(paramObj);
      chanObj->set_params(params);

      m_conf.commit();
    }

    return paramObj;
  }

  const SCTDAL::RodMUR *LookUp::getMur(unsigned int MUR) {
    const SCTDAL::RodMUR *murObj = m_conf.get<SCTDAL::RodMUR>(boost::lexical_cast<std::string>(MUR));
    if(!murObj) {
      // Try named as "MUR3003"
      murObj = m_conf.get<SCTDAL::RodMUR>("MUR" + boost::lexical_cast<std::string>(MUR));
    }
    return murObj;
  }

  const SCTDAL::GeographyMUR *LookUp::getGeogMur(unsigned int MUR) {
    const SCTDAL::GeographyMUR *murObj = m_conf.get<SCTDAL::GeographyMUR>(boost::lexical_cast<std::string>(MUR));
    if(!murObj) {
      // Try named as "mur_3003"
      murObj = m_conf.get<SCTDAL::GeographyMUR>("mur_" + boost::lexical_cast<std::string>(MUR));
    }
    if(!murObj) {
      // Try named as "GEOG3003"
      murObj = m_conf.get<SCTDAL::GeographyMUR>("GEOG" + boost::lexical_cast<std::string>(MUR));
    }
    return murObj;
  }

  const SCTDAL::PowerCrate *LookUp::getPowerCrate(unsigned int crate) {
    std::string query = "(this (\"id\" " + boost::lexical_cast<std::string>(crate) + " =))";
    std::vector<const SCTDAL::PowerCrate *> crates;
    m_conf.get<SCTDAL::PowerCrate>(crates, false, true, query);

    const SCTDAL::PowerCrate * object = 0;
    if(crates.size() == 1) {
      object = crates.front();
      assert(object->get_id() == crate);
    }

    return object;
  }

  const SCTDAL::PowerChannel *LookUp::getPowerChannel(unsigned int crate, unsigned int channel) {
    const SCTDAL::PowerCrate *crateObj = getPowerCrate(crate);

    std::vector<const SCTDAL::PowerChannel *> channels(crateObj->get_channels());

    const SCTDAL::PowerChannel *chanObj = 0;
    for(SCTDAL::PowerChannelIterator i = channels.begin(); i!=channels.end(); i++) {
      if((*i)->get_id() == channel) {
	chanObj = *i;
      }
    }

    return chanObj;
  }

  const SCTDAL::PowerChannel *LookUp::getPowerChannelByMUR(unsigned int MUR, unsigned int number) {
    std::string query = (boost::format("(this (and (\"MUR\" %d =) (\"module\" %d =)))") % MUR % number).str();
    std::vector<const SCTDAL::PowerChannel *> channels;
    m_conf.get<SCTDAL::PowerChannel>(channels, false, true, query);

    SCTDAL::PowerChannel * chanObj = 0;
    if(channels.size() == 1) {
      chanObj = const_cast<SCTDAL::PowerChannel*>(channels.front());
      assert(chanObj->get_MUR() == MUR && chanObj->get_module() == number);
    }

    return chanObj;
  }

  const SCTDAL::SCT_Module *LookUp::getModule(const std::string sn) {
    const SCTDAL::SCT_Module *modObj = m_conf.get<SCTDAL::SCT_Module>(sn);
    if(!modObj) {
      // Try named as "SCT_20220070100011"
      modObj = m_conf.get<SCTDAL::SCT_Module>("SCT_" + sn);
    }
    return modObj;
  }

  const SCTDAL::RodMUR *LookUp::getMurInRod(const SCTDAL::Rod *rodObj, int position) {
    const SCTDAL::RodMUR *murObj = 0;
    std::vector<const SCTDAL::RodMUR*> murs = rodObj->get_rodMURs();
    for(SCTDAL::RodMURIterator i = murs.begin(); i!=murs.end(); i++) {
      if((*i)->get_order() == position) {
	murObj = *i;
      }
    }

    return murObj;
  }

  const SCTDAL::Channel *LookUp::getChannelInRodMur(const SCTDAL::RodMUR *murObj, int position) {
    const SCTDAL::Channel *channelObj = 0;
    std::vector<const SCTDAL::Channel*> channels = murObj->get_channels();
    for(SCTDAL::ChannelIterator i = channels.begin(); i!=channels.end(); i++) {
      if((*i)->get_id() == position) {
	channelObj = *i;
      }
    }

    return channelObj;
  }

  const SCTDAL::RodRModule *LookUp::getRModuleInRodMur(const SCTDAL::RodMUR *murObj, int position) {
    const SCTDAL::RodRModule *mapObj = 0;
    std::vector<const SCTDAL::RodRModule*> maps = murObj->get_rodRModules();
    for(SCTDAL::RodRModuleIterator i = maps.begin(); i!=maps.end(); i++) {
      if((*i)->get_position() == position) {
	mapObj = *i;
      }
    }

    return mapObj;
  }

  const SCTDAL::PowerParam *LookUp::getPowerParamInChannel(const SCTDAL::PowerChannel *chanObj, std::string state, std::string name) {
    std::vector<const SCTDAL::PowerParam *> params = chanObj->get_params();

    const SCTDAL::PowerParam *paramObj = 0;
    for(SCTDAL::PowerParamIterator i = params.begin(); i!=params.end(); i++) {
      if((*i)->get_state() == state && (*i)->get_name() == name) {
	paramObj = *i;
	break;
      } 
    }

    return paramObj;
  }

  const SCTDAL::PowerCard *LookUp::getPowerCardInCrate(const SCTDAL::PowerCrate *crate, unsigned int card, std::string type) {
    std::vector<const SCTDAL::PowerCard *> cards = crate->get_cards();

    const SCTDAL::PowerCard *cardObj = 0;
    for(SCTDAL::PowerCardIterator i = cards.begin(); i!=cards.end(); i++) {
      if((*i)->get_id() == card && (*i)->get_type() == type) {
	cardObj = *i;
	break;
      }
    }

    return cardObj;
  }
}
