/*
 * CalibrationControllerPanel.java
 *
 * Created on 30 July 2003, 23:01
 */

package DisplayGUI;

import javax.swing.*;
import Sct_CalibrationController.*;
import Sct.IS.SctNames;
import is.*;

/**
 *
 * @author  palmer
 * adapted for use in new GUI by Dave
 */
public class CalibrationControllerPanel2 extends javax.swing.JPanel implements InfoListener, displayParams {
    
    gui guiControl;
    CalibrationControllerStatus status=null;
    java.util.regex.Pattern testdatapattern = java.util.regex.Pattern.compile(".*\\.(\\d+\\.\\d+.*)");
    java.util.regex.Matcher matcher;
    int currentBin=-1;
    int maxBin=-1;

	CalibrationControllerPanelMenu calibrationControllerPanelMenu;

    public CalibrationControllerPanel2(gui parent) {
        guiControl=parent;

		calibrationControllerPanelMenu = new CalibrationControllerPanelMenu(guiControl);

        setBackground(new java.awt.Color(255, 255, 255));
        is = GuiComponents.System.ISInterface.getInstance();
        initComponents();
        subscribeToServers();
        updateStatus();
        addMouseListener(new ccMouseAdapter());
    }
    
    public void subscribeToServers() {
        //Get Status from IS if it is there
	close();  //in case old subscriptions are still valid
        try {
            status = new CalibrationControllerStatus();
            is.getRepository().getValue(SctNames.getControlISServer() + ".Status", status);
            updateStatus();
        } catch (RuntimeException e) {
            status=null;
            System.err.println("Couldn't get CalibrationController Status: " + e.getMessage());
            updateStatus();
        }
        
        //Now subscribe to IS server
        try {
            is.subscribe(SctNames.getControlISServer(), "Status", false, this);
        } catch (RuntimeException e) {
            System.err.println("Couldn't subscribe to ControlData IS Server: " + e.getMessage());
        }

        try {
           is.subscribe("SCTAPIServer","currentBin",false,this);
           is.subscribe("SCTAPIServer","maxBin",false,this);
        } catch (RuntimeException e2) {
            System.err.println("Couldn't subscribe to SCTAPIServer: " + e2.getMessage());
        }

    }
    
    /**
     * Call this to get rid of any connections etc
     */
    void close() {
        //Unsubscribe from IS
        try {
            is.unsubscribe(SctNames.getControlISServer(), "Status");
        } catch (RuntimeException e) {
            System.err.println("Couldn't unsubscribe from ControlData IS Server: " + e.getMessage());
        }
        try {
            is.unsubscribe("SCTAPIServer","currentBin");
            is.unsubscribe("SCTAPIServer","maxBin");
        } catch (RuntimeException e2) {
            System.err.println("Couldn't unsubscribe from SCTAPIServer: " + e2.getMessage());
        }
    }
    

    private void initComponents() {//GEN-BEGIN:initComponents

        jLabel1 = new javax.swing.JLabel();
        statusField = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        testField = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        sequenceField = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        scanIndexField = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
//        currentBinField = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();
//        progressBar.setIndeterminate(true);
        progressBar.setStringPainted(true);
        progressBar.setString("");
        progressBar.setMaximum(100);
        progressBar.setValue(0);
//        progressBar.setString("50 out of 100");
        progressBar.setForeground(darkBlueColor);

 //       setLayout(new java.awt.GridLayout(5, 2));
        setLayout(new java.awt.GridBagLayout());
        setBackground(backgroundColor);

        jLabel1.setFont(displayCCBoldFont);
//        jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel1.setText("CC Status : ");
	jLabel1.setHorizontalAlignment(JLabel.RIGHT);
        java.awt.GridBagConstraints gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.weighty = 0.2;
	gridBagConstraints.weightx = 0.3;
//        gridBagConstraints.insets = new java.awt.Insets(10,10,0,10);
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        add(jLabel1, gridBagConstraints);


        statusField.setBackground(backgroundColor);
        statusField.setFont(displayCCFont);
        statusField.addMouseListener(new ccMouseAdapter());
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
	gridBagConstraints.weightx = 0.7;
//        gridBagConstraints.insets = new java.awt.Insets(10,10,0,10);
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        add(statusField, gridBagConstraints);

        jLabel2.setFont(displayCCBoldFont);
        jLabel2.setText("Current Test : ");
	jLabel2.setHorizontalAlignment(JLabel.RIGHT);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
	gridBagConstraints.weighty = 0.2;
//        gridBagConstraints.insets = new java.awt.Insets(10,10,0,10);
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        add(jLabel2, gridBagConstraints);

        testField.setBackground(backgroundColor);
	testField.setMaximumSize(new java.awt.Dimension(5,5));
        testField.setFont(displayCCFont);
        testField.addMouseListener(new ccMouseAdapter());
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
//        gridBagConstraints.insets = new java.awt.Insets(10,10,0,10);
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        add(testField, gridBagConstraints);

        jLabel3.setFont(displayCCBoldFont);
        jLabel3.setText("Current Sequence : ");
	jLabel3.setHorizontalAlignment(JLabel.RIGHT);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
	gridBagConstraints.weighty = 0.2;
//        gridBagConstraints.insets = new java.awt.Insets(10,10,0,10);
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        add(jLabel3, gridBagConstraints);

        sequenceField.setBackground(backgroundColor);
        sequenceField.setFont(displayCCFont);
        sequenceField.addMouseListener(new ccMouseAdapter());
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
//        gridBagConstraints.insets = new java.awt.Insets(10,10,0,10);
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        add(sequenceField, gridBagConstraints);

        jLabel4.setFont(displayCCBoldFont);
        jLabel4.setText("Current Scan Index : ");
	jLabel4.setHorizontalAlignment(JLabel.RIGHT);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
	gridBagConstraints.weighty = 0.2;
//        gridBagConstraints.insets = new java.awt.Insets(10,10,0,10);
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        add(jLabel4, gridBagConstraints);

        scanIndexField.setBackground(backgroundColor);
        scanIndexField.setFont(displayCCFont);
        scanIndexField.addMouseListener(new ccMouseAdapter());
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
//        gridBagConstraints.insets = new java.awt.Insets(10,10,0,10);
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        add(scanIndexField, gridBagConstraints);

        jLabel5.setFont(displayCCBoldFont);
        jLabel5.setText("Scan Progress : ");
	jLabel5.setHorizontalAlignment(JLabel.RIGHT);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
	gridBagConstraints.weighty = 0.2;
//        gridBagConstraints.insets = new java.awt.Insets(10,10,0,10);
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        add(jLabel5, gridBagConstraints);

        progressBar.setBackground(backgroundColor);
        progressBar.setFont(displayCCFont);
        progressBar.setBorderPainted(false);
        progressBar.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(1, 1, 1, 1)));
        progressBar.addMouseListener(new ccMouseAdapter());
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
//        gridBagConstraints.insets = new java.awt.Insets(10,10,0,10);
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        add(progressBar, gridBagConstraints);



       setBorder(new javax.swing.border.TitledBorder(null, "Scan Status", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Arial", 0, 11), new java.awt.Color(0, 0, 153)));
        
    }
    
//***********************************************************************************    
    void updateStatus() {
// run this in the eventDispatching thread, because it changes the gui display
       javax.swing.SwingUtilities.invokeLater(new Runnable() {
          public void run() {  

        if (status == null) {
            statusField.setText("UNAVAILABLE");
            statusField.setForeground(java.awt.Color.red);
            guiControl.menuActions.enableCCMenuItems(false);
        } else {
            switch (status.status) {
                case CalibrationControllerStatus.LOADED:
                    statusField.setText("LOADED");
                    statusField.setForeground(darkGreenColor); // dark green
                    guiControl.menuActions.enableCCMenuItems(false);
                    break;
                case CalibrationControllerStatus.INCONTROL:
                    statusField.setText("INCONTROL");
                    statusField.setForeground(darkGreenColor);
                    guiControl.menuActions.enableCCMenuItems(true);
                    break;
                case CalibrationControllerStatus.BUSY:
                    statusField.setText("BUSY");
                    statusField.setForeground(java.awt.Color.orange);
                    guiControl.menuActions.enableCCMenuItems(false);
                    break;
		case CalibrationControllerStatus.ERROR:
                    statusField.setText("ERROR");
                    statusField.setForeground(java.awt.Color.orange);
                    guiControl.menuActions.enableCCMenuItems(false);
                    break;
                default:
                    statusField.setText("UNKNOWN");
                    statusField.setForeground(java.awt.Color.red);
                    guiControl.menuActions.enableCCMenuItems(false);
            }

	    testField.setText(getLabel(status.currentTest));
            sequenceField.setText(getLabel(status.currentSequence));
            scanIndexField.setText(Integer.toString(status.currentScanIndex));
            calibrationControllerPanelMenu.checkCCUpdateMode(); // update status indicators
        }
          }
        });
    }

    public String getLabel(String original_string) {
	    matcher = testdatapattern.matcher(original_string);
	    String label = matcher.matches() ? original_string.substring(matcher.start(1),matcher.end(1)) : original_string;
	    if (label.length()>30) label = label.substring(0,30)+"...";
	    return label;
	    }


    void updateProgressBar() {
// run this in the eventDispatching thread, because it changes the gui display
       javax.swing.SwingUtilities.invokeLater(new Runnable() {
          public void run() {
            if(maxBin==-1) progressBar.setMaximum(1000);
            else progressBar.setMaximum(maxBin);
            if(currentBin==-1) {
               progressBar.setValue(0);
               progressBar.setString("");
               }
            else {
               progressBar.setValue(currentBin);
               if(maxBin==-1) progressBar.setString(Integer.toString(currentBin));
               else {
                     progressBar.setString(Integer.toString(currentBin)+" out of "+Integer.toString(maxBin)+" bins");
                     if(maxBin==currentBin) progressBar.setForeground(darkGreenColor);
                     else progressBar.setForeground(darkBlueColor);
                    }
            }
          }
        });
    }
    
    abstract class StringRunnable implements Runnable {
	protected is.InfoEvent infoEvent;
	StringRunnable(is.InfoEvent infoEvent) {
	    this.infoEvent = infoEvent;
	}
    }

    
    public void infoCreated(is.InfoEvent infoEvent) {
       javax.swing.SwingUtilities.invokeLater(new StringRunnable(infoEvent) {
          public void run() {
	    String name = infoEvent.getName();
            if(name.equals(SctNames.getControlISServer() + ".Status")) {
              try {
                 status = new CalibrationControllerStatus();
                 infoEvent.getValue(status);
              } catch (InfoNotCompatibleException e) {
	        status=null;
                System.err.println("Unable to read IS object - not a CalibrationControllerStatus: " + e.getMessage());
                }
	      updateStatus();
           }
        else { // SCTAPIServer
          try {
            is.AnyInfo theValue = new is.AnyInfo();
            infoEvent.getValue(theValue);
	    if(name.equals("SCTAPIServer.currentBin")) currentBin=((Integer) theValue.getAttribute(0)).intValue();
            else if(name.equals("SCTAPIServer.maxBin")) maxBin=((Integer) theValue.getAttribute(0)).intValue();
            updateProgressBar();
           }catch(Exception e2){ 
            System.err.println("Unable to update progressBar - "+e2.toString());
            }
           
           }
         }
        });
      

    }
    
    public void infoDeleted(is.InfoEvent infoEvent) {
       javax.swing.SwingUtilities.invokeLater(new StringRunnable(infoEvent) {
          public void run() {
             if(infoEvent.getName().equals(SctNames.getControlISServer() + ".Status")) {
               status=null;
               updateStatus();
               }
             else {
              currentBin=-1;
              maxBin=-1;
              updateProgressBar();
              }
        }
        });
    }
    
    //Pass to infoCreated
    public void infoUpdated(is.InfoEvent infoEvent) {
        infoCreated(infoEvent);
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables

    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel scanIndexField;
    private javax.swing.JLabel sequenceField;
    private javax.swing.JLabel statusField;
    private javax.swing.JLabel testField;
//    private javax.swing.JLabel currentBinField;
    private javax.swing.JProgressBar progressBar;

     private GuiComponents.System.ISInterface is;

    class ccMouseAdapter extends java.awt.event.MouseAdapter {
            public void mouseClicked(java.awt.event.MouseEvent e) {
               switch(e.getModifiers()) {
                 case java.awt.event.InputEvent.BUTTON2_MASK:
                 case java.awt.event.InputEvent.BUTTON3_MASK:             
                        JPopupMenu pMenu = calibrationControllerPanelMenu.getPopupMenu();
        				pMenu.show(e.getComponent(),e.getX(),e.getY());
//        				pMenu.setInvoker(guiControl.guiMenus.getCCPanelMenu());
                 break;
                 default:
                 }
            }
    }
 
}


