/*
 * IndexTable.java
 *
 *
 */

package DisplayGUI;

import javax.swing.JTable;
import javax.swing.event.*;
import javax.swing.ListSelectionModel;
import javax.swing.table.TableModel;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Collections;
import Sct_CalibrationController.TestData;
import GuiComponents.*;
/**
 *
 * @author  robinson
 */
public class IndexTable extends GuiTable implements TableModelListener,ListSelectionListener {

private TestSelection testSelection;
private IVTestSelection ivTestSelection;


public IndexTable(gui parent) {
	super(parent);
	guiControl=parent;
      table.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
     	table.addMouseListener(new tablesMouseAdapter());
      rowSM.addListSelectionListener(this);
      testSelection=null;
      ivTestSelection=null;
	}

public TestSelection getTestSelection() {
      return testSelection;
      }
public IVTestSelection getIVTestSelection() {
	return ivTestSelection;
      }
public void setNullTestSelection() {
	testSelection=null;
	ivTestSelection=null;
	}
public int getSelectedTest() {
	return rowSM.getMinSelectionIndex();
      }
public Vector getTestControlObjectNames() {
      return objectNames;
      }
public String getTestControlObjectName() {
      int selectedRow = rowSM.getMinSelectionIndex();
      if(selectedRow==-1) return null;
      if(objectNames==null) return null;
      if(selectedRow>=objectNames.size()) return null;
      return (String)objectNames.elementAt(selectedRow);
      }

    /** method required by ListSelectionListener interface */
public void valueChanged(ListSelectionEvent e) {
	if(!selectionListenerIsActive) {
//        selectionListenerIsActive=true;
          return;
          }

	if (e.getValueIsAdjusting()) return;

	ListSelectionModel lsm = (ListSelectionModel)e.getSource();

      if (lsm.isSelectionEmpty()) {
                 //no rows are selected
		testSelection=null;
 		ivTestSelection=null;
		guiControl.displayPane.setDisplayColors();
		guiControl.displayPane.refreshDisplay();
		guiControl.selectorPane.initParamList();
		return;
		}

     	int minRow=lsm.getMinSelectionIndex();
      int maxRow=lsm.getMaxSelectionIndex();
      int nselected = maxRow-minRow+1;

      int displayStatus=guiControl.displayStatus.getStatus();
      switch(displayStatus) {
		case DisplayStatus.MODULE_GROUP_SELECTION:
                   if(nselected>0) lsm.setSelectionInterval(minRow,minRow);
                   guiControl.selectorPane.setMouseClickIndex(minRow);
                   guiControl.displayPane.setDisplayColors();
                   guiControl.displayPane.refreshDisplay();
                   break;
            case DisplayStatus.IS_MONITOR:
// if more than one row is selected, assume they want to delete them
                   if(objectNames!=null && objectNames.size()>0) testSelection = new TestSelection((String)objectNames.elementAt(minRow),guiControl.isInterface.getLocation());

                   String runScanRegex = testSelection.getRunScanRegex();
                   if(runScanRegex!=null) guiControl.isInterface.setISCounter(runScanRegex);

                   guiControl.selectorPane.initParamList(); // maybe require change of parameter menu

                   if(testSelection.isRetrievedData()) guiControl.tablesDisplayPane.checkForRetrievedObjects();

                   guiControl.colorScalePane.setColorScale();
                   guiControl.displayPane.setDisplayColors();
                   ModuleCell lastClickedCell;
                   if((lastClickedCell=guiControl.displayPane.getLastClickedCell())!=null) guiControl.tablesDisplayPane.listISObjects(lastClickedCell);
                   guiControl.displayPane.refreshDisplay();
                   break;
		case DisplayStatus.TEST_DATA:
                   if(objectNames!=null && objectNames.size()>0) testSelection = new TestSelection((String)objectNames.elementAt(minRow),guiControl.isInterface.getLocation());
                   guiControl.selectorPane.initParamList(); // maybe require change of parameter menu

                   if(testSelection.isRetrievedData()) guiControl.tablesDisplayPane.checkForRetrievedObjects(2);   // publish test data object, if not already done

                   guiControl.colorScalePane.setColorScale();
                   guiControl.displayPane.setDisplayColors();
                   guiControl.displayPane.refreshDisplay();
                   guiControl.tablesDisplayPane.listTestResults();
                   break;
            case DisplayStatus.SCTDB_DATA:
                   if(objectNames!=null && objectNames.size()>0) testSelection = new TestSelection((String)objectNames.elementAt(minRow),guiControl.isInterface.getLocation());
                   guiControl.selectorPane.initParamList(); // maybe require change of parameter menu
                   guiControl.colorScalePane.setColorScale();
                   guiControl.displayPane.setDisplayColors();
                   guiControl.displayPane.refreshDisplay();
                   guiControl.tablesDisplayPane.listTestResults();
                   break;
            case DisplayStatus.SCTDB_RODDAQ_DATA:
                   if(objectNames!=null && objectNames.size()>0) {
		       	String objectName = (String)objectNames.elementAt(minRow);
			 	java.util.regex.Matcher matcher = ISInterface.SctRodDaqRefPattern.matcher(objectName);
	                	String locn = matcher.matches() ? objectName.substring(matcher.start(ISInterface.LOCATION),matcher.end(ISInterface.LOCATION)) : "Unknown";
		        	testSelection = new TestSelection(objectName,locn);
				}
                   guiControl.selectorPane.initParamList(); // maybe require change of parameter menu
                   guiControl.colorScalePane.setColorScale();
                   guiControl.displayPane.setDisplayColors();
                   guiControl.displayPane.refreshDisplay();
                   guiControl.tablesDisplayPane.listTestResults();
                   break;
            case DisplayStatus.IV_DATA:
                  if(objectNames!=null && objectNames.size()>0) ivTestSelection = new IVTestSelection((String)objectNames.elementAt(minRow));
                  guiControl.selectorPane.initParamList(); // maybe require change of parameter menu
                  guiControl.colorScalePane.setColorScale();
		   	guiControl.displayPane.setDisplayColors();
                  guiControl.displayPane.refreshDisplay();
                  guiControl.tablesDisplayPane.listTestResults();
                  break;
         	default:
            }
   }

   public Vector getTestTableList() {
// generate the test table list ordered by run numer and scan number
       Vector theLine = new Vector();        // each line of the table
       Vector tableContent = new Vector();   // vector of table lines
       Vector orderVector = new Vector();      // vector for re-ordering based on runno and scanno
       Hashtable orderHash = new Hashtable();// hashtable to map order runno/scanno to table lines
       Hashtable objectOrderHash = new Hashtable();

       boolean isSCTDBSctRodDaqData = (guiControl.displayStatus.getStatus()==DisplayStatus.SCTDB_RODDAQ_DATA);

       theLine.addElement("Test");
       theLine.addElement(isSCTDBSctRodDaqData ? "Locn" : "Status");
       theLine.addElement("Run");
       theLine.addElement("Scan");
       theLine.addElement("#Scans");
       theLine.addElement("Source");
       tableContent.addElement(theLine);
       int exceptionCount=0;
       for(int i=0;i<objectNames.size();i++) {
           String thisObject = (String)objectNames.elementAt(i);
           TestData testData = new TestData();
           try {
           GuiComponents.System.ISInterface.getInstance().getRepository().getValue(thisObject, testData);
           theLine=new Vector();
           if(testData.testName.equals("NPtGainTest") && testData.nScans==3) testData.testName = "3PtGainTest";
           theLine.addElement(testData.testName);
	   if(!isSCTDBSctRodDaqData) {
              switch(testData.status) {
                 case 0:
                   theLine.addElement("Executing");
                   break;
               case 1:
                 theLine.addElement("Completed");
                 break;
               case 2:
                 theLine.addElement("Aborted");
                 break;
               default:
                 theLine.addElement("Unknown");
               }
	   }
	   else {
	      java.util.regex.Matcher matcher = ISInterface.SctRodDaqRefPattern.matcher(thisObject);
	      theLine.addElement(matcher.matches() ? thisObject.substring(matcher.start(ISInterface.LOCATION),matcher.end(ISInterface.LOCATION)) : "Unknown");
	      }

           theLine.addElement(new Integer(testData.runNumber));
           theLine.addElement(new Integer(testData.startScanNumber));
           theLine.addElement(new Integer(testData.nScans));

           int offsetCount = (thisObject.matches("RetrievedData.*")) ? 0 : 100000000;
           Integer runScanNumber = new Integer(testData.runNumber*10000 + testData.startScanNumber +offsetCount);

           if(thisObject.matches("RetrievedData.*")) theLine.addElement("Archive");
           else if(thisObject.matches("SCTDBData.*")) theLine.addElement("SCTDB");
           else theLine.addElement("New");

           orderVector.addElement(runScanNumber);
           orderHash.put(runScanNumber,theLine);
           objectOrderHash.put(runScanNumber,thisObject);
           }catch(Exception e){exceptionCount++;}

           }

       Collections.sort(orderVector);
       objectNames=new Vector();
       for(int i=0;i<orderVector.size();i++) {
            Integer runscan= (Integer)orderVector.elementAt(i);
            objectNames.addElement((String)objectOrderHash.get(orderVector.elementAt(i)));          // re-order object names appropriately
            tableContent.addElement((Vector)orderHash.get(orderVector.elementAt(i))); // and the same for the table entries
            }
       if(exceptionCount>0) System.err.println("SctGUI::TablesDisplayPane - exception reading retrieving IS data.");
       return tableContent;
    }

  public Vector getIVScanList() {
// generate the test table list ordered by run numer and scan number
       Vector theLine = new Vector();        // each line of the table
       Vector tableContent = new Vector();   // vector of table lines
       Vector orderVector = new Vector();      // vector for re-ordering based on runno and scanno
       Hashtable orderHash = new Hashtable();// hashtable to map order runno/scanno to table lines
       Hashtable objectOrderHash = new Hashtable();

       theLine.addElement("Run");
       theLine.addElement("Scan");
       theLine.addElement("Date");
       theLine.addElement("Time");
       theLine.addElement("#Modules");
       theLine.addElement("Source");
       tableContent.addElement(theLine);
       for(int i=0;i<objectNames.size();i++) {
           String thisObject = (String)objectNames.elementAt(i);
           IVScanControl iv = new IVScanControl();
           GuiComponents.System.ISInterface.getInstance().getRepository().getValue(thisObject, iv);
           theLine=new Vector();
           theLine.addElement(new Integer(iv.runNo));
           theLine.addElement(new Integer(iv.scanNo));
           theLine.addElement(iv.scanDate);
           theLine.addElement(iv.scanTime);
           theLine.addElement(new Integer(iv.modules.length));

           int offsetCount = (thisObject.matches("RetrievedData.*")) ? 0 : 100000000;
           Integer runScanNumber = new Integer(iv.runNo*10000 + iv.scanNo +offsetCount); 

           if(thisObject.matches("SCTDBData.*")) theLine.addElement("SCTDB");
           else theLine.addElement("DCS");

           orderVector.addElement(runScanNumber);
           orderHash.put(runScanNumber,theLine);
           objectOrderHash.put(runScanNumber,thisObject);

           }

       Collections.sort(orderVector);
       objectNames=new Vector();
       for(int i=0;i<orderVector.size();i++) {
            Integer runscan= (Integer)orderVector.elementAt(i);
            objectNames.addElement((String)objectOrderHash.get(orderVector.elementAt(i)));          // re-order object names appropriately
            tableContent.addElement((Vector)orderHash.get(orderVector.elementAt(i))); // and the same for the table entries
            }
       return tableContent;
    }

    public void updateTest(String testObjectName) {
           if(objectNames==null) return;
           if(!objectNames.contains(testObjectName)) return;
           int rowNumber=objectNames.indexOf(testObjectName);
           try {
           TestData testData = new TestData();
           Sct.IS.SctNames.getISRepository().getValue(testObjectName, testData);
           switch(testData.status) {
               case 0:
                 resultTable.setValueAt("Executing",rowNumber,1);
                 break;
               case 1:
                 resultTable.setValueAt("Completed",rowNumber,1);
                 break;
               case 2:
                 resultTable.setValueAt("Aborted",rowNumber,1);
                 break;
               default:
                 resultTable.setValueAt("Unknown",rowNumber,1);
               }
             resultTable.fireTableCellUpdated(rowNumber,1);
            }catch(Exception updateE){System.err.println("***ERROR updating test status in table list : "+updateE.toString());}

         }

    public void updateTestTable(String testObjectName, int rowNumber) {
           if(rowNumber==-1) return;
           try {
           TestData testData = new TestData();
           Sct.IS.SctNames.getISRepository().getValue(testObjectName, testData);
           switch(testData.status) {
               case 0:
                 resultTable.setValueAt("Executing",rowNumber,1);
                 break;
               case 1:
                 resultTable.setValueAt("Completed",rowNumber,1);
                 break;
               case 2:
                 resultTable.setValueAt("Aborted",rowNumber,1);
                 break;
               default:
                 resultTable.setValueAt("Unknown",rowNumber,1);
               }
             resultTable.fireTableCellUpdated(rowNumber,1);
            }catch(Exception updateE){System.err.println("***ERROR updating test status in table list : "+updateE.toString());}
         }



    class tablesMouseAdapter extends java.awt.event.MouseAdapter {
            public void mouseClicked(java.awt.event.MouseEvent e) {
               switch(e.getModifiers()) {
                 case java.awt.event.InputEvent.BUTTON2_MASK:
                 case java.awt.event.InputEvent.BUTTON3_MASK:
                       int rowSelect = table.rowAtPoint(new java.awt.Point(e.getX(),e.getY()));
                       rowSM.setSelectionInterval(rowSelect,rowSelect);
                       guiControl.menuActions.showTablesPopupMenu(0,e);
                       break;
                 default:
                 }
            }
    }

}
