#!/usr/bin/perl -w
use Fcntl;
use strict;
use CGI qw(param);

use lib "/userdisk/daquser/perllibs2";
#use lib "/afs/cern.ch/user/s/sctroddq/scripts/perllibs2";
use webconfig;
use sctroddaqtests;


my ($outputfile,$resultHeader);
my @chipnames = qw {M0 S1 S2 S3 S4 E5 M8 S9 S10 S11 S12 E13};
my $noDataFlag=".";

my @tests=("FullBypass","Pipeline","StrobeDelay","3PtGain","TrimRange","NPtGain","NoiseOccupancy","TimeWalk","RxThresholdBasedOnConfigRegister",
				      "RxDelay","TxCurrent", "DoubleTriggerNoise","MarkSpaceRatio");
# my @referencefiles = ("HybDelay_Reference_warm.txt","Hyb3PGain_Reference_warm.txt","HybNoise_Reference_warm.txt");

################################################################# Do not edit anything after this line

my %testmap=();
my %datamap=();
my %corrmap=();
my %corr_runno=();

my %daqInfo=();
my %excludedruns=();
my %chipHash=();
my %defects=();
my $summaryDir ="/invalid/directoryname/";

&getConfiguration();

foreach my $dataset (keys %config) {

  %excludedruns=();

  if($config{$dataset}{ACTIVE}==0) {
     print "Dataset $config{$dataset}{TITLE} is not active. Skipping...\n";
     next;
     }  
     
  print "Dataset $config{$dataset}{TITLE} is ACTIVE\n";

  $summaryDir = "$summaryRootDirectory$config{$dataset}{DATADIR}/summaries/";

  my @eruns=split(/,/,$config{$dataset}{RUNEX});
  foreach (@eruns) {$excludedruns{$_}=1;}

  &readDaqTable($dataset);

  my $ntests=@tests;
  for(my $i=0;$i<$ntests;$i++) {

#     if($i==0) {next;}

     $outputfile = $tests[$i] . "_wwwsummary_${instituteName}_${dataset}.txt";
     $resultHeader = ($tests[$i] eq "3PtGain") ? "NPtGainTestResult" : $tests[$i] . "TestResult"; # 3PtGain filename is NPtGain!!
     %testmap=();
     %datamap=();
     %corrmap=();
     %corr_runno=();
     &collectLinks($dataset,$i);
#     &getReferenceData($i);       No longer inserted in lookup files
     &printDataList($dataset,$i);
   }
}


### finished

sub bailout {

    my $error = "@_";

    print <<bailout_marker;
    <b> &gt Request Failed!</b>
    <br><br>
      Reason: $error
    <br><br>
    Sorry!
<br><br>
</body></html>
bailout_marker

die "$error";

}

sub readDaqTable {

      my ($this_index) = @_;
      %daqInfo=();

      my $lookup = "${lookupDir}$config{$this_index}{MOUNTS}";
      
      print "Reading daq table from $lookup\n";

      if(!(-e "$lookup")) {return;}

      open(IN,$lookup) || printError("Cant open mount lookup file $lookup: $!");
      flock(IN,1) || printError("Cant get LOCK_SH on $lookup: $!");

      my @lines = <IN>;
      close(IN) || printError("Cant close $lookup: $!");
#      foreach my $newmodule (keys %addedModules{$this_index}) {push(@lines,"$newmodule=$addedModules{$this_index}{$newmodule}");
      my $sctType = $config{$this_index}{TYPE};
      if(exists $addedModules{$sctType}) {
         for my $module (keys %{ $addedModules{$sctType} } ) {
	     print "Adding extra module $module=$addedModules{$sctType}{$module}\n";
	     push(@lines,"$module=$addedModules{$sctType}{$module}");
	     }
         }

 #     push(@lines,"20220380200139=Barrel 3, Row 27, Position 6");

      foreach my $line (@lines) {
        if($line=~/(\d{14})=Barrel\s*(\d+),\s*Row\s*(\d+),\s*Position\s*([-\d]+)/) {
            $daqInfo{$1}="Barrel $2, Row $3, Position $4";
            }
          }
      foreach my $rmodule (keys %replacedModules) {
          print "checking if $rmodule is possible\n";
	  my $type=$config{$this_index}{TYPE};
          if((exists $replacedModules{$rmodule}{TYPE}) && $replacedModules{$rmodule}{TYPE}==$type) {
	     print "It IS...\n";
	     $daqInfo{$replacedModules{$rmodule}{LAST}} = $daqInfo{$rmodule};
	     print "$rmodule now in same location as $replacedModules{$rmodule}{LAST} from Run $replacedModules{$rmodule}{FROMRUN}\n";
	     }
          }
	  
      my $count = (keys %daqInfo);
      print "# of modules in $config{$this_index}{TITLE} is $count\n";

#      close(IN) || printError("Cant close $lookup: $!");
      }




sub collectLinks {

my ($dataset,$index) = @_;

print "Looking in $summaryDir for summary files...\n";

opendir(HDATA,$summaryDir) || bailout("Cant open directory $summaryDir: $!");

while(my $filename = readdir(HDATA)) {

   if($filename=~/$resultHeader\.(\d+)\.(\d+)\.(\d{14})\.summary.*/) {
   
 #     print "Verifying $filename ...\n";
      my $runno=$1;
      my $scanno = $2;
      my $sn = $3;

      if(!(exists $daqInfo{$sn})) {
        #print "$sn is not a mounted module. skipping...\n";
        next;
	}

      if($runno<$config{$dataset}{RUNLO}) {next;}
      if($runno>$config{$dataset}{RUNHI}) {next;}
      if(exists $excludedruns{$runno}) {print "$filename is excluded.skipping...\n"; next;}
      
      my $paddedRunno = addZeros($runno);
      my $logbookDir ="${lookupDir}$config{$dataset}{BKP_DIR}";
      my $trashfile = "${logbookDir}RUN${paddedRunno}/trashed.txt";

      if(-e $trashfile) {
         print "$filename is flagged as trash. Skipping...\n";
	 next;
	 }


      # Require 3PtGain only
      if($tests[$index] eq "3PtGain") {
        my $nscans = getScanCount("$summaryDir$filename");
	if($nscans>3) {print "$filename does not have 3 scan points. skipping...\n"; next;}
	}
      if($tests[$index] eq "NPtGain") {
        my $nscans = getScanCount("$summaryDir$filename");
	if($nscans<4) {print "$filename has less than 4 scan points. skipping...\n"; next;}
	}

      if(!exists $testmap{$sn}) {
 	 $testmap{$sn}=$filename;
	 next;
	 }

      my $runscankey = $runno * 1e6 + $scanno;

      my $oldfile = $testmap{$sn};
      if($oldfile=~/$resultHeader\.(\d+)\.(\d+)\.(\d{14})\.summary.*/) {
         my $thisrun = $1;
	 my $thisscan=$2;
	 my $thissn=$3;
	 my $thiskey = $thisrun * 1e6 + $thisscan;
         if($runscankey > $thiskey) {
	     $testmap{$sn}=$filename;
	     }
	 }
      }
  }

closedir(HDATA);
}



sub getConfigParams() {    # get cal corrs and BOC settings for run and sn.. this takes time!

 my ($dataset,$sn,$runno) = @_;

#     print "getting cal corrs and BOC settings for run $runno for module $sn\n";
#     print "===================================================\n";

     my $configfile = "$rootDirectory$config{$dataset}{DATADIR}/saved_data/configArchive_${runno}_start.xml.gz";

     if(!(-e $configfile)) {
        print "$sn : configfile $configfile does not exist\n";
	return "CFACTORS=CONF=";
	}


#     my $configfile = $dataDirectory . "configArchive_${runno}_start.xml";
     my $xmlcontent = `gunzip -c $configfile`;

     my @xml = split(/[\n\r]/,$xmlcontent);
     my $chipno=0;
     my $corrline="";
     my $foundit=0;
     my $confline="CONF=";
     my $id="xx";
     my $readdata=0;

     foreach my $line (@xml) {
       if($line=~/<module\s+id="(\d+)".*$sn.*/) {$id=$1; next;}
       if($line=~/<channel\s+id="${id}".*/) {$readdata=1;}
       if($readdata && $line=~/<\/channel>/) {$readdata=0; $id="xx"; next;}
       if($readdata) {
          if($line=~/<stream0\s+threshold="([0-9a-fA-Fx]+)"\s*delay="([0-9x]+)".*/) {
	      my $rx=getIntFromHex($1);
	      my $dl=getIntFromHex($2);
              $confline.=";RX0:$rx" . ";RXD0:$dl";
#	      print "confline=$confline\n";
	      }
         if($line=~/<stream1\s+threshold="([0-9a-fA-Fx]+)"\s*delay="([0-9x]+)".*/) {
              my $rx=getIntFromHex($1);
	      my $dl=getIntFromHex($2);
              $confline.=";RX1:$rx" . ";RXD1:$dl";
#	      print "confline=$confline\n";
	      }
         if($line=~/<output\s+current="([0-9a-fA-Fx]+)"\s*delay="([0-9x]+)".*/) {
              my $tx=getIntFromHex($1);
	      my $dl=getIntFromHex($2);
              $confline.="TX:$tx" . ";TXD:$dl";
#	      print "confline=$confline\n";
	      }
         }


       if($line=~/<sn>$sn<\/sn>.*/) {$foundit=1;}
       if($foundit==0) {next;}

       if($line=~/<c_factor>(.*)<\/c_factor>/) {
	  my $corrfactor=$1;
	  if($corrfactor=~/(^Z.*)/) {printf "**ERROR $sn $chipnames[$chipno] Cal factor $1\n";}
	  else {
	     if($corrline ne "") {$corrline .="*";}
	     $corrline .= "$corrfactor"; $chipno++;
	     }
	  }
       if($line=~/<\/module>.*/) {last;}
       }

      if($chipno!=12) {
             print "**ERROR - Run $runno module $sn chipno=$chipno\n";
	     return "CFACTORS=CONF=";
	     }

#	print "RUN $runno $sn corrline=$corrline\n";
	     return "CFACTORS=$corrline$confline";



}



sub printDataList {

my ($dataset,$index) = @_;

my $totalcount=0;
my $slopedefects=0;
my %defectCountHash=();


my $modulecount=0;
my $outfile = "${sctdaqrefDir}$outputfile";

if(!(-e $outfile)) {
  open(OUTF,">$outfile") || bailout("Couldnt open $outfile: $!");
  close(OUTF) || bailout("cant close $outfile after initial open: $!");
  }

sysopen(OUTF,$outfile, O_RDWR) || bailout("Cant sysopen $outfile : $!");
flock(OUTF,2) || bailout("Cant flock $outfile $!");
seek(OUTF,0,0) || bailout("Cant seek $outfile: $!");
truncate(OUTF,0) || bailout("Cant truncate $outfile $!");

print "Opened $outfile for writing...\n";


foreach my $sn (sort keys %testmap) {
   my $fname = "${summaryDir}$testmap{$sn}";
   my $runno=0;
   my $scanno=0;
   if($fname=~/.*${resultHeader}\.(\d+)\.(\d+)\.(\d{14})\.summary.*/) {$runno=$1;$scanno=$2;}

   my $CONFline = &getConfigParams($dataset,$sn,$runno);

 if(!-e $fname) {bailout("$fname does not exist.");}

 open(IN,$fname) || bailout("Cant open $fname $!");
 flock(IN,1) || bailout("Cant flock $fname $!");

 print "opened $fname \n";

 %chipHash=(); # hash of chipname to data for each chip, separated by '*'


 my $printedDCSInfo=0;
 my $printedDefectTable=0;
 my $delayData=0;
 my $printedTableData=0;

 my $nextLineIsDCS=0;

 my $nextLineIsDelays=0;

 my $printLines=0;

 my $firstchan="Unknown";
 my $defectname="";
 my %thisdefectcounts=();
 my $defectcount=0;

 my $FullBypassTest=0;
 my $nextchips="";

 my $scanCount=0;
 my @scanPoints=();

 my $table=0;
 my $nextLineIsTableData=0;
 my $addon=0;
 my $linkno=-1;
 my $nextLineIsRxDelays=0;

 my $DCSline="DCS=";
 my $dataline="DATA=";

 my $chip="NONE";
 my $dcsIndex=0;


 while(<IN>) {
   if(/^\s+(.*)/) {$_=$1;}   # get rid of first space, if there is one

   if(/#T0\s*T1.*|#VDET\s*IDET.*|#VCC\s*ICC.*|#VDD\s*IDD.*/) {$nextLineIsDCS=1; next;}


   if($nextLineIsDCS==1) {
     my ($d1,$d2) = split(/\s+/);
     if($dcsIndex>0) {$DCSline.="*";}
     $DCSline .= "$d1*$d2";
     if($dcsIndex==0) {     # for temperatures, ie the first 2 DCS data words, add the difference
        if($d1 ne "." && $d2 ne ".") {
          my $diff = abs($d1-$d2);
	  $diff = sprintf("%.2f",$diff);
	  $DCSline .="*$diff";
	  }
	else {$DCSline .="*.";}
	}
     $dcsIndex++;
     $nextLineIsDCS=0;
     }

# table data - decide on type
   if(/^#LoopB.*|^#\s*Offset\s*MeanOcc\s*RMSOcc\s*EstENC.*/) {
      $table=1;
      next;
      }  # LoopA is nptgain fits - ignore
   elsif(/^%OpeTool.*|^#ChipSlope.*/) {
      $table=0;
      $addon=1;
      next;
      }
   elsif(/^%RxThresholdBasedOnConfig.*|^%TxCurrentTest.*/) {
       $addon=1;
       next;
      }
   elsif(/#Average\s+Peak.*/) {
      $table=1;
      next;
      }
   elsif(/[#]?\s*range\s*target\s*ntrim\s*vt50\s*vt50rms\s*tr_off\s*off_rms\s*tr_step.*/) {
      $table=1;
      next;
      }

   if(/^#([MSE]+\d{1,2})$/ && ($table | $addon)) {
      $chip=$1;
      $nextLineIsTableData=1;
      next;
      }
   if($nextLineIsTableData && $addon) {
      my @dataparts = split(/\s+/);
      foreach my $d (@dataparts) {
        if($d !~/^#.*/) {addDataLine($chip,$d);}
	}
      $nextLineIsTableData=0;
      next;
      }
   if($table && (/^#$/ || /^%Defect.*/)) {
      $table=0;
      }
   if($addon  && (/^%Defect.*/ || /^#No defects.*/)) {
      $addon=0;
      }
   if(/^#\s*Link\s*number\s*(\d+).*/) {$linkno=$1;};
   if($addon && /^#\s*MinErrorFreeRxThresh\s*(-?\d+)\s*MaxErrorFreeRxThresh\s*(-?\d+)\s*BestRxThresh\s*(-?\d+).*/) {
      my $istart = ($linkno==0) ? 0 : 6;
      my $iend = $istart+6;
      for(my $i=$istart;$i<$iend;$i++) {
         $chip=$chipnames[$i];
         addDataLine($chip,$1);
         addDataLine($chip,$2);
         addDataLine($chip,$3);
	 }
      }
   if($addon && /^#\s*MinErrorFreeTxCurrent\s*(-?\d+)\s*MaxErrorFreeTxCurrent\s*(-?\d+)\s*BestTxCurrent\s*(-?\d+).*/) {
      my $istart = ($linkno==0) ? 0 : 6;
      my $iend = $istart+6;
      for(my $i=$istart;$i<$iend;$i++) {
         $chip=$chipnames[$i];
         addDataLine($chip,$1);
         addDataLine($chip,$2);
         addDataLine($chip,$3);
	 }
      }
   if($nextLineIsTableData) {
     my @dataline = split(/\s+/);
     foreach my $databit (@dataline) {
       addDataLine($chip,$databit);
       }
     $nextLineIsTableData=0;
     }

   if(/^#Delays$/) {$nextLineIsRxDelays=1; next;}
   if($nextLineIsRxDelays) {
     my @delays=split(/\s+/);
     for(my $y=0;$y<6;$y++) {
        addDataLine($chipnames[$y],$delays[0]);
        addDataLine($chipnames[6+$y],$delays[1]);
	}
     $nextLineIsRxDelays=0;
     }


   if(/#DELAY|%PipelineTest/) {
      $delayData=1;
      next;
      }
   if(/^#M0\s+S1\s+S2\s+S3\s+S4\s+E5/ && $delayData) {$nextLineIsDelays=1; next;}
   if(/^#M8\s+S9\s+S10\s+S11\s+S12\s+E13/ && $delayData) {$nextLineIsDelays=7; $delayData=0; next;}
   if($nextLineIsDelays) {
     my @delays = split(/\s+/);
     my $ycount=$nextLineIsDelays-1;
     foreach my $delay (@delays) {
        addDataLine($chipnames[$ycount],$delay);
	$ycount++;
	}
     $nextLineIsDelays=0;
     }
    if(/^%FullBypassTest.*/) {$FullBypassTest=1; next;}
    if(/^#([MS]\d+\s+[SE]\d+)$/ && $FullBypassTest) {
       $nextchips="$1";
       next;
       }
    if($nextchips=~/([MS]\d+)\s+([SE]\d+)/) {
       my @datas = split(/\s+/);
       addDataLine($1,$datas[0]);
       addDataLine($1,$datas[1]);
       addDataLine($2,$datas[2]);
       addDataLine($2,$datas[3]);
       $nextchips="";
       }
    if(/^#Comment$/) {$FullBypassTest=0; next;}

      if(/DEFECT\s*NAME\s*:\s*(.*)/) {
           if($1 =~ /.*SLOPE.*/) {
	      $slopedefects++;
	      next;
	      }
	   $defectname=$1;
	   next;
	   }
      if(/FIRST\s*CHANNEL\s*:\s*(\d+)/) {$firstchan=$1; next;}
      if(/LAST\s*CHANNEL\s*:\s*(\d+)/ && $defectname ne "") {
          if(!(exists $defectCountHash{$defectname})) {$defectCountHash{$defectname}=1;}
	  else {$defectCountHash{$defectname}++;}
	  $defectcount++;
	  $totalcount++;
          my $lastchan=$1;
	  for(my $y=$firstchan;$y<=$lastchan;$y++) {
	       if(!(exists $thisdefectcounts{$y})) {$thisdefectcounts{$y}=0;}
	       $thisdefectcounts{$y}++;
	       }
	  $defectname="";
	  }
   }
 close(IN) || bailout("Cant close $fname $!");

  my $cCount = (keys %thisdefectcounts);
  my $DATAline = getDataLine();
  print OUTF "RUN=${runno}SCAN=${scanno}SN=${sn}${DATAline}${DCSline}DEF=${defectcount}*${cCount}${CONFline}\n";
  $modulecount++;

#  if($modulecount>3) {last;}

  }

   foreach my $defect (sort keys %defectCountHash) {
     print OUTF "DEFECT${defect}=$defectCountHash{$defect}\n";
     }

   print OUTF "TOTALDEFECTCOUNT=${totalcount}\n";
   print OUTF "SLOPEDEFECTS=${slopedefects}\n";
    printf "Saved $tests[$index] data for $modulecount modules\n";
    close(OUTF) || bailout("Cant close $outfile : $!");

 }

sub getDataLine() {
  my $dataline="DATA=";
  foreach my $chip (@chipnames) {
    if(! exists $chipHash{$chip}) {
       print "*** no data for chip $chip \n";
       return "DATA=";
       }
    if($dataline ne "DATA=") {$dataline .= ";"}
    $dataline .= "${chip}:$chipHash{$chip}";
    }
    return $dataline;
    }

sub addDataLine() {
  my($chip,$data)=@_;
  if($chip eq "NONE") {
    print "**no chip for data $data \n";
    return;
    }
  if(! exists $chipHash{$chip}) {$chipHash{$chip}=$data;}
  else {$chipHash{$chip} .= "*$data";}
  }

sub getScanCount() {
 my $count=0;
 my $fullname = shift;

 open(IN,$fullname) || bailout("Cant open $fullname for scan count");
 flock(IN,1) || bailout("Cant flock $fullname");
 
 my $nextLineIsCount=0;

 while(<IN>) {
   if(/^[\s]?#N_POINTS/) {$nextLineIsCount=1; next;}
   if($nextLineIsCount) {
     if(/[\s]?(\d+)/) {
       $count=$1;
       last;
       }
     }
   }
 close(IN) || bailout("Cant close $fullname");
 return $count;
 }
 
 sub getIntFromHex {
   my $hex = shift;
   if($hex=~/(0x[a-fA-F\d]+)/) {
     return hex $hex;
     }
   return $hex;
   }








