#!/usr/bin/perl -w
use Fcntl;
use strict;
use CGI qw(param);

#use CGI::Carp qw(fatalsToBrowser);

use lib "/userdisk/daquser/perllibs2";
#use lib "/afs/cern.ch/user/s/sctroddq/scripts/perllibs2";
use webconfig;

my @sctViews = ("Barrel 3","Barrel 4","Barrel 5","Barrel 6");



################################################################# Do not edit anything after this line


my %daqInfo = ();
my %daqMapping=();
my $daqInfoExists=0;
my @barrelCount = (0,0,0,0);
my @barrelRows = (32,40,48,56);

my %moduleOrigin=();
my $originInfoExists=0;
my $moduleMountMapIsValid=0;

my $scriptname = $ENV{'SCRIPT_NAME'};  # name of this script for links etc

print "Content-type: text/html\n\n";

my $locn = param("locn") ? param("locn") : "Oxford";
if($locn ne "CERN" && $locn ne "Oxford") {printError("Unrecognised location");}
getConfiguration($locn);

my $showcurrentconfig = param("currentconfig") ? param("currentconfig") : 0;
my $index = param("index") ? param("index") : 0;


&readDaqTable();
&printBarrelFile();


sub printError {
 my ($reason) = @_;
 
   print qq!
  <html>
<head>
<title>Barrel Module Positions</title>
</head>
<body>

<h3><font color=red>Error</font></h3>
<br>
$reason
<br><br>

</body>
</html>
!;
die;
}




sub readDaqTable() {

     # config for that dataset or the current config?
     # ie moduleMounts0.txt or moduleMap0.txt, where 0=barrel3, 1-barrel4 etc

      my $barrel=$index+3;
      my $filepart = ($showcurrentconfig) ? "Barrel${barrel}/moduleMap" : "moduleMounts";
      my $lookup = "${lookupDir}${filepart}${index}.txt";

      if(!(-e "$lookup")) {return;}

      open(IN,$lookup) || printError("Cant open $lookup: $!");
      flock(IN,1) || printError("Cant get LOCK_SH on $lookup: $!");

      my @lines = <IN>;
      close(IN) || printError("Cant close $lookup: $!");
      my $sctType = $config{$index}{TYPE};

      if($showcurrentconfig==0 && exists $addedModules{$index}) {
         for my $module (keys %{ $addedModules{$index} } ) {
 #            print "Adding extra module $module=$addedModules{$index}{$module}\n";
	     push(@lines,"$module=$addedModules{$index}{$module}");
           $daqInfoExists++;
	     }
         }



     foreach (@lines) {
        if(/(\d{14})=Barrel\s*(\d+),\s*Row\s*(\d+),\s*Position\s*([-\d]+)/) {
            $daqInfo{$1}="Barrel $2, Row $3, Position $4";
            $barrelCount[$2-3]++;
            my $position = $4;
            $position+=6;
            if($position>=7) {$position--;}
            my $newpos = ($position + ($3 * 12))<<4;
            my $mindex = $2 - 3 + $newpos;
            $daqMapping{$mindex} = $1;
            $daqInfoExists++;
            }
          }
	  
      foreach my $rmodule (keys %replacedModules) {
          if((exists $replacedModules{$rmodule}{TYPE}) && $replacedModules{$rmodule}{TYPE}==$index) {
	     $daqInfo{$replacedModules{$rmodule}{LAST}} = $daqInfo{$rmodule};
	                $daqInfoExists++;
#	     print "$rmodule now in same location as $replacedModules{$rmodule}{LAST} from Run $replacedModules{$rmodule}{FROMRUN}\n";
	     }
          }

      }


sub printBarrelFile {

my $barrelIndex=$index;
my $rows = $barrelRows[$barrelIndex];

my $line = ($showcurrentconfig) ? " modules are in the current SctRodDaq Configuration for " : " modules have been assembled to ";

my $msize = (keys %daqMapping);
my $infoline = ($daqInfoExists>0) ? "<font color=navy><b>$msize $line $sctViews[$barrelIndex] :</b></font>" : "<font color=red><b>Sorry, the module assignment data for $sctViews[$barrelIndex] is not currently available</b></font>";

my $thisTitle = ($showcurrentconfig) ? "SctRodDaq Configuration Map" : " Module Mount Map";
print qq!
<html>
<head>
<title>Module Positions on Barrel $sctViews[$barrelIndex]</title>
<link rel=stylesheet HREF="http://www-pnp.physics.ox.ac.uk/~daquser/styles/davestyles2.css" type="text/css">
</head>
<body>
<center>
<h3> $sctViews[$barrelIndex] $thisTitle</h3>
!;

my $carg = ($showcurrentconfig) ? "&currentconfig=1&locn=$locn" : "";
for(my $i=0;$i<4;$i++) {
   my $bIndex = $i+3;
   my $link = ($i==$index) ? "Barrel $bIndex" : "<a href=$scriptname\?index=${i}${carg}>Barrel $bIndex</a>";
   if($i>0) {print "&nbsp;&nbsp;-&nbsp;&nbsp;";}
   print "$link";
   }
print qq!
<br><br>
$infoline
</center>
<br>

<table border=0 cellspacing=4 cellpadding=0>


<tr>
 <td nowrap width="50" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="10" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
</tr>

<tr><td nowrap></td><td nowrap colspan=13 align=center><b>Module Position</b></td></tr>

<tr>
<td nowrap></td>
<td nowrap align=center><b>-6</b></td>
<td nowrap align=center><b>-5</b></td>
<td nowrap align=center><b>-4</b></td>
<td nowrap align=center><b>-3</b></td>
<td nowrap align=center><b>-2</b></td>
<td nowrap align=center><b>-1</b></td>
<td nowrap></td>
<td nowrap align=center><b>1</b></td>
<td nowrap align=center><b>2</b></td>
<td nowrap align=center><b>3</b></td>
<td nowrap align=center><b>4</b></td>
<td nowrap align=center><b>5</b></td>
<td nowrap align=center><b>6</b></td>
</tr>
!;

for (my $row=0;$row<$rows; $row++) {
 my $lmt= $row + 1;
 print "<tr><td nowrap align=right><b>LMT $lmt: </b></td>\n";
 for (my $pos=0;$pos<12;$pos++) {
   my $posindex = ($pos + ($row * 12))<<4;
   my $index = $barrelIndex + $posindex;
   my $content = (exists $daqMapping{$index}) ? "<a name=$daqMapping{$index}></a>$daqMapping{$index}" : "";
   print "<td nowrap bgcolor=#eeeeff align=center>$content</td>";
   if($pos==5) {print "<td></td>";}
   }
 print "</tr>\n";
 }

print qq!
</table>
</body>
</html>
!;

}
