#!/usr/bin/perl -w

#
# Barrel Data display script
# Dave Robinson
#

use Fcntl;
use strict;
use CGI qw(param);
use lib "/userdisk/daquser/perllibs2";
#use lib "/afs/cern.ch/user/s/sctroddq/scripts/perllibs2";
use sctroddaqtests;
use webconfig;

use CGI::Carp qw(fatalsToBrowser);

my @chipNames = ("Module Average","Chip Maximum","Chip Minimum","Mean Link0","Mean Link1","Chip M0","Chip S1","Chip S2","Chip S3","Chip S4","Chip E5","Chip M8","Chip S9","Chip S10","Chip S11","Chip S12","Chip E13");

my @testIndexNames=("3PtGain","NoiseOccupancy","StrobeDelay","FullBypass","Pipeline","NPtGain","TrimRange","TimeWalk","RxThresholdBasedOnConfigRegister",
				      "RxDelay","TxCurrent", "DoubleTriggerNoise","MarkSpaceRatio");

my @refnames = ("Nothing","SCTDAQ Warm Reference","SCTDAQ Cold Reference");

my @pnames = (
  ["VT50(mV)","VT50(mV) RMS","Gain (mV/fC)","Gain RMS","Offset (mV)","Offset RMS","Output Noise (mV)","InputNoise(enc)","NoiseRMS(enc)","ChipSlope Noise","ChipSlope Offset","ChipSlope Gain"],
  ["Offset (mV)","Occupancy","RMS Occupancy","InputNoise(enc)","MaxBadness","NoThreshold"],
  ["Delay"],
  ["Token","RToken"],
  ["Working Pipelines"],
  ["VT50(mV)","VT50(mV) RMS","Gain (mV/fC)","Gain RMS","Offset (mV)","Offset RMS","Output Noise (mV)","InputNoise(enc)","NoiseRMS(enc)","ChipSlope Noise","ChipSlope Offset","ChipSlope Gain"],
  ["Trim Range","Target","N(trimmed)","VT50(mV)","VT50(mV) RMS","Offset (mV)","Offset RMS","Step size (mV)","Step Size RMS"],
  ["TimeWalk(ns)","TCal"],
  ["Min Rx Threshold","Max Rx Threshold","Best Rx Threshold"],
  ["Rx Delay"],
  ["Min Tx Current","Max Tx Current","Best Tx Current"],
  ["Average Occupancy","Peak Occupancy","Peak Ratio","Peak Bin"],
  ["Optimum MS Ratio"],
  );

my $separator = "------------";

my @defectnames=("Defect Count","Defective Channels");
my @DCSnames = ($separator,"T0","T1","TDiff","VDet","IDet(nA)","Vcc","Icc","Vdd","Idd");
my @BOCnames = ($separator,"BOC Tx","BOC TXDelay","BOC RX Thr stream0","BOC RX Dly Stream0","BOC RX Thr Stream1","BOC RX Dly Stream1");
my @cellfills = ("Data Value","Serial No","cFactor","None");
my $scriptname = $ENV{'SCRIPT_NAME'};  # name of this script for links etc

my $noDataString = "No Data";

my @barrelCount = (0,0,0,0);
my @barrelRows = (32,40,48,56);
my @parameterIndexNames = qw{enc occ};

my %colorHashC0 = (
   "VT50(mV)" => [0,0,0,255],
   "VT50(mV) Comparison" => [-10,0,0,255],
   "VT50(mV) RMS" => [0,0,0,255],
   "VT50(mV) RMS Comparison" => [-2,0,0,255],
   "Gain (mV/fC)" => [30,0,0,255],
   "Gain (mV/fC) Comparison" => [-5,0,0,255],
   "Gain RMS" => [0,0,0,255],
   "Gain RMS Comparison" => [-1,0,0,255],
   "Offset (mV)" => [-20,0,0,255],
   "Offset (mV) Comparison" => [-10,0,0,255],
   "Offset RMS" => [0,0,0,255],
   "Offset RMS Comparison" => [-2,0,0,255],
   "Output Noise (mV)" => [10,0,0,255],
   "Output Noise (mV) Comparison" => [-10,0,0,255],
   "InputNoise(enc)" => [1200, 0, 0, 255 ],    # label then rgb in 0-255
   "InputNoise(enc) Comparison" => [-150, 0, 0, 255 ],
   "NoiseRMS(enc)" => [0,0,0,255],
   "NoiseRMS(enc) Comparison" => [-15,0,0,255],
   "Trim Range" => [0,0,0,255],
   "Trim Range Comparison" => [-2,0,0,255],
   "Target" => [0,0,0,255],
   "Target Comparison" => [-20,0,0,255],
   "N(trimmed)" => [125,255,0,0],
   "N(trimmed) Comparison" => [-2,255,0,0],
   "Step size (mV)" => [0,0,0,255],
   "Step size (mV) Comparison" => [-10,0,0,255],
   "Step Size RMS" => [0,0,0,255],
   "Step Size RMS Comparison" => [-5,0,0,255],
   "ChipSlope Noise" => [-1.0,0,0,255],
   "ChipSlope Noise Comparison" => [-0.5,0,0,255],
   "ChipSlope Offset" => [-1.0,0,0,255],
   "ChipSlope Offset Comparison" => [-0.5,0,0,255],
   "ChipSlope Gain" => [-1.0,0,0,255],
   "ChipSlope Gain Comparison" => [-0.5,0,0,255],
   "Occupancy" => [0.0000001,0,0,255],
   "Occupancy Comparison" => [-0.0001,0,0,255],
   "RMS Occupancy" => [0.0000001,0,0,255],
   "RMS Occupancy Comparison" => [0.0001,0,0,255],
   "MaxBadness" => [0.0,255,0,0],
   "MaxBadness Comparison" => [-1.0,0,0,255],
   "NoThreshold" => [0,0,0,255],
   "NoThreshold Comparison" => [-10,0,0,255],
   "Delay" => [0,0,0,255],
   "Delay Comparison" => [-5,0,0,255],
   "Token" => [0,0,0,255],
   "Token Comparison" => [-2,0,0,255],
   "RToken" => [0,0,0,255],
   "RToken Comparison" => [-2,0,0,255],
   "Working Pipelines" => [125,255,0,0],
   "Working Pipelines Comparison" => [-2,255,0,0],
   "TimeWalk(ns)" => [0,255,0,0],
   "TimeWalk(ns) Comparison" => [-10,0,0,255],
   "TCal" => [0,255,0,0],
   "TCal Comparison" => [-5,0,0,255],
   "Min Rx Threshold" => [80,0,0,255],
   "Min Rx Threshold Comparison" => [-20,0,0,255],
   "Max Rx Threshold" => [80,0,0,255],
   "Max Rx Threshold Comparison" => [-20,0,0,255],
   "Best Rx Threshold" => [80,0,0,255],
   "Best Rx Threshold Comparison" => [-20,0,0,255],
   "Rx Delay" => [0,0,0,255],
   "Rx Delay Comparison" => [-10,0,0,255],
   "Min Tx Current" => [0,0,0,255],
   "Min Tx Current Comparison" => [-50,0,0,255],
   "Max Tx Current" => [0,0,0,255],
   "Max Tx Current Comparison" => [-50,0,0,255],
   "Best Tx Current" => [0,0,0,255],
   "Best Tx Current Comparison" => [-50,0,0,255],
   "Average Occupancy" => [0.0000001,0,0,255],
   "Average Occupancy Comparison" => [-0.0001,0,0,255],
   "Peak Occupancy" => [0.0000001,0,0,255],
   "Peak Occupancy Comparison" => [-0.0001,0,0,255],
   "Peak Ratio" => [0,0,0,255],
   "Peak Ratio Comparison" => [-2,0,0,255],
   "Peak Bin" => [0,0,0,255],
   "Peak Bin Comparison" => [-5,0,0,255],
   "Optimum MS Ratio" => [0,0,0,255],
   "Optimum MS Ratio Comparison" => [-10,0,0,255],
   "Defective Channels" => [0,0,255,0],
   "Defective Channels Comparison" => [-5,0,0,255],
   "Defect Count" => [0,0,255,0],
   "Defect Count Comparison" => [-5,0,0,255],
   "T0" => [0,0,0,255],
   "T0 Comparison" => [-5,0,0,255],
   "T1" => [0,0,0,255],
   "T1 Comparison" => [-5,0,0,255],
   "VDet" => [0,0,0,255],
   "VDet Comparison" => [-20,0,0,255],
   "IDet(nA)" => [0,0,0,255],
   "IDet(nA) Comparison" => [-150,0,0,255],
   "TDiff" => [0,0,255,0],
   "TDiff Comparison" => [-2,255,0,0],
   "Vdd" => [3.5,0,0,255],
   "Vdd Comparison" => [-0.5,0,0,255],
   "Idd" => [450,0,0,255],
   "Idd Comparison" => [-20,0,0,255],
   "Vcc" => [3.0,0,0,255],
   "Vcc Comparison" => [-0.5,0,0,255],
   "Icc" => [800,0,0,255],
   "Icc Comparison" => [-30,0,0,255],
   "BOC Tx" => [0,0,0,255],
   "BOC Tx Comparison" => [-20,0,0,255],
   "BOC TXDelay" => [0,0,255,0],
   "BOC TXDelay Comparison" => [-10,0,0,255],
   "BOC RX Thr stream0" => [0,0,0,255],
   "BOC RX Thr stream0 Comparison" => [-20,0,0,255],
   "BOC RX Dly Stream0" => [0,0,255,0],
   "BOC RX Dly Stream0 Comparison" => [-10,0,0,255],
   "BOC RX Thr Stream1" => [0,0,0,255],
   "BOC RX Thr Stream1 Comparison" => [-20,0,0,255],
   "BOC RX Dly Stream1" => [0,0,255,0],
   "BOC RX Dly Stream1 Comparison" => [-10,0,0,255]
   );

my %colorHashC1 = (
   "VT50(mV)" => [100,0,255,0],
   "VT50(mV) Comparison" => [-5,0,255,0],
   "VT50(mV) RMS" => [1,0,255,0],
   "VT50(mV) RMS Comparison" => [-1,0,255,0],
   "Gain (mV/fC)" => [40,0,255,0],
   "Gain (mV/fC) Comparison" => [-2,0,255,0],
   "Gain RMS" => [1,0,255,0],
   "Gain RMS Comparison" => [-0.5,0,255,0],
   "Offset (mV)" => [-5,0,255,0],
   "Offset (mV) Comparison" => [-5,0,255,0],
   "Offset RMS" => [1,0,255,0],
   "Offset RMS Comparison" => [-1,0,255,0],
   "Output Noise (mV)" => [12,0,255,0],
   "Output Noise (mV) Comparison" => [-5,0,255,0],
   "InputNoise(enc)" => [1300, 0, 255, 0 ],    # label then rgb in 0-255
   "InputNoise(enc) Comparison" => [-50, 0, 255, 0 ],
   "NoiseRMS(enc)" => [30,0,255,0],
   "NoiseRMS(enc) Comparison" => [-8,0,255,0],
   "Trim Range" => [1,0,255,0],
   "Trim Range Comparison" => [-1,255,255,0],
   "Target" => [50,0,255,0],
   "Target Comparison" => [-10,0,255,0],
   "N(trimmed)" => [126,255,255,0],
   "N(trimmed) Comparison" => [-1,255,255,0],
   "Step size (mV)" => [3,0,255,0],
   "Step size (mV) Comparison" => [-5,0,255,0],
   "Step Size RMS" => [0.25,0,255,0],
   "Step Size RMS Comparison" => [-2,0,255,0],
   "ChipSlope Noise" => [-0.5,0,255,0],
   "ChipSlope Noise Comparison" => [-0.2,0,255,0],
   "ChipSlope Offset" => [-0.5,0,255,0],
   "ChipSlope Offset Comparison" => [-0.2,0,255,0],
   "ChipSlope Gain" => [-0.5,0,255,0],
   "ChipSlope Gain Comparison" => [-0.2,0,255,0],
   "Occupancy" => [0.000001,0,255,0],
   "Occupancy Comparison" => [-0.00001,0,255,0],
   "RMS Occupancy" => [0.000001,0,255,0],
   "RMS Occupancy Comparison" => [-0.00001,0,255,0],
   "MaxBadness" => [0.5,0,255,0],
   "MaxBadness Comparison" => [-0.5,0,255,0],
   "NoThreshold" => [10,0,255,0],
   "NoThreshold Comparison" => [-5,0,255,0],
   "Delay" => [10,0,255,0],
   "Delay Comparison" => [-2,0,255,0],
   "Token" => [2,255,0,0],
   "Token Comparison" => [-1,0,255,0],
   "RToken" => [2,255,0,0],
   "RToken Comparison" => [-1,0,255,0],
   "Working Pipelines" => [126,255,255,0],
   "Working Pipelines Comparison" => [-1,255,255,0],
   "TimeWalk(ns)" => [12,0,255,0],
   "TimeWalk(ns) Comparison" => [-5,0,255,0],
   "TCal" => [10,0,255,0],
   "TCal Comparison" => [-2,0,255,0],
   "Min Rx Threshold" => [120,0,255,0],
   "Min Rx Threshold Comparison" => [-10,0,255,0],
   "Max Rx Threshold" => [120,0,255,0],
   "Max Rx Threshold Comparison" => [-10,0,255,0],
   "Best Rx Threshold" => [120,0,255,0],
   "Best Rx Threshold Comparison" => [-10,0,255,0],
   "Rx Delay" => [6,0,255,0],
   "Rx Delay Comparison" => [-5,0,255,0],
   "Min Tx Current" => [100,0,255,0],
   "Min Tx Current Comparison" => [-10,0,255,0],
   "Max Tx Current" => [100,0,255,0],
   "Max Tx Current Comparison" => [-10,0,255,0],
   "Best Tx Current" => [100,0,255,0],
   "Best Tx Current Comparison" => [-10,0,255,0],
   "Average Occupancy" => [0.000001,0,255,0],
   "Average Occupancy Comparison" => [-0.00001,0,255,0],
   "Peak Occupancy" => [0.000001,0,255,0],
   "Peak Occupancy Comparison" => [-0.00001,0,255,0],
   "Peak Ratio" => [1,0,255,0],
   "Peak Ratio Comparison" => [-1,255,255,0],
   "Peak Bin" => [60,0,255,0],
   "Peak Bin Comparison" => [-2,0,255,0],
   "Optimum MS Ratio" => [10,0,255,0],
   "Optimum MS Ratio Comparison" => [-5,0,255,0],
   "Defective Channels" => [4,255,255,0],
   "Defective Channels Comparison" => [-1,0,255,0],
   "Defect Count" => [4,255,255,0],
   "Defect Count Comparison" => [-2,0,255,0],
   "T0" => [20,0,255,0],
   "T0 Comparison" => [-2,0,255,0],
   "T1" => [20,0,255,0],
   "T1 Comparison" => [-2,0,255,0],
   "VDet" => [150,0,255,0],
   "VDet Comparison" => [-10,0,255,0],
   "IDet(nA)" => [400,0,255,0],
   "IDet(nA) Comparison" => [-50,0,255,0],
   "TDiff" => [1,255,255,0],
   "TDiff Comparison" => [-0.5,0,255,0],
   "Vdd" => [3.8,0,255,0],
   "Vdd Comparison" => [-0.2,0,255,0],
   "Idd" => [500,0,255,0],
   "Idd Comparison" => [-10,0,255,0],
   "Vcc" => [3.4,0,255,0],
   "Vcc Comparison" => [-0.2,0,255,0],
   "Icc" => [950,0,255,0],
   "Icc Comparison" => [-10,0,255,0],
   "BOC Tx" => [100,0,255,0],
   "BOC Tx Comparison" => [-10,0,255,0],
   "BOC TXDelay" => [10,255,255,0],
   "BOC TXDelay Comparison" => [-5,0,255,0],
   "BOC RX Thr stream0" => [100,0,255,0],
   "BOC RX Thr stream0 Comparison" => [-10,0,255,0],
   "BOC RX Dly Stream0" => [10,255,255,0],
   "BOC RX Dly Stream0 Comparison" => [-5,0,255,0],
   "BOC RX Thr Stream1" => [100,0,255,0],
   "BOC RX Thr stream1 Comparison" => [-10,0,255,0],
   "BOC RX Dly Stream1" => [10,255,255,0],
   "BOC RX Dly Stream1 Comparison" => [-5,0,255,0],
   );
my %colorHashC2 = (
   "VT50(mV)" => [200,0,255,0],
   "VT50(mV) Comparison" => [5,0,255,0],
   "VT50(mV) RMS" => [2,0,255,0],
   "VT50(mV) RMS Comparison" => [1,0,255,0],
   "Gain (mV/fC)" => [60,0,255,0],
   "Gain (mV/fC) Comparison" => [2,0,255,0],
   "Gain RMS" => [5,0,255,0],
   "Gain RMS Comparison" => [0.5,0,255,0],
   "Offset (mV)" => [40,0,255,0],
   "Offset (mV) Comparison" => [5,0,255,0],
   "Offset RMS" => [2,0,255,0],
   "Offset RMS Comparison" => [1,0,255,0],
   "Output Noise (mV)" => [14,0,255,0],
   "Output Noise (mV) Comparison" => [5,0,255,0],
   "InputNoise(enc)" => [1600, 0, 255, 0 ],    # label then rgb in 0-255
   "InputNoise(enc) Comparison" => [50, 0, 255, 0 ],
   "NoiseRMS(enc)" => [80,0,255,0],
   "NoiseRMS(enc) Comparison" => [8,0,255,0],
   "Trim Range" => [2,255,255,0],
   "Trim Range Comparison" => [0,0,255,0],
   "Target" => [200,0,255,0],
   "Target Comparison" => [10,0,255,0],
   "N(trimmed)" => [127,255,255,0],
   "N(trimmed) Comparison" => [0,0,255,0],
   "Step size (mV)" => [6,0,255,0],
   "Step size (mV) Comparison" => [5,0,255,0],
   "Step Size RMS" => [0.75,0,255,0],
   "Step Size RMS Comparison" => [2,0,255,0],
   "ChipSlope Noise" => [0.5,0,255,0],
   "ChipSlope Noise Comparison" => [0.2,0,255,0],
   "ChipSlope Offset" => [0.5,0,255,0],
   "ChipSlope Offset Comparison" => [0.2,0,255,0],
   "ChipSlope Gain" => [0.5,0,255,0],
   "ChipSlope Gain Comparison" => [0.2,0,255,0],
   "Occupancy" => [0.00001,0,255,0],
   "Occupancy Comparison" => [0.00001,0,255,0],
   "RMS Occupancy" => [0.00001,0,255,0],
   "RMS Occupancy Comparison" => [0.00001,0,255,0],
   "MaxBadness" => [2.0,255,255,0],
   "MaxBadness Comparison" => [0.5,0,255,0],
   "NoThreshold" => [40,0,255,0],
   "NoThreshold Comparison" => [5,0,255,0],
   "Delay" => [20,0,255,0],
   "Delay Comparison" => [2,0,255,0],
   "Token" => [3,255,255,0],
   "Token Comparison" => [1,0,255,0],
   "RToken" => [3,255,255,0],
   "RToken Comparison" => [1,0,255,0],
   "Working Pipelines" => [127,255,255,0],
   "Working Pipelines Comparison" => [0,0,255,0],
   "TimeWalk(ns)" => [15,0,255,0],
   "TimeWalk(ns) Comparison" => [5,0,255,0],
   "TCal" => [30,0,255,0],
   "TCal Comparison" => [2,0,255,0],
   "Min Rx Threshold" => [160,0,255,0],
   "Min Rx Threshold Comparison" => [10,0,255,0],
   "Max Rx Threshold" => [160,0,255,0],
   "Max Rx Threshold Comparison" => [10,0,255,0],
   "Best Rx Threshold" => [160,0,255,0],
   "Best Rx Threshold Comparison" => [10,0,255,0],
   "Rx Delay" => [18,255,255,0],
   "Rx Delay Comparison" => [5,0,255,0],
   "Min Tx Current" => [180,0,255,0],
   "Min Tx Current Comparison" => [10,0,255,0],
   "Max Tx Current" => [180,0,255,0],
   "Max Tx Current Comparison" => [10,0,255,0],
   "Best Tx Current" => [180,0,255,0],
   "Best Tx Current Comparison" => [10,0,255,0],
   "Average Occupancy" => [0.00001,0,255,0],
   "Average Occupancy Comparison" => [0.00001,0,255,0],
   "Peak Occupancy" => [0.00001,0,255,0],
   "Peak Occupancy Comparison" => [0.00001,0,255,0],
   "Peak Ratio" => [2,255,255,0],
   "Peak Ratio Comparison" => [1,0,255,0],
   "Peak Bin" => [100,255,255,0],
   "Peak Bin Comparison" => [2,0,255,0],
   "Optimum MS Ratio" => [20,255,255,0],
   "Optimum MS Ratio Comparison" => [5,0,255,0],
   "Defective Channels" => [8,255,255,0],
   "Defective Channels Comparison" => [1,0,255,0],
   "Defect Count" => [8,255,255,0],
   "Defect Count Comparison" => [2,0,255,0],
   "T0" => [30,0,255,0],
   "T0 Comparison" => [2,0,255,0],
   "T1" => [30,0,255,0],
   "T1 Comparison" => [2,0,255,0],
   "TDiff" => [2,255,255,0],
   "TDiff Comparison" => [0.5,0,255,0],
   "VDet" => [350,0,255,0],
   "VDet Comparison" => [10,0,255,0],
   "IDet(nA)" => [600,0,255,0],
   "IDet(nA) Comparison" => [20,0,255,0],
   "Vdd" => [4.2,0,255,0],
   "Vdd Comparison" => [0.2,0,255,0],
   "Idd" => [550,0,255,0],
   "Idd Comparison" => [10,0,255,0],
   "Vcc" => [3.6,0,255,0],
   "Vcc Comparison" => [0.2,0,255,0],
   "Icc" => [1050,0,255,0],
   "Icc Comparison" => [10,0,255,0],
   "BOC Tx" => [160,0,255,0],
   "BOC Tx Comparison" => [10,0,255,0],
   "BOC TXDelay" => [16,255,255,0],
   "BOC TXDelay Comparison" => [5,0,255,0],
   "BOC RX Thr stream0" => [140,0,255,0],
   "BOC RX Thr stream0 Comparison" => [10,0,255,0],
   "BOC RX Dly Stream0" => [16,255,255,0],
   "BOC RX Dly Stream0 Comparison" => [5,0,255,0],
   "BOC RX Thr Stream1" => [140,0,255,0],
   "BOC RX Thr stream1 Comparison" => [10,0,255,0],
   "BOC RX Dly Stream1" => [16,255,255,0],
   "BOC RX Dly Stream1 Comparison" => [5,0,255,0],
   );

my %colorHashC3 = (
   "VT50(mV)" => [300,255,0,0],
   "VT50(mV) RMS" => [4,255,0,0],
   "Gain (mV/fC)" => [70,255,0,0],
   "Gain RMS" => [10,255,0,0],
   "Offset (mV)" => [50,255,0,0],
   "Offset RMS" => [4,255,0,0],
   "Output Noise (mV)" => [16,255,0,0],
   "InputNoise(enc)" => [2000, 255, 0, 0 ],    # label then rgb in 0-255
   "NoiseRMS(enc)" => [100,255,0,0],
   "Trim Range" => [3,255,0,0],
   "Target" => [250,255,0,0],
   "N(trimmed)" => [128,0,255,0],
   "Step size (mV)" => [9,255,0,0],
   "Step Size RMS" => [1.0,255,0,0],
   "ChipSlope Noise" => [1.0,255,0,0],
   "ChipSlope Offset" => [1.0,255,0,0],
   "ChipSlope Gain" => [1.0,255,0,0],
   "Occupancy" => [0.0001,255,0,0],
   "RMS Occupancy" => [0.0001,255,0,0],
   "MaxBadness" => [2.5,255,0,0],
   "NoThreshold" => [50,255,0,0],
   "Delay" => [30,255,0,0],
   "Token" => [4,0,255,0],
   "RToken" => [4,0,255,0],
   "Working Pipelines" => [128,0,255,0],
   "TimeWalk(ns)" => [20,255,0,0],
   "TCal" => [40,255,0,0],
   "Min Rx Threshold" => [240,255,0,0],
   "Max Rx Threshold" => [240,255,0,0],
   "Best Rx Threshold" => [240,255,0,0],
   "Rx Delay" => [24,255,0,0],
   "Min Tx Current" => [240,255,0,0],
   "Max Tx Current" => [240,255,0,0],
   "Best Tx Current" => [240,255,0,0],
   "Average Occupancy" => [0.0001,255,0,0],
   "Peak Occupancy" => [0.0001,255,0,0],
   "Peak Ratio" => [3,255,0,0],
   "Peak Bin" => [128,255,0,0],
   "Optimum MS Ratio" => [30,255,0,0],
   "Defective Channels" => [12,255,0,0],
   "Defect Count" => [12,255,0,0],
   "T0" => [35,255,0,0],
   "T1" => [35,255,0,0],
   "TDiff" => [4,255,0,0],
   "VDet" => [500,255,0,0],
   "IDet(nA)" => [1000,255,0,0],
   "Vdd" => [4.5,255,0,0],
   "Idd" => [600,255,0,0],
   "Vcc" => [4.0,255,0,0],
   "Icc" => [1200,255,0,0],
   "BOC Tx" => [200,255,0,0],
   "BOC TXDelay" => [20,255,0,0],
   "BOC RX Thr stream0" => [200,255,0,0],
   "BOC RX Dly Stream0" => [20,255,0,0],
   "BOC RX Thr Stream1" => [200,255,0,0],
   "BOC RX Dly Stream1" => [20,255,0,0],
   "VT50(mV) Comparison" => [10,255,0,0],
   "VT50(mV) RMS Comparison" => [2,255,0,0],
   "Gain (mV/fC) Comparison" => [5,255,0,0],
   "Gain RMS Comparison" => [1,255,0,0],
   "Offset (mV) Comparison" => [10,255,0,0],
   "Offset RMS Comparison" => [2,255,0,0],
   "Output Noise (mV) Comparison" => [10,255,0,0],
   "InputNoise(enc) Comparison" => [150, 255,0,0 ],
   "NoiseRMS(enc) Comparison" => [15,255,0,0],
   "Trim Range Comparison" => [2,255,0,0],
   "Target Comparison" => [20,255,0,0],
   "N(trimmed) Comparison" => [1,255,0,0],
   "Step size (mV) Comparison" => [10,255,0,0],
   "Step Size RMS Comparison" => [5,255,0,0],
   "ChipSlope Noise Comparison" => [0.5,255,0,0],
   "ChipSlope Offset Comparison" => [0.5,255,0,0],
   "ChipSlope Gain Comparison" => [0.5,255,0,0],
   "Occupancy Comparison" => [0.0001,255,0,0],
   "RMS Occupancy Comparison" => [0.0001,255,0,0],
   "MaxBadness Comparison" => [1.0,255,0,0],
   "NoThreshold Comparison" => [10,255,0,0],
   "Delay Comparison" => [5,255,0,0],
   "Token Comparison" => [2,255,0,0],
   "RToken Comparison" => [2,255,0,0],
   "Working Pipelines Comparison" => [1,255,0,0],
   "TimeWalk(ns) Comparison" => [10,255,0,0],
   "TCal Comparison" => [5,255,0,0],
   "Min Rx Threshold Comparison" => [20,255,0,0],
   "Max Rx Threshold Comparison" => [20,255,0,0],
   "Best Rx Threshold Comparison" => [20,255,0,0],
   "Rx Delay Comparison" => [10,255,0,0],
   "Min Tx Current Comparison" => [50,255,0,0],
   "Max Tx Current Comparison" => [50,255,0,0],
   "Best Tx Current Comparison" => [50,255,0,0],
   "Average Occupancy Comparison" => [0.0001,255,0,0],
   "Peak Occupancy Comparison" => [0.0001,255,0,0],
   "Peak Ratio Comparison" => [2,255,0,0],
   "Peak Bin Comparison" => [5,255,0,0],
   "Optimum MS Ratio Comparison" => [10,255,0,0],
   "Defective Channels Comparison" => [5,255,0,0],
   "Defect Count Comparison" => [5,255,0,0],
   "T0 Comparison" => [5,255,0,0],
   "T1 Comparison" => [5,255,0,0],
   "VDet Comparison" => [20,255,0,0],
   "IDet(nA) Comparison" => [150,255,0,0],
   "TDiff Comparison" => [2,255,0,0],
   "Vdd Comparison" => [0.5,255,0,0],
   "Idd Comparison" => [20,255,0,0],
   "Vcc Comparison" => [0.5,255,0,0],
   "Icc Comparison" => [30,255,0,0],
   "BOC Tx Comparison" => [20,255,0,0],
   "BOC TXDelay Comparison" => [10,255,0,0],
   "BOC RX Thr stream0 Comparison" => [20,255,0,0],
   "BOC RX Dly Stream0 Comparison" => [10,255,0,0],
   "BOC RX Thr Stream1 Comparison" => [20,255,0,0],
   "BOC RX Dly Stream1 Comparison" => [10,255,0,0]
   );




################################################################# Do not edit anything after this line

print "Content-type: text/html\n\n";

my $locn = param("locn") ? param("locn") : "Oxford";
if($locn ne "CERN" && $locn ne "Oxford") {printError("Unrecognised location");}
&getConfiguration($locn);

my $logScript = "${cgi_base}sctroddaqsummary.cgi";

if(-e "${lookupDir}maintenancefile.txt") {
 print qq!
 <html>
 <head>
 <title>Barrel Display not available</title>
 </head>
 <body>
 <h3>The Barrel Display is not available</h3>
 Sorry, the barrel data display is not available, due to maintenance.
 <br><br>
 Please try again in a few moments.
 <br><br>
 If this message persists, please <a href="mailto:robinson\@hep.phy.cam.ac.uk">complain</a>.
 </body>
 </html>
 !;
 die;
 }

my %daqInfo = ();
my %daqMapping=();
my %dataMap=();    # map of sn to data value
my %cMap=();       # map of sn to cfactor
my %refMap=();     # map of sn to reference data (if used)
my %defectCount=();
my $totaldefectcount=-1;
my $daqInfoExists=0;
my %replacementModules=();

my $slopedefects=0;

my %datalinks=();
my $originInfoExists=0;
my $moduleMountMapIsValid=0;

my $moduleCount=0;

# testIndex = index of test name
# parameterIndex = index of displayed parameter
# chip = index of chip option

my $testIndex = param("testIndex") ? param("testIndex") : 0;
my $parameterIndex = param("parameterIndex") ? param("parameterIndex") : 0;
my $chipIndex = param("chipIndex") ? param("chipIndex") : 0;
my $cellfillIndex = param("cellfillIndex") ? param("cellfillIndex") : 0;
my $ndsets = (keys %config);
my $dataSet = param("dataSet") ? param("dataSet") : $ndsets;
my $compIndex = param("compIndex") ? param("compIndex") : 0;

my @paramlist = (@{$pnames[$testIndex]},@defectnames,@DCSnames,@BOCnames);
if($parameterIndex>@paramlist) {$parameterIndex=0;}

if($paramlist[$parameterIndex] eq $separator) {$parameterIndex--;}

my $isOccupancyData = ($pnames[$testIndex][$parameterIndex]=~/.*Occupancy.*/);

my $dataPointer = $parameterIndex;

my $fname = "${sctdaqrefDir}$testIndexNames[$testIndex]_wwwsummary_${instituteName}_$dataSet.txt";

# sort out refernce files, if any
my @reffiles = ("None");
my $reffname = "";
my $barrelnum = $config{$dataSet}{TYPE} + 3;
push(@reffiles,"${sctdaqrefDir}$testIndexNames[$testIndex]_Barrel${barrelnum}_Reference_warm.txt");
push(@reffiles,"${sctdaqrefDir}$testIndexNames[$testIndex]_Barrel${barrelnum}_Reference_cold.txt");
my $reftitles = $config{$dataSet}{REFS};
my @reft = split(/,/,$reftitles);
if(@reft>0) {
  my @extra_fnames = split(/,/,$config{$dataSet}{REFFILES});
  @refnames=(@refnames,@reft);
  foreach my $f (@extra_fnames) {
     my $efile = "${sctdaqrefDir}$testIndexNames[$testIndex]_wwwsummary_${f}.txt";
     push(@reffiles,$efile);
     }
  }

my $fcount= @refnames - 1;
if($compIndex>$fcount) {$compIndex=0;}



SWITCH: {
    param("showrawdata") && do {&printPageHeader("Raw Data"); &printRawData($testIndex,$parameterIndex,$chipIndex); last SWITCH;};
    param("getTest") && do {&printPageHeader("Test Data"); &showTestData(); last SWITCH;};
    param("getList") && do{&printPageHeader("Test List");showTestList(); last SWITCH;};
    param("showDataSetInfo") && do{&printPageHeader("DataSet Information"); showDataSetInfo(); last SWITCH;};

    &printPageHeader("SctRodDaq Data Display");
    &readDaqTable($config{$dataSet}{TYPE});
    %dataMap = &readParamTable($testIndex,$parameterIndex,$chipIndex,$fname,1);
    if($compIndex>0) {%refMap = &readParamTable($testIndex,$parameterIndex,$chipIndex,$reffiles[$compIndex],0);}
    &printBarrelFile($config{$dataSet}{TYPE});
    my $sname = "$scriptname\?locn=$locn\&testIndex=$testIndex\&parameterIndex=$parameterIndex\&dataSet=$dataSet\&chipIndex=$chipIndex\&cellfillindex=$cellfillIndex\&showrawdata=1";
    print "<br><br>&gt;&nbsp;<a href=\"#\" onClick=\"window.open('$sname','RawData','menubar=1,scrollbars=1'); return false;\">View the raw data</a> used to generate this web page.";
    $sname="$scriptname\?locn=$locn\&testIndex=$testIndex\&parameterIndex=$parameterIndex\&dataSet=$dataSet\&chipIndex=$chipIndex\&cellfillindex=$cellfillIndex\&showDataSetInfo=1";
    print "<br>&gt;&nbsp;<a href=\"#\" onClick=\"window.open('$sname','RawData','menubar=1,scrollbars=1'); return false;\">Click here</a> to view information on the current dataset.";
}


print qq!
</body>
</html>
!;

sub printRawData() {

    my ($testIndex,$parameterIndex,$chipIndex) = @_;

    open(IN,$fname) || printError("Cant open $fname: $!");
    flock(IN,1) || printError("Cant get LOCK_SH on $fname: $!");

    print qq!
<html>
<head>
<title>Raw Data</title>
<link rel=stylesheet HREF="http://www-pnp.physics.ox.ac.uk/~daquser/styles/davestyles2.css" type="text/css">
!;
print "<\!-- Raw data from file $fname -->";
print qq!
</head>
<body>
!;
   while(<IN>) {
      print "$_ <br>\n";
      }
    close(IN) || printError("Cant close $fname: $!");
    }

sub printError {
 my ($reason) = @_;

   print qq!
  <html>
<head>
<title>Barrel Module Positions</title>
</head>
<body>

<h3><font color=red>Error</font></h3>
<br>
$reason
<br><br>

</body>
</html>
!;
die;
}

sub readParamTable() {

    my %dMap=();
    my ($testIndex,$parameterIndex,$chipIndex,$fname,$readConfig) = @_;

    my $linecount=0;

    open(IN,$fname) || return %dMap;
    flock(IN,1) || printError("Cant get LOCK_SH on $fname: $!");
#RUN=1371SCAN=0SN=20220040200007NOISE=M0:1183.21;S1:1211.68;S2:1293.97;S3:1283.25;S4:1257.9;E5:1254.65;M8:1220.48;S9:1280.05;S10:1266.41;S11:1214.65;S12:1338.63;E13:1307.26CFACTORS=M0:1.00;S1:1.00;S2:1.00;S3:1.00;S4:1.00;E5:1.00;M8:1.00;S9:1.00;S10:1.00;S11:1.00;S12:1.00;E13:1.00
    while(<IN>) {
       if(/^DEFECT(.*)=(\d+)/) {if($1 !~ /.*SLOPE.*/) {$defectCount{$1}=$2;}}
       if(/^SLOPEDEFECTS=(\d+)/) {$slopedefects=$1;}
       if(/^TOTALDEFECTCOUNT=(\d+)/) {$totaldefectcount=$1;}
       if(/.*RUN=(\d+)SCAN=(\d+)SN=(\d{14})DATA=(.*)DCS=(.*)DEF=(.*)CFACTORS=(.*)CONF=(.*)/) {

            $linecount++;
            my $runno=$1;
	    my $scanno=$2;
	    my $sn=$3;
	    my $dataline=$4;
	    my $dcsline=$5;
	    my $defectline=$6;
	    my $cfactorline=$7;
	    my $confline=$8;

	    if(exists $replacedModules{$sn}) {
	       if($runno<$replacedModules{$sn}{FROMRUN}) {next;}
	       }

	    if($readConfig) {$datalinks{$sn}="&sn=$sn\&test=$testIndexNames[$testIndex]\&runno=$runno\&scanno=$scanno\&testIndex=$testIndex\&parameterIndex=$parameterIndex\&dataSet=$dataSet\&chipIndex=$chipIndex\&cellfillIndex=$cellfillIndex";}

	    my @chipdata = split(/;/,$dataline);
	    my @cfactordata = split(/\*/,$cfactorline);

	    my $chipCount=0;
	    my $chipCount0=0;
	    my $chipCount1=0;
	    my $thisData=0.;
	    my $cData=0.;

	    my $no_testparams = @{$pnames[$testIndex]};
	    my $no_defectparams = @defectnames;
	    my $no_dcsparams = @DCSnames;
	    my $no_confparams = @BOCnames;

	    #default for test index 0 - nptgain
	    my $defectIndex = $no_testparams;
	    my $dcsIndex=$defectIndex + $no_defectparams;
	    my $confIndex = $dcsIndex + $no_dcsparams;

	    if($parameterIndex>=$confIndex) {
	       my @confbits = split(/;/,$confline);
	       $confIndex++;
	       if($confline eq "") {$thisData=$noDataString;}
	       else {
	          my $cbit = $confbits[$parameterIndex-$confIndex];
	          if($cbit=~/[RXTD01]+:(\d+)/) {$thisData=$1;}
		  }
	       $dMap{$sn}=$thisData;
               if(exists $daqInfo{$sn} && $readConfig) {
	       if(exists $replacementModules{$sn} && $runno<=$replacementModules{$sn}) {}
	       else {$moduleCount++;}
		  }
	       next;
	       }

	    if($parameterIndex>=$dcsIndex) {
	       my @thisline= split(/\*/,$dcsline);
	       $dcsIndex++;
	       if($dcsline eq "") {$thisData=$noDataString;}
	       else {$thisData = ($thisline[$parameterIndex-$dcsIndex] eq ".") ? $noDataString : $thisline[$parameterIndex-$dcsIndex];}
	       if($thisData ne $noDataString) {$thisData =sprintf("%.2f",$thisData);}
               $dMap{$sn}=$thisData;
	       if(exists $daqInfo{$sn} && $readConfig) {
	         if(exists $replacementModules{$sn} && $runno<=$replacementModules{$sn}) {}
	         else {$moduleCount++;}
	         }
	       next;
	       }

	    if($parameterIndex>=$defectIndex) {
	       my @thisline= split(/\*/,$defectline);
	       $thisData=$thisline[$parameterIndex-$defectIndex];
               $dMap{$sn}=$thisData;
	       if(exists $daqInfo{$sn} && $readConfig) {
	         if(exists $replacementModules{$sn} && $runno<=$replacementModules{$sn}) {}
	         else {$moduleCount++;}
		  }
	       next;
	       }

            if($chipIndex<=4) {  # mean overall
	        my $total=0.;
		my $minValue=99999.;
		my $maxValue=0.;
		my $mean0=0.;
		my $mean1=0.;
	        $thisData=0.;
		$cData=0.;
	        for(my $i=0;$i<12;$i++) {
		   my ($chipName,$dataStar) = split(/:/,$chipdata[$i]);
		   my @datas = split(/\*/,$dataStar);
		   my $asize = @datas;
		   $asize--;
		   if($parameterIndex<=$asize) {
		     my $data = $datas[$parameterIndex];
		     if($data eq "nan") {next;}
		     if($data eq "0" && $testIndexNames[$testIndex] !~ /.*FullBypass|TrimRange.*/) {next;}

		     $total += $data;
		     if($i<6) {$mean0 += $data; $chipCount0++;}
		     if($i>=6) {$mean1 += $data; $chipCount1++;}
		     if($data<$minValue) {$minValue=$data;}
		     if($data>$maxValue) {$maxValue=$data;}

		     my $cdata = $cfactordata[$i];
		     $cData += $cdata;
		     }
		   else {$thisData=$noDataString;}
		   $chipCount++;
		   }

		if($chipIndex==0) {
		   $thisData = ($chipCount>0) ? $total/$chipCount : $noDataString;
		   }
		elsif($chipIndex==1) {$thisData = $maxValue;}
		elsif($chipIndex==2) {$thisData = $minValue;}
		elsif($chipIndex==3) {
		    $thisData = ($chipCount0>0) ? $mean0/$chipCount0 : $noDataString;
		    }
		elsif($chipIndex==4) {
		    $thisData = ($chipCount1>0) ? $mean1/$chipCount1 : $noDataString;
		    }

		if($chipCount>0) {$cData /= $chipCount;}
		}
             else {
	        my ($chipName,$dataStar) = split(/:/,$chipdata[$chipIndex-5]);
		my $ci = $chipIndex-5;
		my @datas = split(/\*/,$dataStar);
		my $asize = @datas;
		$asize--;
		if($parameterIndex<=$asize) {
		  $thisData = $datas[$parameterIndex];
		  my $cdata = $cfactordata[$chipIndex-5];
		  $cData=$cdata;
		  }
		else {$thisData=$noDataString;}
		}
	  if($thisData eq "nan" || ($thisData eq "0" && $testIndexNames[$testIndex] !~ /.*FullBypass|TrimRange.*/)) {$thisData=$noDataString;}

          if($thisData ne $noDataString) {$thisData = ($isOccupancyData==1) ? sprintf("%.2E",$thisData) : sprintf("%.2f",$thisData);}

          $dMap{$sn}=$thisData;
	  if(exists $daqInfo{$sn} && $readConfig) {
	       if(exists $replacementModules{$sn} && $runno<=$replacementModules{$sn}) {}
	       else {$moduleCount++;}
	    }
	  $cData = sprintf("%.3f",$cData);
	  if($readConfig) {$cMap{$sn}=$cData;}

	  }
        }
     close(IN) || printError("Cant close datafile: $!");
     return %dMap;
   }




sub readDaqTable() {

      my ($this_index) = @_;
      my $lookup = "$lookupDir$config{$dataSet}{MOUNTS}";


      if(!(-e "$lookup")) {return;}

      open(IN,$lookup) || printError("Cant open $lookup: $!");
      flock(IN,1) || printError("Cant get LOCK_SH on $lookup: $!");
      
      my @lines = <IN>;
      close(IN) || printError("Cant close $lookup: $!");
      my $sctType = $config{$dataSet}{TYPE};
      if(exists $addedModules{$sctType}) {
         for my $module (keys %{ $addedModules{$sctType} } ) {
#	     print "Adding extra module $module=$addedModules{$sctType}{$module}\n";
	     push(@lines,"$module=$addedModules{$sctType}{$module}");
	     }
         }

 #     push(@lines,"20220380200139=Barrel 3, Row 27, Position 6");

      foreach my $line (@lines) {
        if($line=~/(\d{14})=Barrel\s*(\d+),\s*Row\s*(\d+),\s*Position\s*([-\d]+)/) {
            $daqInfo{$1}="Barrel $2, Row $3, Position $4";
            $barrelCount[$2-3]++;
            my $position = $4;
            $position+=6;
            if($position>=7) {$position--;}
            my $newpos = ($position + ($3 * 12))<<4;
            my $index = $2 - 3 + $newpos;
            $daqMapping{$index} = $1;

            $daqInfoExists++;
            }
          }
	  
      foreach my $rmodule (keys %replacedModules) {
	  my $type=$config{$dataSet}{TYPE};
          if((exists $replacedModules{$rmodule}{TYPE}) && $replacedModules{$rmodule}{TYPE}==$type) {
	     $daqInfo{$replacedModules{$rmodule}{LAST}} = $daqInfo{$rmodule};
#	     print "$rmodule now in same location as $replacedModules{$rmodule}{LAST} from Run $replacedModules{$rmodule}{FROMRUN}\n";
	     $replacementModules{$replacedModules{$rmodule}{LAST}} = $replacedModules{$rmodule}{FROMRUN}; # map replaced module to replacement date
	     }
          }

#      close(IN) || printError("Cant close $lookup: $!");
      }


sub printBarrelFile {

my $barrel=$_[0];


my $rows = $barrelRows[$barrel];

#my $thisTitle = "Latest $testNames[$testIndex] $parameterNames[$parameterIndex] Data for Barrel $barrel";

print qq!

<h3>$instituteName SctRodDaq Data Display for $config{$dataSet}{TITLE}</h3>

<form method="POST">

!;

print qq!
<table border=0 cellspacing=5 cellpadding=0>
<tr><td align=right valign=top>Test:</td>
<td colspan=5 align=left valign=top>
!;
my $noTests = @testIndexNames;
for (my $i=0;$i<$noTests;$i++) {
  if($i>0 && $i!=9) {print "\&nbsp;-\&nbsp;";}
  my $link = ($testIndex==$i) ? $testIndexNames[$i] : "<a href=$scriptname\?locn=$locn\&testIndex=$i\&parameterIndex=$parameterIndex\&dataSet=$dataSet\&chipIndex=$chipIndex\&cellfillindex=$cellfillIndex\&compIndex=$compIndex>$testIndexNames[$i]</a>";
  print "$link";
  if($i==8) {print "<br>";}
  }
print qq!
</td></tr>
<tr><td align=right>DataSet:</td><td align=left><SELECT NAME="dataSet">
!;
my $ndsets = (keys %config);
for (my $i=$ndsets;$i>0;$i--) {
    my $selectflag = ($dataSet == $i) ? "SELECTED" : "";
    my $title = ($i==0) ? "$config{$i}{TITLE}" : $config{$i}{TITLE};
    print "<option value=$i $selectflag>$title</option>\n";
    }
print qq!
</SELECT></td>
<td align=right nowrap>Parameter:</td><td align=left><SELECT NAME="parameterIndex">
!;
  my $count=0;
  foreach (@paramlist) {
    my $selectflag = ($parameterIndex == $count) ? "SELECTED" : "";
    print "<option value=$count $selectflag>$_</option>\n";
    $count++;
    }
print qq!
</SELECT></td>

<td align=right nowrap>Chip (if app):</td><td align=left><SELECT NAME="chipIndex">
!;
  $count=0;
  foreach (@chipNames) {
    my $selectflag = ($chipIndex == $count) ? "SELECTED" : "";
    print "<option value=$count $selectflag>$_</option>\n";
    $count++;
    }

print qq!
  </SELECT></td></tr>
  <tr><td align=right nowrap>Cell Label:</td><td align=left><SELECT NAME="cellfillIndex">
!;
    $count=0;
    foreach (@cellfills) {
      my $selectflag = ($cellfillIndex==$count) ? "SELECTED" : "";
      print "<option value=$count $selectflag>$_</option>\n";
      $count++;
    }
print qq!
</SELECT></td>
  <td align=right nowrap>Compare Data with:</td><td align=left><SELECT NAME="compIndex">
!;
$count=0;

    foreach (@refnames) {
      my $selectflag = ($compIndex==$count) ? "SELECTED" : "";
      print "<option value=$count $selectflag>$_</option>\n";
      $count++;
    }
print qq!
</SELECT>
</td>
<td colspan=2 align=right nowrap><INPUT TYPE="SUBMIT" NAME="viewplots" VALUE="Refresh Display"></td></tr>


</table>
<input type="HIDDEN" name="parameterIndex" VALUE="$parameterIndex">
<input type="HIDDEN" name="testIndex" VALUE="$testIndex">
<input type="HIDDEN" name="locn" VALUE="$locn">
</form>
<br>
!;
my $ctext = ($compIndex>0) ? " SUBTRACTED BY <b>$refnames[$compIndex]</b>" : "";
print "This page is currently displaying <b>$paramlist[$parameterIndex] ($chipNames[$chipIndex])</b> data for <b>$moduleCount</b> modules from the most recent <b>$testIndexNames[$testIndex]</b> tests during <b>$config{$dataSet}{TITLE}</b>$ctext <br><br>";


&printColorScale();

print qq!


<table border=0 cellspacing=4 cellpadding=0>


<tr>
 <td nowrap width="50" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="10" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
</tr>

<tr><td nowrap></td><td nowrap colspan=13 align=center><b>Module Position</b></td></tr>

<tr>
<td nowrap></td>
<td nowrap align=center><b>-6</b></td>
<td nowrap align=center><b>-5</b></td>
<td nowrap align=center><b>-4</b></td>
<td nowrap align=center><b>-3</b></td>
<td nowrap align=center><b>-2</b></td>
<td nowrap align=center><b>-1</b></td>
<td nowrap></td>
<td nowrap align=center><b>1</b></td>
<td nowrap align=center><b>2</b></td>
<td nowrap align=center><b>3</b></td>
<td nowrap align=center><b>4</b></td>
<td nowrap align=center><b>5</b></td>
<td nowrap align=center><b>6</b></td>
</tr>
!;

for (my $row=0;$row<$rows; $row++) {
 my $lmt = $row + 1;
 print "<tr><td nowrap align=right><b>LMT $lmt: </b></td>\n";
 for (my $pos=0;$pos<12;$pos++) {
   my $posindex = ($pos + ($row * 12))<<4;
   my $index = $barrel + $posindex;
   my $content = "";
   my $cellColor = "#eeeeff";
   if(exists $daqMapping{$index}) {
     my $sn = $daqMapping{$index};

     # Check for replaced modules.
     # We might need $index to point to the older (replaced) module.
     # First check if this is module has replaced another
     if(exists $replacedModules{$sn}{LAST}) {
# if the datalinks exist, it means there is data from the new module
       if((exists $datalinks{$sn}) && $datalinks{$sn}=~/.*runno=(\d+).*/) {
# if the run number is >= FROMRUN, the use the new module, otherwise use the old module
         if($1 < $replacedModules{$sn}{FROMRUN}) {$sn = $replacedModules{$sn}{LAST};}
	 }
	 # if no data exists for the new module, then assume this inde points to the replaced module
       else {$sn = $replacedModules{$sn}{LAST};}
       }

     if($compIndex>0) {
        if(exists $dataMap{$sn} && $dataMap{$sn} ne $noDataString && exists $refMap{$sn} && $refMap{$sn} ne $noDataString) {
	# make some corrections to the ref data, for enc noise and gain
#	      print "$paramlist[$parameterIndex] cdata=$cMap{$sn} <br>\n";
              if($paramlist[$parameterIndex]=~/.*Gain \(mV\/fC\).*/) {
	            if($cMap{$sn}=~/(\d+\.\d+)/ && $compIndex<=2) {
		       my $cfactor = $1;
		       my $dd = $refMap{$sn};
		       my $x = $dd;
		       if($cfactor>0) { $x=$dd / $cfactor;}
	               $refMap{$sn} =  $x;
		       }
		  }
              if($paramlist[$parameterIndex]=~/.*InputNoise\(enc\).*/) {
	          if($cMap{$sn}=~/(\d+\.\d+)/  && $compIndex<=2) {
		    my $x = $refMap{$sn} * $1;
	            $refMap{$sn} = $x;
		    }
		  }
          $dataMap{$sn} -= $refMap{$sn};
	  $dataMap{$sn} = ($isOccupancyData==1) ? sprintf("%.2E",$dataMap{$sn}) : sprintf("%.2f",$dataMap{$sn});
          if($cellfillIndex==0) {$content="<a href=${scriptname}\?locn=$locn\&getTest=1\&$datalinks{$sn} class=\"white\">$dataMap{$sn}</a>";}
	  elsif($cellfillIndex==1) {$content="<a href=${scriptname}\?locn=$locn\&getList=1\&$datalinks{$sn} class=\"white\">$sn</a>";}
	  elsif($cellfillIndex==2) {$content = (exists $cMap{$sn}) ? $cMap{$sn} : "Unavailable";}
	  else {$content="";}
          $cellColor = &getCellColor($dataMap{$sn});
	  }
	else {$content="No Data";}
	}
     elsif(exists $dataMap{$sn}) {
        if($cellfillIndex==0) {$content="<a href=${scriptname}\?locn=$locn\&getTest=1\&$datalinks{$sn} class=\"white\">$dataMap{$sn}</a>";}
	elsif($cellfillIndex==1) {$content="<a href=${scriptname}\?locn=$locn\&getList=1\&$datalinks{$sn} class=\"white\">$sn</a>";}
	elsif($cellfillIndex==2) {$content = (exists $cMap{$sn}) ? $cMap{$sn} : "Unavailable";}
	else {$content="";}
        $cellColor = &getCellColor($dataMap{$sn});
	}
      else {$content="No Data";}
     }
 #  else {print "No daqMapping for $index \n";}

   print "<td nowrap bgcolor=\"$cellColor\" align=center>$content</td>";
   if($pos==5) {print "<td></td>";}
   }
 print "</tr>\n";
 }
 print "</table>\n";
 
 print qq!
 <br><br><b>Defective Channel Counts</b><br>
 !;
 if($totaldefectcount==-1) {
   print "<br>No defects present...<br>";
   return;
   }
 print qq!
 <table border=1 cellspacing=0 cellpadding=0>
 <col width=150>
 <col width=100>
 <tr><td><b>Defect Name</b></td><td><b>#Channels</b></td></tr>
 !;
 foreach my $defect (keys %defectCount) {
   print "<tr><td align=left nowrap>$defect</td><td align=left>$defectCount{$defect}</td></tr>\n";
   }
 print qq!
 <tr><td align=left nowrap><b>Total:</b></td><td align=left nowrap><b>$totaldefectcount</b></td></tr>
 </table>
 !;
 if($slopedefects>0) {print "NOTE: $slopedefects slope-related defects have beene excluded from the display.<br>\n";}




}

sub getCellColor() {


  my ($data) = @_;

  if($data eq $noDataString) {return "#f1f1f1";}

  my $color = "000000";
  my $bluebit=0;
  my $redbit=0;
  my $greenbit=0;
  
  my $paramKey = $paramlist[$parameterIndex];
  if($compIndex>0) {$paramKey .= " Comparison";}
  if(!(exists $colorHashC0{$paramKey})) {printError("No color scale");}

  if($data < $colorHashC0{$paramKey}[0]) {
    $color = sprintf("%02x%02x%02x",$colorHashC0{$paramKey}[1],$colorHashC0{$paramKey}[2],$colorHashC0{$paramKey}[3]);
    return $color;
    }
    
  if($data > $colorHashC3{$paramKey}[0]) {
    $color = sprintf("%02x%02x%02x",$colorHashC3{$paramKey}[1],$colorHashC3{$paramKey}[2],$colorHashC3{$paramKey}[3]);
    return $color;
    }


  my $value0 = $colorHashC0{$paramKey}[0];
  my $value1 = $colorHashC1{$paramKey}[0];
  my $value2 = $colorHashC2{$paramKey}[0];
  my $value3 = $colorHashC3{$paramKey}[0];

  if($data<$value1) {
    my $diff = $data - $value0;
    my $max = $value1 - $value0;
    my $frac = $diff/$max;

    my $redbit0 = $colorHashC0{$paramKey}[1];
    my $greenbit0 = $colorHashC0{$paramKey}[2];
    my $bluebit0 = $colorHashC0{$paramKey}[3];
    
    my $redbit1 = $colorHashC2{$paramKey}[1];
    my $greenbit1 = $colorHashC2{$paramKey}[2];
    my $bluebit1 = $colorHashC2{$paramKey}[3];


    if($redbit0 == $redbit1) {$redbit=$redbit0;}
    elsif($redbit1 > $redbit0) {$redbit = $redbit0 + ($frac * ($redbit1-$redbit0));}
    else {$redbit = $redbit0 - ($frac * ($redbit0-$redbit1));}

    if($greenbit0 == $greenbit1) {$greenbit=$greenbit0;}
    elsif($greenbit1 > $greenbit0) {$greenbit = $greenbit0 + ($frac * ($greenbit1-$greenbit0));}
    else {$greenbit = $greenbit0 - ($frac * ($greenbit0-$greenbit1));}

    if($bluebit0 == $bluebit1) {$bluebit=$bluebit0;}
    elsif($bluebit1 > $bluebit0) {$bluebit = $bluebit0 + ($frac * ($bluebit1-$bluebit0));}
    else {$bluebit = $bluebit0 - ($frac * ($bluebit0-$bluebit1));}

    $color = sprintf("%02x%02x%02x",$redbit,$greenbit,$bluebit);

    return "#$color";

    }
    
  if($data>$value2) {
    my $diff = $data - $value2;
    my $max = $value3 - $value2;
    my $frac = $diff/$max;

    my $redbit0 = $colorHashC2{$paramKey}[1];
    my $greenbit0 = $colorHashC2{$paramKey}[2];
    my $bluebit0 = $colorHashC2{$paramKey}[3];
    
    my $redbit1 = $colorHashC3{$paramKey}[1];
    my $greenbit1 = $colorHashC3{$paramKey}[2];
    my $bluebit1 = $colorHashC3{$paramKey}[3];


    if($redbit0 == $redbit1) {$redbit=$redbit0;}
    elsif($redbit1 > $redbit0) {$redbit = $redbit0 + ($frac * ($redbit1-$redbit0));}
    else {$redbit = $redbit0 - ($frac * ($redbit0-$redbit1));}

    if($greenbit0 == $greenbit1) {$greenbit=$greenbit0;}
    elsif($greenbit1 > $greenbit0) {$greenbit = $greenbit0 + ($frac * ($greenbit1-$greenbit0));}
    else {$greenbit = $greenbit0 - ($frac * ($greenbit0-$greenbit1));}

    if($bluebit0 == $bluebit1) {$bluebit=$bluebit0;}
    elsif($bluebit1 > $bluebit0) {$bluebit = $bluebit0 + ($frac * ($bluebit1-$bluebit0));}
    else {$bluebit = $bluebit0 - ($frac * ($bluebit0-$bluebit1));}

    $color = sprintf("%02x%02x%02x",$redbit,$greenbit,$bluebit);

    return "#$color";

    }
    
  if($data>=$value1 && $data<=$value2) {
    my $diff = $data - $value1;
    my $max = $value2 - $value1;
    my $frac = $diff/$max;

    my $redbit0 = $colorHashC1{$paramKey}[1];
    my $greenbit0 = $colorHashC1{$paramKey}[2];
    my $bluebit0 = $colorHashC1{$paramKey}[3];
    
    my $redbit1 = $colorHashC2{$paramKey}[1];
    my $greenbit1 = $colorHashC2{$paramKey}[2];
    my $bluebit1 = $colorHashC2{$paramKey}[3];


    if($redbit0 == $redbit1) {$redbit=$redbit0;}
    elsif($redbit1 > $redbit0) {$redbit = $redbit0 + ($frac * ($redbit1-$redbit0));}
    else {$redbit = $redbit0 - ($frac * ($redbit0-$redbit1));}

    if($greenbit0 == $greenbit1) {$greenbit=$greenbit0;}
    elsif($greenbit1 > $greenbit0) {$greenbit = $greenbit0 + ($frac * ($greenbit1-$greenbit0));}
    else {$greenbit = $greenbit0 - ($frac * ($greenbit0-$greenbit1));}

    if($bluebit0 == $bluebit1) {$bluebit=$bluebit0;}
    elsif($bluebit1 > $bluebit0) {$bluebit = $bluebit0 + ($frac * ($bluebit1-$bluebit0));}
    else {$bluebit = $bluebit0 - ($frac * ($bluebit0-$bluebit1));}

    $color = sprintf("%02x%02x%02x",$redbit,$greenbit,$bluebit);

    return "#$color";

    }


  return "#ffffff";
  }

sub printColorScale {

my $paramKey = $paramlist[$parameterIndex];
if($compIndex>0) {$paramKey .= " Comparison";}
my $C0color = sprintf("#%02x%02x%02x",$colorHashC0{$paramKey}[1],$colorHashC0{$paramKey}[2],$colorHashC0{$paramKey}[3]);
my $C1color = sprintf("#%02x%02x%02x",$colorHashC1{$paramKey}[1],$colorHashC1{$paramKey}[2],$colorHashC1{$paramKey}[3]);
my $C2color = sprintf("#%02x%02x%02x",$colorHashC2{$paramKey}[1],$colorHashC2{$paramKey}[2],$colorHashC2{$paramKey}[3]);
my $C3color = sprintf("#%02x%02x%02x",$colorHashC3{$paramKey}[1],$colorHashC3{$paramKey}[2],$colorHashC3{$paramKey}[3]);
my $label0 = ($isOccupancyData) ? sprintf("%.1E",$colorHashC0{$paramKey}[0]) : $colorHashC0{$paramKey}[0];
my $label1 = ($isOccupancyData) ? sprintf("%.1E",$colorHashC1{$paramKey}[0]) : $colorHashC1{$paramKey}[0];
my $label2 = ($isOccupancyData) ? sprintf("%.1E",$colorHashC2{$paramKey}[0]) : $colorHashC2{$paramKey}[0];
my $label3 = ($isOccupancyData) ? sprintf("%.1E",$colorHashC3{$paramKey}[0]) : $colorHashC3{$paramKey}[0];

print qq!
<table border=0 cellspacing=0 cellpadding=0>
<tr><td align=center valign=bottom>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;${label0}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
<td align=center valign=bottom>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;${label1}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
<td align=center valign=bottom>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;${label2}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
<td align=center valign=bottom>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;${label3}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td></tr>
<tr><td align=center valign=bottom>|</td>
<td align=center valign=bottom>|</td>
<td align=center valign=bottom>|</td>
<td align=center valign=bottom>|</td></tr>
<tr>


<td align=center bgcolor="$C0color">&nbsp;</td>
<td align=center bgcolor="$C1color">&nbsp;</td>
<td align=center bgcolor="$C2color">&nbsp;</td>
<td align=center bgcolor="$C3color">&nbsp;</td></tr>
</table>
!;
}

sub printPageHeader() {
my ($title) = @_;
print qq!
<html>
<head>
<title>$title</title>
<link rel=stylesheet HREF="http://www-pnp.physics.ox.ac.uk/~daquser/styles/davestyles2.css" type="text/css">
!;
print "<\!-- \@author Dave Robinson 21/10/2004 -->\n";
print qq!
</head>
<body>
!;
}

sub showTestData() {
 my $runno = param("runno") ? param("runno") : "";
 my $scanno = param("scanno") ? param("scanno") : "0";
 my $testNameHeader = param("test") ? param("test") : "";
 my $sn = param("sn") ? param("sn") : "";
 my $dataSet = param("dataSet") ? param("dataSet") : "0";

 if($runno eq "" || $testNameHeader eq "" || $sn eq "") {printError("invalid arguments to script");}

 my $linkback = "\&gt;&nbsp;&nbsp;<a href=$scriptname\?locn=$locn\&testIndex=$testIndex\&parameterIndex=$parameterIndex\&dataSet=$dataSet\&chipIndex=$chipIndex\&cellfillindex=$cellfillIndex>Click here</a> to return to the main display.<br><br>";
 $linkback .= "\&gt;&nbsp;&nbsp;<a href=$scriptname\?locn=$locn\&getList=1\&sn=$sn\&testIndex=$testIndex\&parameterIndex=$parameterIndex\&dataSet=$dataSet\&chipIndex=$chipIndex\&cellfillindex=$cellfillIndex>Click here</a> to list ALL tests for this module.<br><br>";

 parseSummaryData("$summaryRootDirectory$config{$dataSet}{DATADIR}/summaries/",$runno,$scanno,"${testNameHeader}TestResult",$sn,$linkback);
 }

sub showTestList() {
 my $sn = param("sn") ? param("sn") : "";

 if($sn eq "") {printError("invalid arguments to script");}
 
 my $linkback = "\&gt;&nbsp;&nbsp;<a href=$scriptname\?locn=$locn\&testIndex=$testIndex\&parameterIndex=$parameterIndex\&dataSet=$dataSet\&chipIndex=$chipIndex\&cellfillindex=$cellfillIndex>Click here</a> to return to the main display.<br><br>";

 printModuleTestList("$rootDirectory$config{$dataSet}{DATADIR}/summaries/",$dataSet,$sn,$scriptname,$linkback);

  }
  
sub showDataSetInfo() {
my $rmsg = ($config{$dataSet}{RUNEX} eq "") ? "No runs have been excluded when compiling this dataset." : "Run(s) $config{$dataSet}{RUNEX} were excluded when compiling this dataset.";
my $runmsg = ($config{$dataSet}{RUNHI} ==99999) ? "This dataset comprises the most recent data taken since Run <b>$config{$dataSet}{RUNLO}</b>" : "This dataset comprises the most recent data from Run <b>$config{$dataSet}{RUNLO}</b> to Run <b>$config{$dataSet}{RUNHI}";
 print qq!
 <h3>$testIndexNames[$testIndex] Dataset Information for $config{$dataSet}{TITLE}</h3>
 $runmsg<br><br>
 $rmsg
 <br>
 <br>
 <table border=1>
 <col width=150>
 <col width=100>
 <col width=100>
 <col width=250>
 <tr><td>Serial Number</td><td>Run</td><td>Scan</td><td>Comments</td></tr>
 !;

 open(IN,$fname) || printError("Cant open $fname: $!");
 flock(IN,1) || printError("Cant get LOCK_SH on $fname: $!");
#RUN=1371SCAN=0SN=20220040200007NOISE=M0:1183.21;S1:1211.68;S2:1293.97;S3:1283.25;S4:1257.9;E5:1254.65;M8:1220.48;S9:1280.05;S10:1266.41;S11:1214.65;S12:1338.63;E13:1307.26CFACTORS=M0:1.00;S1:1.00;S2:1.00;S3:1.00;S4:1.00;E5:1.00;M8:1.00;S9:1.00;S10:1.00;S11:1.00;S12:1.00;E13:1.00
    while(<IN>) {
       if(/^DEFECT(.*)=(\d+)/) {$defectCount{$1}=$2;}
       if(/^TOTALDEFECTCOUNT=(\d+)/) {$totaldefectcount=$1;}
       if(/.*RUN=(\d+)SCAN=(\d+)SN=(\d{14})DATA=(.*)DCS=(.*)DEF=(.*)CFACTORS=(.*)CONF=(.*)/) {
            my $runno=$1;
	    my $scanno=$2;
	    my $sn=$3;
	    my $comment = "";
	    if($5 eq ".*.*.*.*.*.*.*.*.") {$comment = "No DCS info. ";}
	    if($7=~/.*1.000?;.*/) {$comment .= "Some or all data taken with cfactor=1.00 ";}
	    if($comment eq "") {$comment = "None";}
	    print "<tr><td>$sn</td><td>$runno</td><td>$scanno</td><td>$comment</td></tr>\n";
	    }
       }
    close(IN) || printError("Cant close $fname: $1");
    print qq!
    </table>
    <br>
    &gt;&nbsp;&nbsp;<a href=$scriptname\?locn=$locn\&testIndex=$testIndex\&parameterIndex=$parameterIndex\&dataSet=$dataSet\&chipIndex=$chipIndex\&cellfillindex=$cellfillIndex>Click here</a> to return to the main display.<br><br>
    !;
}


 


